% ----------------------------------------------------------------------------------- %
% % calcmodel function                                                              % %
% % calculate consumption, savings, utility and job choices for the household       % %
% ----------------------------------------------------------------------------------- %

function calc = calcmodel(tcat, gv, data, calc, arglist)

% define some parameters to simplify codes;
rho = arglist.rvec(calc.obs, 1); % rho is time discount factor; 
tp = 1 + rho; % 1 / tp is time discount rate;
sqrt2 = sqrt(2); % square root of 2; 
consadj = 2 ^ ( 0.5 * (1 - calc.alpha)); % marginal utility adjustment factor for couples; the utility functional form for couples are: sqrt(2) * (C(t)/sqrt(2)) ^ alpha / alpha;
agew = gv.T + data.birthyear - data.spbirthyear; % wife's age when the husband is at the terminal age;

% initialization;
% clarification of dimensions;
% age (45, 20 or 30): 45 indicates from 25 to 69, 20 indicates from 50 to 69, 30 indicates from 70 to 99;
% survival status (1 - 3): 1 is both spouses alive, 2 is only husband alive, 3 is only wife alive;
% job category besides career job (1 - 3): 1 is returned full time work, 2 is part time work, 3 is retired;
% delayed DB pension (1 - 2): 1 is no, receiving DB pension regularly; 2 is yes, delayed receiving DB pension; 
% DB Pension benefits (1 - calc.npcatlim): DB pension benefits grid; the benefit amounts are discounted back to age 50;
% DC pension balance (1 - calc.nlcatlim): DC pension balance grid;
% SS benefits (1 - gv.nscatlim): SS benefits by retirement age (50, EEA, NRA, 70);
% leisure preference (1 - gv.necatlim): epsilon grid;
% asset (1 - gv.nacatlim): asset grid;
marguc = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.nacatlim); % marginal utility in the current period, older than lastage;
totuc = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.nacatlim); % total utility in the current period, older than lastage;
marguf = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.nacatlim); % marginal utility next period , older than lastage;
totuf = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.nacatlim); % total utility next period, older than lastage;
margucw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % marginal utility in the current period, currently working in career job;
totucw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % total utility in the current period, currently working in career job;
margufw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % marginal utility next period, currently working in career job;
totufw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % total utility next period, currently working in career job;
margucr = zeros(3, gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % marginal utility in the current period, currently retired from career job;
totucr = zeros(3, gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % total utility in the current period, currently retired from career job;
margufr = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % marginal utility next period, currently retired from career job;
totufr = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % total utility next period, currently retired from career job;
cmatw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % consumption equivalents of future expected marginal utility, currently working in career job;
tumatw = zeros(gv.nvcatlim, calc.nlcatlim, gv.necatlim, gv.nacatlim); % total utility next period discounted back to current period, currently working in career job;
cmatr = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % consumption equivalents of future expected marginal utility, currently retired from career job;
tumatr = zeros(gv.nvcatlim, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % total utility next period discounted back to current period, currently retired from career job;
% optimal total utility and saving matrices;
totuw = zeros(45, calc.nlcatlim, gv.necatlim, gv.nacatlim); % expected total utility from current period to the terminal age given optimal behaviors in later periods, currently working in career job;
totud = zeros(20, 3, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % expected total utility from current period to the terminal age given optimal behaviors in later periods, currently retired from career job; the 2nd dimension is job category;
savw = zeros(45, calc.nlcatlim, gv.necatlim, gv.nacatlim); % optimal savings, currently working in career job;
savd = zeros(20, 3, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % optimal savings, currently retired from career job; the 2nd dimension is job category;
savr = zeros(30, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.nacatlim); % optimal savings, surely retired;
% retirement decisions matrices;
mjchoice = zeros(20, calc.nlcatlim, gv.necatlim, gv.nacatlim); % job (retirement) decision, currently working in career job; 1 is staying in career job, 2 is returned full time work, 3 is part time work, 4 is retired;
retchoice = zeros(20, gv.ndcatlim, calc.npcatlim, gv.nscatlim, gv.necatlim, gv.nacatlim); % job (retirement) decision, currently retired from career job; 2 is returned full time work, 3 is part time work, 4 is retired;
% asset grid point transition matrices;
lacats = zeros(calc.nlcatlim, gv.nacatlim); % transition matrix of asset grid points after adding DC pension wealth, when quiting career job at lastage + 1;
lacatsw = zeros(calc.nlcatlim, gv.nacatlim); % transition matrix of asset grid points after adding DC pension wealth, previously working in career job;
lacatsr = zeros(calc.nlcatlim, gv.nacatlim); % transition matrix of asset grid points after adding DC pension wealth and early out window amount, previously working in career job;

% backward induction;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate consumption, saving and utility at the terminal age %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% individuals are surely retired at the terminal age;
% after retirement, income only comes from DB pension, Social Security benefits and other income;
% the optimal decision is to consume all resources at the terminal age;
% for each asset grid point, calculate marginal utility corresponding to the optimal consumption and save it for future calculations;
% do calculations given the following economic state: vcat, dcat, pcat, scat, acat;
% DB pension benefits are not 0 iff husband is alive and eligible for DB pension;
% actuarial adjustment of DB pension benefits are required because pcats have been discounted back to age 50;
% calculate SS benefits from ssbenr (SS benefits after retirement) because he's surely retired at the terminal age; 
% savr (saving after retirement) is defined as income - consumption (could be negative); 
% if saving is negative, it means households decumulate; that is, consume more than their income;
for vcat = 1 : 3; % survival status;
    for pcat = 1 : calc.npcatlim; % DB pension benefits;
        if pcat == 1 || vcat < 3; 
            
            % calculate DB pension benefits; 
            if vcat < 3 && calc.npcatlim > 1; % eligible for DB pension benefits;
                pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (gv.T - 50));
            else % not eligible;
                pben = 0;
            end;
            
            for scat = 1 : gv.nscatlim; % SS benefits;
                
                % calculate SS benefits; 
                sben = calc.ssbenr(gv.T - 50 + 1, vcat, scat);
                
                for acat = 1 : gv.nacatlim; % asset;
                    
                    % total resources;
                    y = calc.acats(acat, 1) + pben + sben + calc.othincome(gv.T - 25 + 1, vcat);
                    
                    % setup boundary;
                    if y < 10; 
                        y = 10; 
                    end;
                    
                    % it is optimal to consume everything;
                    mu = y ^ (calc.alpha - 1); % marginal utility in the current period;
                    tu = y * mu / calc.alpha; % total utility in the current period;
                    if vcat == 1; % married couple adjustment;
                        mu = consadj * mu;
                        tu = consadj * tu;
                    end;
                    
                    % save to marginal and total utility next period (when calculating backward at age - 1) for future calculations;
                    % the second dimension (dcat) is 1; only calculate this to save computational power;
                    marguf(vcat, 1, pcat, scat, acat) = mu;
                    totuf(vcat, 1, pcat, scat, acat) = tu;
                    
                    % calculate optimal savings;
                    if vcat == 1; % both spouses alive;
                        savr(gv.T - 70 + 1, 1, pcat, scat, acat) = - calc.acats(acat, 1); % optimal savings, surely retired;
                    end;
                    
                end;
                
            end;
            
        end;
    end;
end; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate consumption, saving and utility for surely retired (ages from lastage + 1 to gv.T - 1) %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% individuals are surely retired in this period;
% after retirement, income only comes from DB pension, Social Security benefits and other income;
% do calculations given the following economic state: vcat, dcat, pcat, scat, acat;
% when age <= 62, it's impossible vcat > 1; avoid calculations to save computational power;
% DB pension benefits are not 0 iff husband is alive and eligible for DB pension;
% when dcat == 2 && vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage, individuals do not receive DB pension benefits;
% actuarial adjustment of DB pension benefits are required because pcats have been discounted back to age 50;
% calculate SS benefits from ssbenr (SS benefits after retirement) because he's surely retired at the terminal age; 
% most savings are saved in savr (saving after retirement); 
% some savings are saved in savd (saving in decision periods), if lastage < 69;  
% if saving is negative, it means households decumulate; that is, consume more than their income;
% the 2nd dimension of savd is job category, 1 if secjob, 2 if part time work, 3 if retired; 
% the 2nd last dimension is ecat (leisure preference); the values are the same for all ecat categories after retirement;
% (end of period asset + saving) * asset return shock is asset next period;

for age = 98 : -1 : calc.lastage + 1;
    
    % calculate expectated marginal and total utility with respect to asset returns;
    for vcat = 1 : 3; % survival status;
        if vcat == 1 || age > 62; 
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage); 
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        if pcat == 1 || vcat < 3;
                            for scat = 1 : gv.nscatlim; % SS benefits;
                                
                                % initialization;
                                % intermediate variables;
                                margu = zeros(gv.nacatlim, 1); % expected marginal utility next period by asset, given current economic state;
                                totu = zeros(gv.nacatlim, 1); % expected total utility next period by asset, given current economic state;
                                
                                for acat = 1 : gv.nacatlim; % asset;
                                    
                                    % initialization;
                                    mur = 0; % expected marginal utility next period given current economic state;
                                    tur = 0; % expected total utility next period given current economic state;
                                    
                                    for rcat = 1 : gv.nrcatlim; % asset return shocks;

                                        % find out the corresponding next period asset grid point after asset return shock;
                                        % interpolation;
                                        acati = floor(calc.rtransa(acat, rcat)); % integer part of asset grid point after asset return shock; 
                                        if acati >= gv.nacatlim;
                                            acati = gv.nacatlim - 1;
                                        end;
                                        fa = calc.rtransa(acat, rcat) - acati; % fraction part of asset grid point after asset return shock;

                                        mu0 = marguf(vcat, dcat, pcat, scat, acati);
                                        mu1 = marguf(vcat, dcat, pcat, scat, acati + 1);
                                        tu0 = totuf(vcat, dcat, pcat, scat, acati);
                                        tu1 = totuf(vcat, dcat, pcat, scat, acati + 1);

                                        if mu0 <= 0 || mu1 <= 0;
                                            fprintf('Interpolation Error: Bad mu in post-retirement section: %2d %2d %14.6e %14.6e %2d %2d %2d %2d %2d %2d',...
                                                calc.obs, age, mu0, mu1, vcat, dcat, pcat, scat, acat, acati);
                                            return;
                                        end;

                                        if fa <= 1;
                                            mu = (1 - fa) * mu0 + fa * mu1; % marginal utility next period given current economic state and asset return shock;
                                            tu = (1 - fa) * tu0 + fa * tu1; % total utility next period given current economic state and asset return shock;
                                        else
                                            if mu0 > mu1 && tu1 > tu0;
                                                dmu = mu0 - mu1;
                                                dtu = tu1 - tu0;
                                                muratio = mu0 / mu1;
                                                mufact = dmu / mu0;
                                                mu = mu1 / (1 + (fa - 1) * mufact);
                                                tu = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                                            else
                                                mu = mu1;
                                                if tu1 > tu0;
                                                    tu = (1 - fa) * tu0 + fa * tu1;
                                                else
                                                    tu = tu1;
                                                end;
                                            end;
                                        end;

                                        mur = mur + calc.rprob(rcat, 1) * mu;
                                        tur = tur + calc.rprob(rcat, 1) * tu;

                                    end;
                                    
                                    margu(acat, 1) = mur;
                                    totu(acat, 1) = tur;
                                    
                                end;
                                
                                % save to marginal and total utility matrices next period, older than lastage, for later calculations;
                                for acat = 1 : gv.nacatlim;
                                    marguf(vcat, dcat, pcat, scat, acat) = margu(acat, 1);
                                    totuf(vcat, dcat, pcat, scat, acat) = totu(acat, 1);
                                end;
                            end;
                        end
                    end;
                end;
            end;
        end;
    end;
    
    % calculate expected marginal and total utility with respect to survival status; 
    % rewrite utlity for case both spouses alive (vcat == 1), because in this case, the next period survival status (vcat) could be 1, 2, or 3;
    % dcat is 1 in case husband is dead because DB pension benedfits are 0;
    % maybe it's easier to also calculate expected marginal and total utility for cases only husband alive and only wife alive here; ???
    if age >= 62;
        for dcat = 1 : 2; % delayed DB pension benefits;
            if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage);
                for pcat = 1 : calc.npcatlim; % DB pension benefits;
                    for scat = 1 : gv.nscatlim; % SS benefits;
                        for acat = 1 : gv.nacatlim; % asset;

                            marguf0 = marguf(1, dcat, pcat, scat, acat); % both spouses alive next period;
                            totuf0 = totuf(1, dcat, pcat, scat, acat);
                            
                            marguf1 = marguf(2, dcat, pcat, scat, acat); % only husband alive next period;
                            totuf1 = totuf(2, dcat, pcat, scat, acat);
                            
                            marguf2 = marguf(3, 1, 1, scat, acat); % only wife alive next period;
                            totuf2 = totuf(3, 1, 1, scat, acat);
                            
                            % rewrite marginal and total utility matrices next period, older than lastage for later calculations;
                            marguf(1, dcat, pcat, scat, acat) ...
                                = calc.transrate(1, age - 25 + 1) * marguf0 ... 
                                + calc.transrate(2, age - 25 + 1) * marguf1 ... 
                                + calc.transrate(3, age - 25 + 1) * marguf2; 
                            totuf(1, dcat, pcat, scat, acat)...
                                = calc.transrate(1, age - 25 + 1) * totuf0 ... 
                                + calc.transrate(2, age - 25 + 1) * totuf1 ... 
                                + calc.transrate(3, age - 25 + 1) * totuf2;

                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate optimal consumption and utility;
    for vcat = 1 : 3; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        if pcat == 1 || vcat < 3; 

                            % calculate DB pension benefits;
                            if vcat < 3 && dcat == 1 && age >= calc.earlypenage; % eligible for DB pension benefits;
                                pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
                            else % not eligible;
                                pben = 0;
                            end;

                            for scat = 1 : gv.nscatlim; % SS benefits;
                                
                                % initialization;
                                cv = zeros(gv.nacatlim, 1); % consumption equivalents of future expected marginal utility;
                                tuv = zeros(gv.nacatlim, 1); % total utility next period discounted back to current period;
                                for acat = 1 : gv.nacatlim; % asset;

                                    % pick corresponding marginal and total utility, given current economic status;
                                    mu = marguf(vcat, dcat, pcat, scat, acat);
                                    tu = totuf(vcat, dcat, pcat, scat, acat);
                                    
                                    if mu <= 0;
                                        fprintf('Error: Bad mu for post-retirement decision: %2d %14.6e %2d %2d %2d %2d %2d',...
                                            age, mu0, mu1, vcat, dcat, pcat, scat, acat);
                                        fprintf('\n');
                                        return;
                                    end;

                                    % recover corresponding consumption, given expected marginal utility;
                                    % discount future total utility to current period;
                                    if vcat == 1; % married couple adjustment;
                                        cv(acat, 1) = sqrt2 * (mu / tp) ^ (1 / (calc.alpha - 1));
                                        tuv(acat, 1) = tu / tp;
                                    else
                                        cv(acat, 1) = (mu * calc.srate(vcat - 1, age - 25 + 1) / tp) ^ (1 / (calc.alpha - 1));
                                        tuv(acat, 1) = tu * calc.srate(vcat - 1, age - 25 + 1) / tp;
                                    end;
                                    
                                end;

                                % calculate SS benefits; 
                                sben = calc.ssbenr(age - 50 + 1, vcat, scat);
                                
                                % calculate total income;
                                y = pben + sben + calc.othincome(age - 25 + 1, vcat);
                                
                                % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                                [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                                
                                % save optimal decisions for later calculations;
                                marguc(vcat, dcat, pcat, scat, :) = margu; % marginal utility at the current period;
                                
                                if vcat ~= 1;
                                    totuc(vcat, dcat, pcat, scat, :) = totu; % total utility at the current period;
                                elseif vcat == 1;
                                    totuc(vcat, dcat, pcat, scat, :) = totutil; % total utility if both spouses alive;                                   
                                    if age < 70;
                                        savd(age - 50 + 1, 3, dcat, pcat, scat, 1, :) = saving; % optimal savings, currently retired from career job;
                                    elseif age >= 70;
                                        savr(age - 70 + 1, dcat, pcat, scat, :) = saving; % optimal savings, surely retired;
                                    end;
                                end;

                            end;

                        end;
                    end;
                end;
            end;
            
            % allow for delayed DB pension receipt;
            if vcat < 3;
                if age > calc.earlypenage && age == calc.delayedpenage;
                    marguc(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim) = marguc(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim);
                    totuc(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim) = totuc(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim);
                elseif age == calc.earlypenage && age < calc.delayedpenage;
                    marguc(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim) = marguc(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim);
                    totuc(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim) = totuc(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.nscatlim);
                end;
            end;

        end;
    end;
    
    % save to marginal and total utility matrices next period (when calculating backward at age - 1) for later calculations;
    marguf = marguc;
    totuf = totuc;
    
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% force retirement at lastage + 1 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate marginal utility next period (at lastage + 1), currently working in career job at lastage;
% people will surely retire at lastage + 1;
% DC pension balance is added to asset when retiring career job if husband is eligible for DC pension;
% do not need to consider vcat == 3 because husband is alive and working;
% set consumption and utility for all epsilon categories the same because it doesn't affect anything in surely retired periods;
% calculate transition matrix of asset grid after adding DC pension balance when retiring career job at lastage + 1;
eowamt = data.eowamount((calc.lastage + 1) - 50 + 1, 1); % early out window amount;
for lcat = 1 : calc.nlcatlim; % DC pension balance;
    if (lcat > 1 && calc.lastage + 1 <= data.penjobend + 1 && calc.lastage + 1 > calc.contribstartage) || (lcat == 1 && eowamt > 0); % eligible for DC pension;
        for acat = 1 : gv.nacatlim; % asset;
            
            % calculate asset after including DC pension balance;
            pwealth = calc.acats(acat, 1) + calc.lcats(lcat, 1) + eowamt;
            
            % find the corresponding grid point in asset grid;
            % interpolation;
            if acat < gv.nacatlim;
                lacat = acat + 1;
            else
                lacat = gv.nacatlim;
            end;
            while lacat < gv.nacatlim && pwealth > calc.acats(lacat, 1);
                lacat = lacat + 1;
            end;
            lacat = lacat - 1;
            
            lacats(lcat, acat) = lacat + (pwealth - calc.acats(lacat, 1)) / (calc.acats(lacat + 1, 1) - calc.acats(lacat, 1));
        end;
    else % no DC pension;
        if lcat == 1; 
            for acat = 1 : gv.nacatlim;
                lacats(lcat, acat) = acat;
            end;
        end;
    end;
end;

% prepare for interpolation;
pcat = floor(calc.initialpcat((calc.lastage + 1) - 50 + 1, 1));
fp = calc.initialpcat((calc.lastage + 1) - 50 + 1, 1) - pcat;
scat = floor(calc.initialscat((calc.lastage + 1) - 50 + 1, 1));
fs = calc.initialscat((calc.lastage + 1) - 50 + 1, 1) - scat;
for vcat = 1 : 2; % survival status;
    if vcat == 1 || calc.lastage + 1 > 62;
        for lcat = 1 : calc.nlcatlim; % DC pension balance;
            if lcat == 1 || (calc.lastage + 1 <= data.penjobend + 1 && calc.lastage + 1 > calc.contribstartage);
                for acat = 1 : gv.nacatlim; % asset;
                    
                    % prepare for interpolation;
                    lacat = floor(lacats(lcat, acat));
                    if lacat > gv.nacatlim - 1;
                        lacat = gv.nacatlim - 1;
                    end;
                    fla = lacats(lcat, acat) - lacat;
                    
                    for utype = 1 : 2; % calculate marginal utility or total utility; 1 is marginal utility, 2 is total utility;
                        
                        % if retires at lastage + 1, DB pension grid point is between pcat and pcat + 1; SS benefits grid point is between scat and scat + 1;
                        for  pcati = pcat : pcat + 1; % DB pension benefits grid point;
                            if pcati == pcat || fp ~= 0;
                                for scati = scat : scat + 1; % SS benefits grid point;
                                    if scati == scat || fs ~= 0;
                                        
                                        % 3 dimension interpolation;
                                        % interpolation: asset;
                                        if fla ~= 0; 
                                            if utype == 1; % calculate marginal utility;
                                                
                                                % pick corresponding marginal utility, given current economic status; 
                                                mu0 = marguf(vcat, 1, pcati, scati, lacat);
                                                mu1 = marguf(vcat, 1, pcati, scati, lacat + 1);
                                                
                                                if fla <= 1;
                                                    temp = (1 - fla) * mu0 + fla * mu1;
                                                else
                                                    if mu0 > mu1;
                                                        dmu = mu0 - mu1;
                                                        mufact = dmu / mu0;
                                                        temp = mu1 / (1 + (fla - 1) * mufact);
                                                    else
                                                        temp = mu1;
                                                    end;
                                                end;
                                                
                                            elseif utype == 2; % calculate total utility;
                                                
                                                % pick corresponding total utility, given current economic status;
                                                tu0 = totuf(vcat, 1, pcati, scati, lacat);
                                                tu1 = totuf(vcat, 1, pcati, scati, lacat + 1);
                                                
                                                if fla <= 1;
                                                    temp = (1 - fla) * tu0 + fla * tu1;
                                                else
                                                    mu0 = marguf(vcat, 1, pcati, scati, lacat);
                                                    mu1 = marguf(vcat, 1, pcati, scati, lacat + 1); 
                                                    
                                                    if mu0 > mu1 && tu1 > tu0;
                                                        dmu = mu0 - mu1;
                                                        dtu = tu1 - tu0;
                                                        muratio = mu0 / mu1;
                                                        mufact = dmu / mu0;
                                                        temp = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                                                    else
                                                        if tu1 > tu0;
                                                            temp = (1 - fla) * tu0 + fla * tu1;
                                                        else
                                                            temp = tu1;
                                                        end;
                                                    end;
                                                end;
                                                
                                            end;
                                        else
                                            if utype == 1;
                                                temp = marguf(vcat, 1, pcati, scati, lacat);
                                            else
                                                temp = totuf(vcat, 1, pcati, scati, lacat);
                                            end;
                                        end;
                                        
                                        if scati == scat;
                                            temps(1, 1) = temp;
                                        else
                                            temps(2, 1) = temp;
                                        end;
                                    end;
                                end; 
                                
                                % interpolation: SS benefits;
                                % interpolation codes do not look right; util should be replaced by temp???
%                                 if fs ~= 0;
%                                     util = (1 - fs) * temps(1, 1) + fs * temps(2, 1);
%                                 else
%                                     util = temps(1, 1);
%                                 end;
                                if fs ~= 0;
                                    temp = (1 - fs) * temps(1, 1) + fs * temps(2, 1);
                                else
                                    temp = temps(1, 1);
                                end;
                                
                                if pcati == pcat;
                                    tempp(1, 1) = temp;
                                else
                                    tempp(2, 1) = temp;
                                end;
                            end;
                        end;
                        
                        % interpolation: DB pension benefits;
                        if fp ~= 0;
                            util = (1 - fp) * tempp(1, 1) + fp * tempp(2, 1);
                        else
                            util = tempp(1, 1);
                        end;
                        
                        if utype == 1;
                            % save to marginal utility next period, currently working in career job, for future calculations;
                            for ecat = 1 : gv.necatlim;
                                margufw(vcat, lcat, ecat, acat) = util;
                            end;
                        else
                            % save to total utility next period, currently working in career job, for future calculations;
                            for ecat = 1 : gv.necatlim;
                                totufw(vcat, lcat, ecat, acat) = util;
                            end;
                        end;
                        
                    end;
                end;
            end;
        end;
    end;
end;

% calculate marginal utility next period (at lastage + 1), already retired from career job at lastage;
% no DC transfer at lastage + 1;
% leisure preference is useful only if the husband is alive;
% set consumption and utility for all epsilon categories the same;
for vcat = 1 : 3; % survival status;
    if vcat == 1 || calc.lastage + 1 > 62;
        for dcat = 1 : 2; % delayed DB pension;
            if dcat == 1 || (vcat < 3 && calc.earlypenage < calc.lastage + 1 && calc.lastage + 1 <= calc.delayedpenage);
                for pcat = 1 : calc.npcatlim; % DB pension benefits;
                    if pcat == 1 || vcat < 3;
                        for scat = 1 : gv.nscatlim; % SS benefits;
                            for ecat = 1 : gv.necatlim; % leisure preference;
                                if ecat == 1 || vcat < 3; 
                                    for acat = 1 : gv.nacatlim; % asset;
                                        
                                        % save to marginal and total utility next period, retired from career job, for future calculations;
                                        margufr(vcat, dcat, pcat, scat, ecat, acat) = marguf(vcat, dcat, pcat, scat, acat);
                                        totufr(vcat, dcat, pcat, scat, ecat, acat) = totuf(vcat, dcat, pcat, scat, acat);
                                        
                                        if margufr(vcat, dcat, pcat, scat, ecat, acat) <= 0;
                                            fprintf('Error: dd %2d %d %d %d %d %d %d', calc.lastage + 1, vcat, dcat, pcat, scat, ecat, acat); 
                                            return;
                                        end;
                                        
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate consumption, saving and utility from firstage to lastage %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% people could work in career job, or in three other job categories if retired from career job in this period:
% returned full time job, part time job, and retired;
% when working, wage is paid base on job types;
% after retirement, income only comes from DB pension, Social Security benefits and other income;
% do calculations given the following economic state: 
% if working in career job: vcat, lcat, ecat, acat;
% if retired from career job: vcat, dcat, pcat, scat, ecat, acat;
% when age <= 62, it's impossible vcat > 1; avoid calculations to save computational power;
% when dcat == 2 && vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage, people do not receive DB pension benefits;
% when only wife alive (vcat == 3), DB benefit is 0. assign the corresponding value to the 1st pcats grid point;
% DB pension benefits are not 0 iff husband is alive and eligible for DB pension;
% actuarial adjustment of DB pension benefits are required because pcats have been discounted back to age 50;
% calculate SS benefits received based on job types (earnings' test); 
% if working in career job, calculate using ssbenm;
% if working in returned full time job, calculate using ssbens;
% if working in part time job, calculate using ssbenp;
% if retired, calculate using ssbenr; 
% pia is not affected by job history, it is only a function of age;
% most savings are saved in savd (saving in decision periods) ; 
% some savings are saved in savr (saving after retirement)
% some savings are saved in savw (saving when working in career job);  
% if saving is negative, it means households decumulate; that is, consume more than their income;
% the 2nd dimension of savd is job category, 1 if secjob, 2 if part time work, 3 if retired;
% (end of period asset + saving) * asset return shock is asset next period;

for age = calc.lastage : -1 : calc.firstage;
    
    % calculate expected marginal utility in the next period, currently retired from career job, with respect to asset returns;
    for vcat = 1 : 3; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        if pcat == 1 || vcat < 3;
                            for scat = 1 : gv.nscatlim; % SS benefits;
                                for ecat = 1 : gv.necatlim; % leisure preference;
                                    if ecat == 1 || vcat < 3;
                                        
                                        % initialization;
                                        % intermediate variables;
                                        margu = zeros(gv.nacatlim, 1); % expected marginal utility next period by asset given current economic state;
                                        totu = zeros(gv.nacatlim, 1); % expected total utility next period by asset given current economic state;
                                        for acat = 1 : gv.nacatlim;
                                            
                                            mu = 0; % expected marginal utility next period given current economic state;
                                            tu = 0; % expected marginal utility next period given current economic state;
                                            for rcat = 1 : gv.nrcatlim;
                                                
                                                % interpolation;
                                                acati = floor(calc.rtransa(acat, rcat));
                                                if acati >= gv.nacatlim;
                                                    acati = gv.nacatlim - 1;
                                                end;
                                                fa = calc.rtransa(acat, rcat) - acati;
                                                
                                                mu0 = margufr(vcat, dcat, pcat, scat, ecat, acati);
                                                mu1 = margufr(vcat, dcat, pcat, scat, ecat, acati + 1);
                                                tu0 = totufr(vcat, dcat, pcat, scat, ecat, acati);
                                                tu1 = totufr(vcat, dcat, pcat, scat, ecat, acati + 1);
                                                
                                                if mu0 <= 0 || mu1 <= 0;
                                                    fprintf('Interpolation Error: Bad mu in decision-retired section: %2d %14.6e %14.6e %2d %2d %2d %2d %2d %2d %2d %2d',...
                                                        age, mu0, mu1, vcat, dcat, pcat, scat, ecat, acat, acati, rcat);
                                                    return;
                                                end;
                                                
                                                if fa <= 1;
                                                    mur = (1 - fa) * mu0 + fa * mu1; % marginal utility next period given current economic state and asset return shock;
                                                    tur = (1 - fa) * tu0 + fa * tu1; % total utility next period given current economic state and asset return shock;
                                                else
                                                    if mu0 > mu1 && tu1 > tu0;
                                                        dmu = mu0 - mu1;
                                                        dtu = tu1 - tu0;
                                                        muratio = mu0 / mu1;
                                                        mufact = dmu / mu0;
                                                        
                                                        mur = mu1 / (1 + (fa - 1) * mufact);
                                                        tur = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                                                    else
                                                        mur = mu1;
                                                        if tu1 > tu0;
                                                            tur = (1 - fa) * tu0 + fa * tu1;
                                                        else
                                                            tur = tu1;
                                                        end;
                                                    end;
                                                end;
                                                
                                                mu = mu + calc.rprob(rcat, 1) * mur;
                                                tu = tu + calc.rprob(rcat, 1) * tur;
                                            end;
                                            
                                            margu(acat, 1) = mu;
                                            totu(acat, 1) = tu;
                                        end;
                                        
                                        % save to marginal and total utility matrices next period, currently retired from career job, for later calculations;
                                        for acat = 1 : gv.nacatlim;
                                            margufr(vcat, dcat, pcat, scat, ecat, acat) = margu(acat, 1);
                                            totufr(vcat, dcat, pcat, scat, ecat, acat) = totu(acat, 1);
                                        end;
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate expected marginal utility in the next period, currently working in career job, with respect to asset returns;
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for ecat = 1 : gv.necatlim; % leisure preference;
                
                % initialization;
                % intermediate variables;
                mula = zeros(calc.nlcatlim, gv.nacatlim); % marginal utility next period by end of period asset, and DC pension balance, given current economic state;
                tula = zeros(calc.nlcatlim, gv.nacatlim); % total utility next period by end of period asset, and DC pension balance, given current economic state;
                
                mular = zeros(calc.nlcatlim, gv.nacatlim, gv.nrcatlim); % marginal utility next period by end of period asset, DC pension balance and asset return shock, given current economic state;
                tular = zeros(calc.nlcatlim, gv.nacatlim, gv.nrcatlim); % total utility next period by end of period asset, DC pension balance and asset return shock, given current economic state;
                
                for lcat = 1 : calc.nlcatlim; % DC pension balance;
                    if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                        for acat = 1 : gv.nacatlim; % asset;
                            
                            mu = 0; % marginal utility next period by end of period asset, given current economic state;
                            tu = 0; % total utility next period by end of period asset, given current economic state;
                            for rcat = 1 : gv.nrcatlim;
                                
                                % 2 dimension interpolation (asset and DC pension balance);
                                acati = floor(calc.rtransa(acat, rcat));                                
                                if acati >= gv.nacatlim;
                                    acati = gv.nacatlim - 1;
                                end;
                                fa = calc.rtransa(acat, rcat) - acati;
                                
                                if calc.nlcatlim > 1 && age < data.penjobend + 1 && age > calc.contribstartage;
                                    
                                    lcati = floor(calc.rtransl(age - 25 + 1, lcat, rcat));                                    
                                    if lcati >= calc.nlcatlim;
                                        lcati = calc.nlcatlim - 1;
                                    end;                                    
                                    fl = calc.rtransl(age - 25 + 1, lcat, rcat) - lcati;
                                    
                                    if fa <= 1 && fl <= 1;
                                        mur = (1 - fl) * ((1 - fa) * margufw(vcat, lcati, ecat, acati) ...
                                            + fa * margufw(vcat, lcati, ecat,acati + 1)) ...
                                            + fl * ((1 - fa) * margufw(vcat, lcati + 1, ecat, acati) ...
                                            + fa * margufw(vcat, lcati + 1, ecat, acati + 1));
                                        tur = (1 - fl) * ((1 - fa) * totufw(vcat, lcati, ecat, acati) ...
                                            + fa * totufw(vcat, lcati, ecat,acati + 1)) ...
                                            + fl * ((1 - fa) * totufw(vcat, lcati + 1, ecat, acati) ...
                                            + fa * totufw(vcat, lcati + 1, ecat, acati + 1));
                                        
                                        if mur <= 0;
                                            fprintf('Interpolation Error: Bad mu in decision-main job section: %2d %14.6e %14.6e %14.6e %14.6e',...
                                                age, margufw(vcat, lcati, ecat, acati),...
                                                margufw(vcat, lcati, ecat, acati + 1),...
                                                margufw(vcat, lcati + 1, ecat, acati),...
                                                margufw(vcat, lcati + 1, ecat, acati + 1));
                                            fprintf(' %2d %2d %2d %2d %2d %2d %2d', vcat, lcat, lcati, ecat, acat, acati, rcat);
                                            return;
                                        end;
                                        
                                    else
                                        if rcat >= 3;
                                            mu0 = mular(lcat, acat, rcat - 2);
                                            mu1 = mular(lcat, acat, rcat - 1);
                                            tu0 = tular(lcat, acat, rcat - 2);
                                            tu1 = tular(lcat, acat, rcat - 1);
                                            
                                            if mu0 > mu1 && tu1 > tu0;
                                                dmu = mu0 - mu1;
                                                dtu = tu1 - tu0;
                                                muratio = mu0 / mu1;
                                                mufact = dmu / mu0;
                                                
                                                mur = mu1 / (1 + mufact);
                                                tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                                            else
                                                mur = mu1;
                                                if tu1 > tu0;
                                                    tur = 2 * tu1 - tu0;
                                                else
                                                    tur = tu1;
                                                end;
                                            end;
                                        else
                                            if lcat > calc.nlcatlim - 4;
                                                mu0 = mular(lcat - 2, acat, rcat);
                                                mu1 = mular(lcat - 1, acat, rcat);
                                                tu0 = tular(lcat - 2, acat, rcat);
                                                tu1 = tular(lcat - 1, acat, rcat);
                                                
                                                if mu0 > mu1 && tu1 > tu0;
                                                    dmu = mu0 - mu1;
                                                    dtu = tu1 - tu0;
                                                    muratio = mu0 / mu1;
                                                    mufact = dmu / mu0;
                                                    
                                                    mur = mu1 / (1 + (fl - 1) * mufact);
                                                    tur = tu1 + (dtu / log(muratio)) * log(1 + (fl - 1) * mufact);
                                                else
                                                    mur = mu1;
                                                    
                                                    if tu1 > tu0;
                                                        tur = (1 - fl) * tu0 + fl * tu1;
                                                    else
                                                        tur = tu1;
                                                    end;
                                                end;
                                            else
                                                fprintf('Interpolation Error: fl problem: age %d rcat %d lcat %d %d %6.3f acat %d %d %6.3f calc.rtransl %6.3f',...
                                                    age, rcat, lcat, lcati, fl, acat, acati, fa, calc.rtransl(age - 25 + 1, lcat, rcat));
                                                return;
                                            end;
                                        end;
                                        
                                        
                                        if rcat > 1;
                                            if mur > mular(lcat, acat, rcat - 1);
                                                mur = mular(lcat, acat, rcat - 1);
                                            end;
                                        end;
                                        
                                        if lcat > 1;
                                            if mur > mular(lcat - 1, acat, rcat);
                                                mur = mular(lcat - 1, acat, rcat);
                                            end;
                                        end;
                                        
                                        if rcat > 1;
                                            if tur < tular(lcat, acat, rcat - 1);
                                                tur = tular(lcat, acat, rcat - 1);
                                            end;
                                        end;
                                        
                                        if lcat > 1;
                                            if tur < tular(lcat - 1, acat, rcat);
                                                tur = tular(lcat - 1, acat, rcat);
                                            end;
                                        end;                                        
                                    end;
                                else
                                    if fa <= 1;
                                        mur = (1 - fa) * margufw(vcat, 1, ecat, acati) ...
                                            +  fa     * margufw(vcat, 1, ecat, acati + 1);
                                        tur = (1 - fa) * totufw(vcat, 1, ecat, acati) ...
                                            +  fa     * totufw(vcat, 1, ecat, acati + 1);
                                    else
                                        if rcat >= 3;
                                            mu0 = mular(1, acat, rcat - 2);
                                            mu1 = mular(1, acat, rcat - 1);
                                            tu0 = tular(1, acat, rcat - 2);
                                            tu1 = tular(1, acat, rcat - 1);
                                            
                                            if mu0 > mu1 && tu1 > tu0;
                                                dmu = mu0 - mu1;
                                                dtu = tu1 - tu0;
                                                muratio = mu0 / mu1;
                                                mufact = dmu / mu0;
                                                
                                                mur = mu1 / (1 + mufact);
                                                tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                                            else
                                                mur = mu1;
                                                
                                                if tu1 > tu0;
                                                    tur = 2 * tu1 - tu0;
                                                else
                                                    tur = tu1;
                                                end;
                                            end;
                                        else
                                            fprintf('Error: fl problem: age %d rcat %d lcat %d %d %6.3f acat %d %d %6.3f calc.rtransl %6.3f',...
                                                age, rcat, lcat, lcati, fl, acat, acati, fa, calc.rtransl(age - 25 + 1, lcat, rcat));
                                            return;
                                        end;
                                        
                                        if rcat > 1;
                                            if mur > mular(lcat, acat, rcat - 1);
                                                mur = mular(lcat, acat, rcat - 1);
                                            end;
                                        end;
                                        
                                        if rcat > 1;
                                            if tur < tular(lcat, acat, rcat - 1);
                                                tur = tular(lcat, acat, rcat - 1);
                                            end;
                                        end;
                                    end;
                                end;
                                
                                mular(lcat, acat, rcat) = mur; % marginal utility next period by end of period asset, DC pension balance and asset return shock, given other current economic state;
                                tular(lcat, acat, rcat) = tur; % total utility next period by end of period asset, DC pension balance and asset return shock, given other current economic state;
                                
                                mu = mu + calc.rprob(rcat, 1) * mur; % marginal utility next period by end of period asset, given current economic state;
                                tu = tu + calc.rprob(rcat, 1) * tur; % marginal utility next period by end of period asset, given current economic state;
                            end;
                            mula(lcat, acat) = mu;
                            tula(lcat, acat) = tu;
                        end;
                    end;
                end;
                for lcat = 1 : gv.nlcatlim;
                    if lcat == 0 || (age <= data.penjobend && age > calc.contribstartage);
                        for acat = 1 : gv.nacatlim;
                           
                            % save to marginal and total utility matrices next period, currently working in career job, for later calculations;
                            margufw(vcat, lcat, ecat, acat) = mula(lcat, acat);
                            totufw(vcat, lcat, ecat, acat) = tula(lcat, acat);
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate expected marginal and total utility next period, currently retired from career job, with respect to survival state;
    % vcat == 1;
    if age >= 62; 
        for dcat = 1 : 2; % delayed DB pension;
            if dcat == 1 || (age >= calc.earlypenage && age < calc.delayedpenage);
                for pcat = 1 : calc.npcatlim; % DB pension benefits;
                    for scat = 1 : gv.nscatlim; % SS benefits;
                        for ecat = 1 : gv.necatlim; % leisure preference;
                            for acat = 1 : gv.nacatlim; % asset;
                                
                                margufr0 = margufr(1, dcat, pcat, scat, ecat, acat); % both spouses alive next period;
                                totufr0 = totufr(1, dcat, pcat, scat, ecat, acat);
                                
                                margufr1 = margufr(2, dcat, pcat, scat, ecat, acat); % only husband alive next period;
                                totufr1 = totufr(2, dcat, pcat, scat, ecat, acat);
                                
                                margufr2 = margufr(3, 1, 1, scat, 1, acat); % only wife alive next period; pay attention to dimensions;
                                totufr2 = totufr(3, 1, 1, scat, 1, acat);
                                
                                % rewrite marginal and total utility matrices next period, currently retired from career job, for later calculations;
                                margufr(1, dcat, pcat, scat, ecat, acat) = calc.transrate(1, age - 25 + 1) * margufr0 ...
                                    + calc.transrate(2, age - 25 + 1) * margufr1 ...
                                    + calc.transrate(3, age - 25 + 1) * margufr2;
                                
                                totufr(1, dcat, pcat, scat, ecat, acat) = calc.transrate(1, age - 25 + 1) * totufr0 ...
                                    + calc.transrate(2, age - 25 + 1) * totufr1 ...
                                    + calc.transrate(3, age - 25 + 1) * totufr2;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate expected marginal and total utility next period, currently working in career job, with respect to survival state;
    % vcat = 1;
    if age >= 62;
        
        % if he dies next period, he is considered as retired next period;
        % prepare for interpolation;
        scat = floor(calc.initialscat((calc.lastage + 1) - 50 + 1, 1));
        fs = calc.initialscat((calc.lastage + 1) - 50 + 1, 1) - scat;
        
        for ecat = 1 : gv.necatlim; % leisure preference;
            for lcat = 1 : calc.nlcatlim; % DC pension balance;
                if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                    for acat = 1 : gv.nacatlim; % asset;
                        
                        margufw0 = margufw(1, lcat, ecat, acat); % both spouses alive next period;
                        totufw0 = totufw(1, lcat, ecat, acat);
                        
                        margufw1 = margufw(2,lcat, ecat, acat); % only husband alive next period;
                        totufw1 = totufw(2,lcat, ecat, acat);
                        
                        % only wife alive next period;
                        % if the husband dies in career job, he is considered as retired and his DC pension is added to asset immediately;
                        % pay attention to type of marginal utility used;
                        % calculate asset after including DC pension balance;
                        wealth = calc.acats(acat, 1) + calc.lcats(lcat, 1);
                        
                        % interpolation: asset;
                        if acat < gv.nacatlim;
                            acati = acat + 1;
                        else
                            acati = gv.nacatlim;
                        end;
                        while acati < gv.nacatlim && wealth > calc.acats(acati, 1);
                            acati = acati + 1;
                        end;
                        acati = acati - 1;
                        fa = (wealth - calc.acats(acati, 1)) / (calc.acats(acati + 1, 1) - calc.acats(acati, 1));
                        
                        % interpolation: SS benefits; 
                        % original codes out of bound if scat == 4; 
                        % although it is correct as fs == 0, better to revise to avoid confusion; ???
                        if scat == 4;
                            mu0 = margufr(3, 1, 1, scat, 1, acati );
                            mu1 = margufr(3, 1, 1, scat, 1, acati + 1);
                            tu0 = totufr(3, 1, 1, scat, 1, acati);
                            tu1 = totufr(3, 1, 1, scat, 1, acati + 1);
                        else
                            mu0 = (1 - fs) * margufr(3, 1, 1, scat, 1, acati ) ...
                                + fs * margufr(3, 1, 1, scat + 1, 1, acati);
                            mu1 = (1 - fs) * margufr(3, 1, 1, scat, 1, acati + 1) ...
                                + fs * margufr(3, 1, 1, scat + 1, 1, acati + 1);
                            tu0 = (1 - fs) * totufr(3, 1, 1, scat, 1, acati) ...
                                + fs * totufr(3, 1, 1, scat + 1, 1, acati);
                            tu1 = (1 - fs) * totufr(3, 1, 1, scat, 1, acati + 1) ...
                                + fs * totufr(3, 1, 1, scat + 1, 1, acati + 1);
                        end;
                        
                        if fa <= 1;
                            margufr2 = (1 - fa) * mu0 + fa * mu1;
                            totufr2 = (1 - fa) * tu0 + fa * tu1;
                        else
                            if mu0 > mu1 && tu1 > tu0;
                                dmu = mu0 - mu1;
                                dtu = tu1 - tu0;
                                muratio = mu0 / mu1;
                                mufact = dmu / mu0;
                                
                                margufr2 = mu1 / (1 + (fa - 1) * mufact);
                                totufr2 = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                            else
                                margufr2 = mu1;
                                
                                if tu1 > tu0;
                                    totufr2 = (1 - fa) * tu0 + fa * tu1;
                                else
                                    totufr2 = tu1;
                                end;
                            end;
                        end;

                        % rewrite marginal and total utility matrices next period, currently working in career job, for later calculations;
                        margufw(1, lcat, ecat, acat) ...
                            = calc.transrate(1, age - 25 + 1) * margufw0 ...
                            + calc.transrate(2, age - 25 + 1) * margufw1 ...
                            + calc.transrate(3, age - 25 + 1) * margufr2;
                        
                        totufw(1, lcat, ecat, acat)...
                            = calc.transrate(1, age - 25 + 1) * totufw0 ...
                            + calc.transrate(2, age - 25 + 1) * totufw1 ...
                            + calc.transrate(3, age - 25 + 1) * totufr2;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate consumption equivalents of future marginal utility, currently retired from career job;
    for vcat = 1 : 3; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        if pcat == 1 || vcat < 3; 
                            for scat = 1 : gv.nscatlim; % SS benefits;
                                for ecat = 1 : gv.necatlim; % leisure prefence;
                                    if ecat == 1 || vcat < 3;
                                        for acat = 1 : gv.nacatlim; % asset;
                                            
                                            % pick corresponding marginal and total utility given current economic status;
                                            mu = margufr(vcat, dcat, pcat, scat, ecat, acat);
                                            tu = totufr(vcat, dcat, pcat, scat, ecat, acat);                                            
                                            if mu <= 0;
                                                fprintf('Error: Bad mu for decision-retired consumption: %2d %14.6e %2d %2d %2d %2d %2d %2d',...
                                                    age, mu0, mu1, vcat, dcat, pcat, scat, ecat, acat);
                                                return;
                                            end;
                                            
                                            % recover corresponding consumption, given expected marginal utility;
                                            % discount future total utility to current period;
                                            if vcat == 1; % married couple adjustment;
                                                cmatr(vcat, dcat, pcat, scat, ecat, acat) = sqrt2 * (mu / tp) ^ (1 / (calc.alpha - 1)); % consumption;
                                                tumatr(vcat, dcat, pcat, scat, ecat, acat) = tu / tp; % discounted total utility;
                                            else
                                                cmatr(vcat, dcat, pcat, scat, ecat, acat) = (mu * calc.srate(vcat - 1, age - 25 + 1) / tp) ^ (1 / (calc.alpha - 1)); % consumption;
                                                tumatr(vcat, dcat, pcat, scat, ecat, acat) = tu * calc.srate(vcat - 1, age - 25 + 1) / tp; % discounted utility;
                                            end;
                                            
                                        end;
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate consumption equivalents of future marginal utility, currently working in career job;
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62; 
            for ecat = 1 : gv.necatlim; % leisure prefence;
                for lcat = 1 : calc.nlcatlim; % DC pension balance;
                    if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                        for acat = 1 : gv.nacatlim; % asset;
                            
                            % pick corresponding marginal and total utility, given current economic status;
                            mu = margufw(vcat, lcat, ecat, acat);
                            tu = totufw(vcat, lcat, ecat, acat);
                            if mu <= 0;
                                fprintf('Error: Bad mu for decision-main job consumption: %2d %14.6e %2d %2d %2d %2d', ...
                                    age, mu0, mu1, vcat, lcat, ecat, acat);
                                return;
                            end;
                            
                            % recover corresponding consumption, given expected marginal utility;
                            % discount future total utility to current period;
                            if vcat == 1; % married couple adjustment;
                                cmatw(vcat, lcat, ecat, acat) = sqrt2 * (mu / tp) ^ (1 / (calc.alpha - 1)); % consumption;
                                tumatw(vcat, lcat, ecat, acat) = tu / tp; % discounted total utility;
                            else
                                cmatw(vcat, lcat, ecat, acat) = (mu * calc.srate(vcat - 1, age - 25 + 1) / tp) ^ (1 / (calc.alpha - 1)); % consumption;
                                tumatw(vcat, lcat, ecat, acat) = tu * calc.srate(vcat - 1, age - 25 + 1) / tp; % discounted total utility;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate optimal consumption, saving, marginal and total utility for staying in career job;
    % calculate DB pension benefits;
    % staying in career job could also receive DB pension benefits; how does data.penjobend work ???
    if calc.npcatlim > 1 && age > data.penjobend; % eligible for DB pension benefits;
        
        % interpolation;
        pcat = floor(calc.initialpcat((max(data.penjobend, 49) + 1) - 50 + 1, 1));
        fp = calc.initialpcat((max(data.penjobend, 49) + 1) - 50 + 1, 1) - pcat;
        if fp ~= 0;
            pben = ((1 - fp) * calc.pcats(pcat, 1) + fp * calc.pcats(pcat + 1, 1)) ...
                * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
        else
            pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
        end;
        
    else % not eligible;
        pben = 0;
    end;
    
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            
            % calculate SS benefits;
            % subject to earnings' test, working in career job;
            sben = calc.ssbenm(age - 25 + 1, vcat);
            
            for lcat = 1 : calc.nlcatlim; % DC pension balance;
                if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                    for ecat = 1 : gv.necatlim;
                        
                        % pick corresponding consumption and total utility next period, given current economic status;
                        % initialization;
                        cv = zeros(gv.nacatlim, 1); % consumption;
                        tuv = zeros(gv.nacatlim, 1); % total utility;
                        
                        for acat = 1: gv.nacatlim; % asset;
                            cv(acat, 1) = cmatw(vcat, lcat, ecat, acat);
                            tuv(acat, 1) = tumatw(vcat, lcat, ecat, acat);
                        end;
                        
                        % calculate total income;
                        y = data.ftwage(age - 25 + 1, 1) + pben + sben + calc.othincome(age - 25 + 1, vcat);
                        
                        % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                        [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                        
                        % save optimal decisions for later calculations;
                        margucw(vcat, lcat, ecat, :) = margu; % marginal utility current period, currently working in career job;
                        totucw(vcat, lcat, ecat, :) = totu; % total utility current period, currently working in career job;
                        
                        if vcat == 1;
                            savw(age - 25 + 1, lcat, ecat, :) = saving; % optimal savings, currently working in career job;
                            totuw(age - 25 + 1, lcat, ecat, :) = totutil; % total utility if both spouses alive, currently working in career job; 
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate optimal consumption, saving, marginal and total utility for retirement;
    for vcat = 1 : 3; % survival status;
        if vcat == 1 || age > 62            
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (vcat < 3 && age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        if pcat == 0 || vcat < 3;
                            
                            % calculate DB pension benefits;
                            if vcat < 3 && dcat == 1 && age >= calc.earlypenage; % eligible for DB pension benefits;
                                pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
                            else % not eligible;
                                pben = 0;
                            end;
                            
                            for scat = 1 : gv.nscatlim; % SS benefits;
                                
                                % calculate SS benefits;
                                sben = calc.ssbenr(age - 50 + 1, vcat, scat);
                                
                                for ecat = 1 : gv.necatlim; % leisure preference;
                                    if ecat == 1 || vcat < 3;
                                        
                                        % pick corresponding consumption and total utility, given current economic status;
                                        % initialization;
                                        cv = zeros(gv.nacatlim, 1); % consumption;
                                        tuv = zeros(gv.nacatlim, 1); % total utility;
                                        
                                        for acat = 1 : gv.nacatlim; % asset;
                                            cv(acat, 1) = cmatr(vcat, dcat, pcat, scat, ecat, acat);
                                            tuv(acat, 1) = tumatr(vcat, dcat, pcat, scat, ecat, acat);
                                        end;
                                        
                                        % calculate total income;
                                        y = pben + sben + calc.othincome(age - 25 + 1, vcat);
                                        
                                        % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                                        [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                                        
                                        % calculate total utility by including utility from leisure;
                                        for acat = 1 : gv.nacatlim;
                                            totu(acat, 1) = totu(acat, 1) + calc.utilr(ecat, age - 50 + 1); % include utility from leisure;
                                            if vcat == 1
                                                totutil(acat, 1) = totutil(acat, 1) + calc.utilr(ecat, age - 50 + 1); % include utility from leisure;
                                            end;
                                        end;
                                        
                                        % save optimal decisions for later calculations;
                                        margucr(3, vcat, dcat, pcat, scat, ecat, :) = margu; % marginal utility current period, retired from career job; 3 indicates retired;
                                        totucr(3, vcat, dcat, pcat, scat, ecat, :) = totu; % total utility current period, retired from career job; 3 indicates retired;
                                        
                                        if vcat == 1;
                                            savd(age - 50 + 1, 3, dcat, pcat, scat, ecat, :) = saving; % optimal savings, currently retired from career job; 3 indicates retired;
                                            totud(age - 50 + 1, 3, dcat, pcat, scat, ecat, :) = totutil; % total utility if both spouses alive, currently retired from career job; 3 indicates retired;
                                        end;
                                        
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate optimal consumption, saving, marginal and total utility for part time work;
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        
                        % calculate DB pension benefits;
                        if dcat == 1 && age >= calc.earlypenage; % eligible for DB pension benefits;
                            pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
                        else % not eligible;
                            pben = 0;
                        end;
                        
                        for scat = 1 : gv.nscatlim; % SS benefits;
                            
                            % calculate SS benefits;
                            if calc.sserage <= age && age < 70;
                                
                                benchange = calc.benchangep(age - 62 + 1, vcat, scat); % partitial retirement benefits change;
                                if benchange > 0;
                                    sbenf = calc.ssbenf(scat, 1) + calc.benchangep(age - 62 + 1, vcat, scat);
                                    
                                    fs = -1;
                                    if scat == 4;
                                        scati = 5;
                                    else
                                        for scati = scat + 1 : gv.nscatlim;
                                            if fs == -1 && sbenf < calc.ssbenf(scati, 1);
                                                fs = (scati - 1) + (sbenf - calc.ssbenf(scati - 1, 1))...
                                                    / (calc.ssbenf(scati, 1) - calc.ssbenf(scati - 1, 1));
                                            end;
                                        end;
                                    end;
                                    
                                    if fs == -1;
                                        fs = (scati - 2) + 0.99;
                                    end;
                                    
                                    scati = floor(fs);
                                    fs = fs - scati;
                                else
                                    scati = scat;
                                    fs = 0;
                                end;
                            else
                                scati = scat;
                                fs = 0;
                            end;
                            
                            sben = calc.ssbenp(age - 50 + 1, vcat, scat);
                            
                            for ecat = 1 : gv.necatlim; % leisure preference;
                                
                                % initialization;
                                cv = zeros(gv.nacatlim, 1); % consumption equivalents of future expected marginal utility;
                                tuv = zeros(gv.nacatlim, 1); % total utility next period discounted back to current period;
                                
                                % interpolation;
                                for acat = 1 : gv.nacatlim; % asset;
                                    if fs ~= 0;
                                        cv(acat, 1) = (1 - fs) .* cmatr(vcat, dcat, pcat, scati, ecat, acat) ...
                                            + fs .* cmatr(vcat, dcat, pcat, scati + 1, ecat, acat); % consumption equivalents of future expected marginal utility;
                                        tuv(acat, 1) = (1 - fs) .* tumatr(vcat, dcat, pcat, scati, ecat, acat)...
                                            + fs .* tumatr(vcat, dcat, pcat, scati + 1, ecat, acat); % total utility next period discounted back to current period;
                                    else
                                        cv(acat, 1) = cmatr(vcat, dcat, pcat, scati, ecat, acat);
                                        tuv(acat, 1) = tumatr(vcat, dcat, pcat, scati, ecat, acat);
                                    end;
                                end;
                                
                                % calculate total income;
                                y = data.prwage(age - 50 + 1, 1) + pben + sben + calc.othincome(age - 25 + 1, vcat);
                                
                                % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                                [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                                
                                % calculate total utility by including utility from leisure;
                                for acat = 1 : gv.nacatlim;
                                    totu(acat, 1) = totu(acat, 1) + calc.tcats(tcat, age - 50 + 1) * calc.utilr(ecat, age - 50 + 1); % include utility from leisure;
                                    if vcat == 1;
                                        totutil(acat, 1) = totutil(acat, 1) + calc.tcats(tcat, age - 50 + 1) * calc.utilr(ecat, age - 50 + 1); % include utility from leisure;
                                    end;
                                end;
                                
                                % save optimal decisions for later calculations;
                                margucr(2, vcat, dcat, pcat, scat, ecat, :) = margu; % marginal utility current period, retired from career job; 2 indicates part time work;
                                totucr(2, vcat, dcat, pcat, scat, ecat, :) = totu; % total utility next current period, retired from career job; 2 indicates part time work;
                                if vcat == 1;
                                    savd(age - 50 + 1, 2, dcat, pcat, scat, ecat, :) = saving; % optimal savings, currently retired from career job; 2 indicates part time work;
                                    totud(age - 50 + 1, 2, dcat, pcat, scat, ecat, :) = totutil; % total utility if both spouses alive, currently retired from career job; 2 indicates part time work;     
                                end;
                                
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;

    % calculate optimal consumption, saving, marginal and total utility for returned full time work;
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 : calc.npcatlim; % DB pension benefits;
                        
                        % calculate DB pension benefits;
                        if dcat == 1 && age >= calc.earlypenage; % eligible for DB pension benefits;
                            pben = calc.pcats(pcat, 1) * exp(- gv.inflrate * (1 - gv.penadjrate) * (age - 50));
                        else % not eligible;
                            pben = 0;
                        end;
                        
                        for scat = 1 : gv.nscatlim; % SS benefits;
                            
                            % calculate SS benefits;
                            if calc.sserage <= age && age < 70;
                                
                                benchange = calc.benchanges(age - 62 + 1, vcat, scat); % returned full time work benefits change;
                                if benchange > 0;
                                    sbenf = calc.ssbenf(scat, 1) + benchange;
                                    fs = -1;
                                    
                                    if scat == 4;
                                        scati = 5;
                                    else
                                        for scati = scat + 1 : gv.nscatlim;
                                            if fs == -1 && sbenf < calc.ssbenf(scati, 1);
                                                fs = (scati - 1) + (sbenf - calc.ssbenf(scati - 1, 1)) ...
                                                    / (calc.ssbenf(scati, 1) - calc.ssbenf(scati - 1, 1));
                                            end;
                                        end;
                                    end;
                                    
                                    if fs == -1;
                                        fs = (scati - 2) + 0.99;
                                    end;
                                    scati = floor(fs);
                                    fs = fs - scati;
                                    
                                else
                                    scati = scat;
                                    fs = 0;
                                end;
                            else
                                scati = scat;
                                fs = 0;
                            end;
                            sben = calc.ssbens(age - 50 + 1, vcat, scat);
                            
                            for ecat = 1 : gv.necatlim;
                                
                                % initialization;
                                cv = zeros(gv.nacatlim, 1); % consumption equivalents of future expected marginal utility;
                                tuv = zeros(gv.nacatlim, 1); % total utility next period discounted back to current period;
                                
                                % interpolation;
                                for acat = 1 : gv.nacatlim;
                                    if fs ~= 0;
                                        cv(acat, 1) = (1 - fs) .* cmatr(vcat, dcat, pcat, scati, ecat, acat) ...
                                            + fs .* cmatr(vcat, dcat, pcat, scati + 1, ecat, acat);  % consumption equivalents of future expected marginal utility;
                                        tuv(acat, 1) = (1 - fs) * tumatr(vcat, dcat, pcat, scati, ecat, acat) ...
                                            + fs .* tumatr(vcat, dcat, pcat, scati + 1, ecat, acat);  % total utility next period discounted back to current period;
                                    else
                                        cv(acat, 1) = cmatr(vcat, dcat, pcat, scati, ecat, acat);
                                        tuv(acat, 1) = tumatr(vcat, dcat, pcat, scati, ecat, acat);
                                    end;
                                end;
                                
                                % calculate total income;
                                y = data.secwage(age - 50 + 1, 1) + pben + sben + calc.othincome(age - 25 + 1, vcat);

                                % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                                [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                                
                                % returned full time work; no utility from leisure;
                                % save optimal decisions for later calculations;
                                margucr(1, vcat, dcat, pcat, scat, ecat, :) = margu; % marginal utility current period, retired from career job; 1 indicates returned full time work;
                                totucr(1, vcat, dcat, pcat, scat, ecat, : ) = totu; % total utility current period, retired from career job; 1 indicates returned full time work;
                                if vcat == 1;
                                    savd(age - 50 + 1, 1, dcat, pcat, scat, ecat, :) = saving; % optimal savings, currently retired from career job; 1 indicates returned full time work;
                                    totud(age - 50 + 1, 1, dcat, pcat, scat, ecat, :) = totutil; % total utility if both spouses alive, currently retired from career job; 1 indicates returned full time work; 
                                end;
                                
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % choose between work and retirement if previously retired from career job;
    % initialization;
    muj = zeros(4, 1); % marginal utility by four types of working status;
    tuj = zeros(4, 1); % total utility by four types of working status;

    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for dcat = 1 : 2; % delayed DB pension;
                if dcat == 1 || (age >= calc.earlypenage && age < calc.delayedpenage);
                    for pcat = 1 :calc.npcatlim; % DB pension benefits;
                        for scat = 1 : gv.nscatlim; % SS benefits;
                            for ecat = 1 : gv.necatlim; % leisure preference;
                                for acat = 1 : gv.nacatlim; % asset;
                                    for job = 1 : 3; % job category besides career job;
                                        
                                        % pick corresponding marginal and total utility given current economic state;
                                        muj(job, 1) = margucr(job, vcat, dcat, pcat, scat, ecat, acat);
                                        tuj(job, 1) = totucr(job, vcat, dcat, pcat, scat, ecat, acat);
                                    end;
                                    
                                    % pick which type of job generates the largest total utility;
                                    jobmax = -1;
                                    tumax = 0;
                                    for job = 1 : 3;
                                        if jobmax == -1 || tuj(job, 1) > tumax;
                                            tumax = tuj(job, 1);
                                            jobmax = job;
                                        end;
                                    end;
                                    
                                    % save marginal and total utility from optimal decisions to marginal and total utility matrices next period (when calculating backward at age - 1), retired from career job, for later calculations;
                                    margufr(vcat, dcat, pcat, scat, ecat, acat) = muj(jobmax, 1);
                                    totufr(vcat, dcat, pcat, scat, ecat, acat) = tuj(jobmax, 1);
                                    
                                    % record retirement decision among job categories besides career job; 
                                    % retchoice = 2 is returned full time job; 3 is part time job; 4 is retired;
                                    if vcat == 1; 
                                        retchoice(age - 50 + 1, dcat, pcat, scat, ecat, acat) = jobmax + 1;
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
            
            % allow for delayed DB pension receipt;
            if calc.earlypenage < age && age == calc.delayedpenage;
                margufr(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim) = ...
                    margufr(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim);
                totufr(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim) = ...
                    totufr(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim);
            else
                if calc.earlypenage == age && age < calc.delayedpenage;
                    margufr(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim) = ...
                        margufr(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim);
                    totufr(vcat, 1, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim) = ...
                        totufr(vcat, 2, 1 : calc.npcatlim, 1 : gv.nscatlim, 1 : gv.necatlim, 1 : gv.nacatlim);
                end;
            end;
        end;
    end;
    
    % vcat = 3;
    % no job choices;
    % consider husband retired;
    if age > 62; 
        if calc.sserage <= agew && agew < 70;
            for scat = 1 : gv.nscatlim; % SS benefits;
                for acat = 1 : gv.nacatlim; % asset;
                    % save to marginal and total utility matrices next period (when calculating backward at age - 1), retired from career job, for later calculations;
                    margufr(3, 1, 1, scat, 1, acat) = margucr(3, 3, 1, 1, scat, 1, acat);
                    totufr(3, 1, 1, scat, 1, acat) = totucr(3, 3, 1, 1, scat, 1, acat);
                end;
            end;
        end;
    end;
    
    % choose between work and retirement if previously working in career job;
    pcat = floor(calc.initialpcat(age - 50 + 1, 1));
    fp = calc.initialpcat(age - 50 + 1, 1) - pcat;
    scat = floor(calc.initialscat(age - 50 + 1, 1));
    fs = calc.initialscat(age - 50 + 1, 1) - scat;
    
    % calculate transition matrix of asset grid points after adding DC pension wealth, previously working in career job;
    for lcat = 1 : calc.nlcatlim; % DC pension balance;
        if lcat > 1 && age == data.penjobend + 1; % eligible for DC pension;
            for acat = 1 : gv.nacatlim; % asset;
                
                % calculate asset after including DC pension balance;
                pwealth = calc.acats(acat, 1) + calc.lcats(lcat, 1);
                
                % find the corresponding grid point in asset grid;
                % interpolation;
                if acat < gv.nacatlim;
                    lacat = acat + 1;
                else
                    lacat = gv.nacatlim;
                end;
                while lacat < gv.nacatlim && pwealth > calc.acats(lacat, 1)
                    lacat = lacat + 1;
                end;
                lacat = lacat - 1;
                
                lacatsw(lcat, acat) = lacat + (pwealth - calc.acats(lacat, 1)) / (calc.acats(lacat + 1, 1) - calc.acats(lacat, 1));
            end;
        else % not eligible;
            if lcat == 1 || (age >= data.penjobend && age < calc.contribstartage);
                for acat = 1 : gv.nacatlim;
                    lacatsw(lcat, acat) = acat;
                end;
            end;
        end;
    end;
    
    % calculate transition matrix of asset grid points after adding DC pension wealth and early out window amount, previously working in career job;
    eowamt = data.eowamount(age - 50 + 1, 1); % early out window amounts;
    for lcat = 1 : calc.nlcatlim; % DC pension balance;
        if (lcat > 1 && age <= data.penjobend + 1 && age > calc.contribstartage) || (lcat == 1 && eowamt > 0); % eligible for DC pension;
            for acat = 1 : gv.nacatlim; % asset;
                
                % calculate asset after including DC pension balance and early out window amounts;
                pwealth = calc.acats(acat, 1) + calc.lcats(lcat, 1) + eowamt;
                
                % find the corresponding grid point in asset grid;
                % interpolation;
                if acat < gv.nacatlim;
                    lacat = acat + 1;
                else
                    lacat = gv.nacatlim;
                end;
                while lacat < gv.nacatlim && pwealth > calc.acats(lacat, 1);
                    lacat = lacat + 1;
                end;
                lacat = lacat - 1;
                
                lacatsr(lcat, acat) = lacat + (pwealth - calc.acats(lacat, 1)) / (calc.acats(lacat + 1, 1) - calc.acats(lacat, 1));
            end;
        else % not eligible;
            if lcat == 1;
                for acat = 1 : gv.nacatlim;
                    lacatsr(lcat, acat) = acat;
                end;
            end;
        end;
    end;
    
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for lcat = 1 : calc.nlcatlim; % DC pension balance;
                if lcat == 1 || (age <= data.penjobend + 1 && age > calc.contribstartage);
                    for ecat = 1 : gv.necatlim; % leisure preference;
                        for acat = 1 : gv.nacatlim; % asset;
                            
                            % interpolation;
                            lacat = floor(lacatsw(lcat, acat));
                            if lacat > gv.nacatlim - 1
                                lacat = gv.nacatlim - 1;
                            end;
                            fla = lacatsw(lcat, acat) - lacat;
                            
                            if age == data.penjobend + 1;
                                if fla > 0;
                                    
                                    mu0 = margucw(vcat, 1, ecat, lacat);
                                    mu1 = margucw(vcat, 1, ecat, lacat + 1);
                                    tu0 = totucw(vcat, 1, ecat, lacat);
                                    tu1 = totucw(vcat, 1, ecat, lacat + 1);
                                    
                                    if fla <= 1;
                                        muj(1, 1) = (1 - fla) * mu0 + fla * mu1; % marginal utility of staying in career job;
                                        tuj(1, 1) = (1 - fla) * tu0 + fla * tu1; % total utility of staying in career job;
                                    else
                                        if mu0 > mu1 && tu1 > tu0;
                                            dmu = mu0 - mu1;
                                            dtu = tu1 - tu0;
                                            muratio = mu0 / mu1;
                                            mufact = dmu / mu0;
                                            
                                            muj(1, 1) = mu1 / (1 + (fla - 1) * mufact);
                                            tuj(1, 1) = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                                        else
                                            muj(1, 1) = mu1;
                                            
                                            if tu1 > tu0;
                                                tuj(1, 1) = (1 - fla ) * tu0 + fla * tu1;
                                            else
                                                tuj(1, 1) = tu1;
                                            end;
                                        end;
                                    end;
                                else
                                    muj(1, 1) = margucw(vcat, 1, ecat, lacat);
                                    tuj(1, 1) = totucw(vcat, 1, ecat, lacat);
                                end;
                                
                            else
                                muj(1, 1) = margucw(vcat, lcat, ecat, acat);
                                tuj(1, 1) = totucw(vcat, lcat, ecat, acat);
                            end;
                            
                            lacat = floor(lacatsr(lcat, acat));
                            if lacat > gv.nacatlim - 1;
                                lacat = gv.nacatlim - 1;
                            end;
                            fla = lacatsr(lcat, acat) - lacat;
                            
                            % calculate marginal and total utility of quitting career job;
                            for choice = 1 : 3; % job category; 1 is returned full time work, 2 is part time work, 3 is retired;
                                if vcat < 3 || choice == 3;
                                    
                                    for utype = 1 : 2; % utility type; 1 is marginal utility, 2 is total utility;
                                        for pcati = pcat : pcat + 1; % DB pension benefits;
                                            if pcati == pcat || fp ~= 0;
                                                for scati = scat : scat + 1; % SS benefits;
                                                    if scati == scat || fs ~= 0;
                                                        if fla ~= 0;
                                                            if utype == 1; % marginal utility;

                                                                % interpolation: asset;
                                                                % pick corresponding marginal utility in the current period, currently retired from career job, given current economic status;
                                                                mu0 = margucr(choice, vcat, 1, pcati, scati, ecat, lacat);
                                                                mu1 = margucr(choice, vcat, 1, pcati, scati, ecat, lacat + 1);
                                                                
                                                                if fla <= 1;
                                                                    temp = (1 - fla) * mu0 + fla * mu1;
                                                                else
                                                                    if mu0 > mu1;
                                                                        dmu = mu0 - mu1;
                                                                        mufact = dmu / mu0;
                                                                        
                                                                        temp = mu1 / (1 + (fla - 1) * mufact);
                                                                    else
                                                                        temp = mu1;
                                                                    end;
                                                                end;
                                                            else % total utility;
                                                                
                                                                % pick corresponding total utility in the current period, currently retired from career job, given current economic status;
                                                                tu0 = totucr(choice, vcat, 1, pcati, scati, ecat, lacat);
                                                                tu1 = totucr(choice, vcat, 1, pcati, scati, ecat, lacat + 1);
                                                                
                                                                if fla <= 1;
                                                                    temp = (1 - fla) * tu0 + fla * tu1;
                                                                else
                                                                    mu0 = margucr(choice, vcat, 1, pcati, scati, ecat, lacat);
                                                                    mu1 = margucr(choice, vcat, 1, pcati, scati, ecat, lacat + 1);
                                                                    
                                                                    if mu0 > mu1 && tu1 > tu0;
                                                                        dmu = mu0 - mu1;
                                                                        dtu = tu1 - tu0;
                                                                        muratio = mu0 / mu1;
                                                                        mufact = dmu / mu0;
                                                                        
                                                                        temp = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                                                                    else
                                                                        if tu1 > tu0;
                                                                            temp = (1 - fla) * tu0 + fla * tu1;
                                                                        else
                                                                            temp = tu1;
                                                                        end;
                                                                    end;
                                                                end;
                                                            end;
                                                        else
                                                            if utype == 1;
                                                                temp = margucr(choice, vcat, 1, pcati, scati, ecat, lacat);
                                                            else
                                                                temp = totucr(choice, vcat, 1, pcati, scati, ecat, lacat);
                                                            end;
                                                        end;
                                                        
                                                        % interpolation: SS benefits;
                                                        if scati == scat;
                                                            temps(1, 1) = temp;
                                                        else
                                                            temps(2, 1) = temp;
                                                        end;
                                                    end;
                                                end;
                                                
                                                if fs ~= 0;
                                                    temp = (1 - fs) * temps(1, 1) + fs * temps(2, 1);
                                                else
                                                    temp = temps(1, 1);
                                                end;
                                                
                                                % interpolation: DB pension benefits;
                                                if pcati == pcat;
                                                    tempp(1, 1) = temp;
                                                else
                                                    tempp(2, 1) = temp;
                                                end;
                                            end;
                                        end;
                                        
                                        if fp ~= 0;
                                            util = (1 - fp) * tempp(1, 1) + fp * tempp(2, 1);
                                        else
                                            util = tempp(1, 1);
                                        end
                                        
                                        if utype == 1;
                                            muj(choice + 1, 1) = util; % marginal utility of job categories, retired from career job;
                                        else
                                            tuj(choice + 1, 1) = util;
                                        end;
                                    end;
                                end;
                            end;
                            
                            % pick which type of job generates the largest total utility;
                            jobmax = -1;
                            tumax = 0;
                            for job = 1 : 4
                                if jobmax == -1 || tuj(job, 1) > tumax;
                                    tumax = tuj(job, 1);
                                    jobmax = job;
                                end;
                            end;
                            
                            % save marginal and total utility from optimal decisions to marginal and total utility matrices next period (when calculating backward at age - 1), currently (age - 1) working in career job, for later calculations;
                            margufw(vcat, lcat, ecat, acat) = muj(jobmax, 1);
                            totufw(vcat, lcat, ecat, acat) = tuj(jobmax, 1);
                            
                            % record retirement decision among job categories besides career job; 
                            % mjchoice = 1 is staying in career job, 2 is returned full time job; 3 is part time job; 4 is retired;
                            if vcat == 1;
                                mjchoice(age - 50 + 1, lcat, ecat, acat) = jobmax;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate consumption and saving for age 25 to firstage - 1 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% individuals are surely working in career job if not dead in this period;
% when husband alive (vcat < 3):
% husband gets paid career job wage and accumulate asset and DC pension balance;
% do calculations given the following economic state: vcat, lcat, ecat, acat;
% when only wife alive (vcat == 3):
% do calculations given the following economic state: scat, acat;
% calculate SS benefits using ssbenr;
% optimal savings are saved in savw (saving when working in career job);  
% if saving is negative, it means households decumulate; that is, consume more than their income;
% (end of period asset + saving) * asset return shock is asset next period;

for age = calc.firstage - 1 : -1 : 25;

    % calculate expectated marginal and total utility with respect to asset returns, husband alive (firstage > 63);
    % 2 dimension interpolation;
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for ecat = 1 : gv.necatlim; % leisure preference;
                
                % initialization;
                % intermediate variables;
                mula = zeros(calc.nlcatlim, gv.nacatlim); % marginal utility next period by end of period asset and DC pension balance, given current economic state;
                tula = zeros(calc.nlcatlim, gv.nacatlim); % total utility next period by end of period asset and DC pension balance, given current economic state;
                mular = zeros(calc.nlcatlim, gv.nacatlim, gv.nrcatlim); % marginal utility next period by end of period asset, DC pension balance and asset return shock, given current economic state;
                tular = zeros(calc.nlcatlim, gv.nacatlim, gv.nrcatlim); % total utility next period by end of period asset, DC pension balance and asset return shock, given current economic state;
                
                for lcat = 1 : calc.nlcatlim; % DC pension balance;
                    if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                        for acat = 1 : gv.nacatlim; % asset;
                            
                            mu = 0; % expected marginal utility given end of period asset and DC pension balance;
                            tu = 0; % expected total utility given end of period asset and DC pension balance;
                            
                            for rcat = 1 : gv.nrcatlim; % asset return shocks;
                                
                                % interpolation: asset and DC pension balance;;
                                acati = floor(calc.rtransa(acat, rcat));
                                if acati >= gv.nacatlim;
                                    acati = gv.nacatlim - 1;
                                end;
                                fa = calc.rtransa(acat, rcat) - acati;
                                
                                if calc.nlcatlim > 1 && age <= data.penjobend && age > calc.contribstartage; % eligible for DC pension;
                                    
                                    lcati = floor(calc.rtransl(age - 25 + 1, lcat, rcat));
                                    if lcati >= calc.nlcatlim;
                                        lcati = calc.nlcatlim - 1;
                                    end;
                                    fl = calc.rtransl(age - 25 + 1, lcat, rcat) - lcati;
                                    
                                    if fa <= 1 && fl <= 1;
                                        
                                        if margufw(vcat, lcati, ecat, acati) <= 0 || margufw(vcat, lcati, ecat, acati + 1) <= 0 ...
                                                || margufw(vcat, lcati + 1, ecat, acati) <= 0 || margufw(vcat, lcati + 1, ecat, acati + 1) <= 0;
                                            fprintf('Interpolation Error: Bad mu in work section-living: %5d %2d %14.6e %14.6e %14.6e %14.6e',...
                                                calc.obs, age, margufw(vcat, lcati, ecat, acati), margufw(vcat, lcati, ecat, acati + 1),...
                                                margufw(vcat, lcati + 1, ecat, acati), margufw(vcat, lcati + 1, ecat, acati + 1));
                                            fprintf(' %2d %2d %2d %2d %2d %2d %2d', vcat, lcat, lcati, ecat, acat, acati, rcat);
                                            return;
                                        end;
                                        
                                        mur = (1 - fl) * ((1 - fa) * margufw(vcat, lcati, ecat, acati) ... 
                                            + fa * margufw(vcat, lcati, ecat, acati + 1)) ...
                                            + fl * ((1 - fa) * margufw(vcat, lcati + 1, ecat, acati) ...
                                            + fa * margufw(vcat, lcati + 1, ecat, acati + 1)); % expected marginal utility given end of period asset, DC pension balance and asset return shock;
                                        
                                        tur = (1 - fl) * ((1 - fa) * totufw(vcat, lcati, ecat, acati) ...
                                            + fa * totufw(vcat, lcati, ecat, acati + 1)) ...
                                            + fl * ((1 - fa) * totufw(vcat, lcati + 1, ecat, acati) ...
                                            + fa * totufw(vcat, lcati + 1, ecat, acati + 1)); % expected total utility given end of period asset, DC pension balance and asset return shock;
                                        
                                    else
                                        if rcat >= 3;
                                            mu0 = mular(lcat, acat, rcat - 2);
                                            mu1 = mular(lcat, acat, rcat - 1);
                                            tu0 = tular(lcat, acat, rcat - 2);
                                            tu1 = tular(lcat, acat, rcat - 1);
                                            
                                            if mu0 > mu1 && tu1 > tu0;
                                                dmu = mu0 - mu1;
                                                dtu = tu1 - tu0;
                                                muratio = mu0 / mu1;
                                                mufact = dmu / mu0;
                                                
                                                mur = mu1 / (1 + mufact);
                                                tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                                            else
                                                mur = mu1;
                                                
                                                if tu1 > tu0;
                                                    tur = 2 * tu1 - tu0;
                                                else
                                                    tur = tu1;
                                                end;
                                            end;
                                        else
                                            if lcat > calc.nlcatlim - 4;
                                                mu0 = mular(lcat - 2, acat, rcat);
                                                mu1 = mular(lcat - 1, acat, rcat);
                                                tu0 = tular(lcat - 2, acat, rcat);
                                                tu1 = tular(lcat - 1, acat, rcat);
                                                
                                                if mu0 > mu1 && tu1 > tu0;
                                                    dmu = mu0 - mu1;
                                                    dtu = tu1 - tu0;
                                                    muratio = mu0 / mu1;
                                                    mufact = dmu / mu0;
                                                    
                                                    mur = mu1 / (1 + (fl - 1) * mufact);
                                                    tur = tu1 + (dtu / log(muratio)) * log(1 + (fl - 1) * mufact);
                                                else
                                                    mur = mu1;
                                                    
                                                    if tu1 > tu0;
                                                        tur = (1 - fl) * tu0 + fl * tu1;
                                                    else
                                                        tur = tu1;
                                                    end;
                                                end;
                                            else
                                                fprintf('Error: fl problem: age %d rcat %d lcat %d', age, rcat, lcat);
                                                return;
                                            end;
                                        end;
                                        
                                        if rcat > 1;
                                            if mur > mular(lcat, acat, rcat - 1);
                                                mur = mular(lcat, acat, rcat - 1);
                                            end;
                                            
                                            if tur < tular(lcat, acat, rcat - 1);
                                                tur = tular(lcat, acat, rcat - 1);
                                            end;
                                        end;
                                        
                                        if lcat > 1;
                                            if mur > mular(lcat - 1, acat, rcat);
                                                mur = mular(lcat - 1, acat, rcat);
                                            end
                                            
                                            if tur < tular(lcat - 1, acat, rcat);
                                                tur = tular(lcat - 1, acat, rcat);
                                            end;
                                        end;
                                    end;
                                    
                                    mular(lcat, acat, rcat) = mur;
                                    tular(lcat, acat, rcat) = tur;
                                    
                                else % not eligible for DC pension;
                                    
                                    % interpolation: asset;
                                    if fa <= 1;
                                        
                                        mur = (1 - fa) * margufw(vcat, 1, ecat, acati) ...
                                            + fa * margufw(vcat, 1, ecat, acati + 1); % marginal utility next period by end of period asset and asset return shock;
                                        
                                        tur = (1 - fa) * totufw(vcat, 1, ecat, acati) ...
                                            + fa * totufw(vcat, 1, ecat, acati + 1); % total utility next period by end of period asset and asset return shock;
                                    
                                    else
                                        
                                        mu0 = mular(1, acat, rcat - 2);
                                        mu1 = mular(1, acat, rcat - 1);
                                        tu0 = tular(1, acat, rcat - 2);
                                        tu1 = tular(1, acat, rcat - 1);
                                        
                                        if mu0 > mu1 && tu1 > tu0;
                                            dmu = mu0 - mu1;
                                            dtu = tu1 - tu0;
                                            muratio = mu0 / mu1;
                                            mufact = dmu / mu0;
                                            
                                            mur = mu1 / (1 + mufact);
                                            tur = tu1 + (dtu / log(muratio)) * log(1 + mufact);
                                        else
                                            mur = mu1;
                                            
                                            if tu1 > tu0;
                                                tur = 2 * tu1 - tu0;
                                            else
                                                tur = tu1;
                                            end;
                                        end;
                                    end;
                                end;
                                
                                mu = mu + calc.rprob(rcat, 1) * mur; % expected marginal utility given end of period asset and DC pension balance;
                                tu = tu + calc.rprob(rcat, 1) * tur; % expected total utility given end of period asset and DC pension balance;
                            end;
                            
                            mula(lcat, acat) = mu;
                            tula(lcat, acat) = tu;
                        end;
                    end;
                end;
                
                % save to marginal and total utility matrices next period (when calculating backward at age - 1), currently working in career job, for later calculations;
                for lcat = 1 : gv.nlcatlim;
                    if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage);
                        for acat = 1 : gv.nacatlim;
                            margufw(vcat, lcat, ecat, acat) = mula(lcat, acat);
                            totufw(vcat, lcat, ecat, acat) = tula(lcat, acat);
                        end;
                    end;
                end;
            end;
        end;
    end;
    
    % calculate expectated marginal and total utility with respect to asset returns, husband deceased (vcat == 3);
    % consider husband retires from career job when he dies;
    % DC pension balance has been transfered to asset when he dies;
    % this part only has SS benefits and asset involved;
    % husband cannot die before age 62;
    if age > 62;
        for scat = 1 : gv.nscatlim; % SS benefits;
            
            % initialization;
            % intermediate variables;
            margu = zeros(gv.nacatlim, 1); % expected marginal utility by end of period asset;
            totu = zeros(gv.nacatlim, 1); % expected total utility by end of period asset;
            for acat = 1 : gv.nacatlim; % asset;
                
                mu = 0; % expected marginal utility given end of period asset;
                tu = 0; % expected marginal utility given end of period asset;
                
                for rcat = 1 : gv.nrcatlim; % asset return shock;
                    
                    % interpolation; 
                    acati = floor(calc.rtransa(acat, rcat));
                    if acati >= gv.nacatlim;
                        acati = gv.nacatlim - 1;
                    end;
                    fa = calc.rtransa(acat, rcat) - acati;
                    
                    mu0 = margufr(3, 1, 1, scat, 1, acati); % vcat, dcat, pcat, scat, ecat, acat;
                    mu1 = margufr(3, 1, 1, scat, 1, acati + 1);
                    tu0 = totufr(3, 1, 1, scat, 1, acati);
                    tu1 = totufr(3, 1, 1, scat, 1, acati + 1);
                    
                    if mu0 <= 0 || mu1 <= 0;
                        fprintf('Error: Bad mu in work section-deceased: %2d %14.6e %14.6e %2d %2d %2d %2d',...
                            age, mu0, mu1, scat, acat, acati, rcat);
                        return;
                    end;
                    
                    if fa <= 1;
                        mu = mu + calc.rprob(rcat, 1) * ((1 - fa) * mu0 + fa * mu1);
                        tu = tu + calc.rprob(rcat, 1) * ((1 - fa) * tu0 + fa * mu1);
                    else
                        if mu0 > mu1 && tu1 > tu0;
                            dmu = mu0 - mu1;
                            dtu = tu1 - tu0;
                            muratio = mu0 / mu1;
                            mufact = dmu / mu0;
                            
                            mu = mu + calc.rprob(rcat, 1) * mu1 / (1 + (fa - 1) * mufact);
                            tu = tu + calc.rprob(rcat, 1) * (tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact));
                        else
                            mu = mu + calc.rprob(rcat, 1) * mu1;
                            if tu1 > tu0;
                                tu = tu + calc.rprob(rcat, 1) * ((1 - fa) * tu0 + fa * tu1);
                            else
                                tu = tu + calc.rprob(rcat, 1) * tu1;
                            end;
                        end;
                    end;
                end;
                margu(acat, 1) = mu;
                totu(acat, 1) = tu;
            end;
            
            % save to marginal and total utility matrices next period (when calculating backward at age - 1), currently retired from career job, for later calculations;
            for acat = 1 : gv.nacatlim;
                margufr(3, 1, 1, scat, 1, acat) = margu(acat, 1);
                totufr(3, 1, 1, scat, 1, acat) = totu(acat, 1);
            end;
        end;
    end;
    
    % calculate expectated marginal and total utility with respect to survival status, husband alive;
    if age >= 62;
        
        % prepare for interpolation;
        scat = floor(calc.initialscat(age - 50 + 1, 1));
        fs = calc.initialscat(age - 50 + 1, 1) - scat;
        
        for lcat = 1 : calc.nlcatlim; % DC pension balance;
            if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage); % eligible for DC pension;
                for ecat = 1 : gv.necatlim; % leisure preference;
                    for acat = 1 : gv.nacatlim; % asset;
                        
                        % calculate asset after including DC pension balance;
                        wealth = calc.acats(acat, 1) + calc.lcats(lcat, 1);
                        
                        % interpolation;
                        if acat < gv.nacatlim;
                            acati = acat + 1;
                        else
                            acati = gv.nacatlim;
                        end;
                        while acati < gv.nacatlim && wealth > calc.acats(acati, 1);
                            acati = acati + 1;
                        end;
                        acati = acati - 1;
                        
                        fa = (wealth - calc.acats(acati, 1)) / (calc.acats(acati + 1, 1) - calc.acats(acati, 1));
                        
                        margufw0 = margufw(1, lcat, ecat, acat); % both spouses alive next period;
                        totufw0 = totufw(1, lcat, ecat, acat);
                        
                        margufw1 = margufw(2, lcat, ecat, acat); % only husband alive next period;
                        totufw1 = totufw(2, lcat, ecat, acat);
                        
                        % only wife alive next period;
                        % interpolation: SS benefits;
                        mu0 = (1 - fs) * margufr(3, 1, 1, scat, 1, acati) ...
                            +  fs * margufr(3, 1, 1, scat + 1, 1, acati);
                        mu1 = (1 - fs) * margufr(3, 1, 1, scat, 1, acati + 1) ...
                            +  fs * margufr(3, 1, 1, scat + 1, 1, acati + 1);
                        tu0 = (1 - fs) * totufr(3, 1, 1, scat, 1, acati) ...
                            +  fs * totufr(3, 1, 1, scat + 1, 1, acati);
                        tu1 = (1 - fs) * totufr(3, 1, 1, scat, 1, acati + 1) ...
                            +  fs * totufr(3, 1, 1, scat + 1, 1, acati + 1);
                        
                        % interpolation: asset;
                        if fa <= 1;
                            margufr2 = (1 - fa) * mu0 + fa * mu1; 
                            totufr2 = (1 - fa) * tu0 + fa * tu1;
                        else
                            if mu0 > mu1 && tu1 > tu0;
                                dmu = mu0 - mu1;
                                dtu = tu1 - tu0;
                                muratio = mu0 / mu1;
                                mufact = dmu / mu0;
                                
                                margufr2 = mu1 / (1 + (fa - 1) * mufact);
                                totufr2 = tu1 + (dtu / log(muratio)) * log(1 + (fa - 1) * mufact);
                            else
                                margufr2 = mu1;
                                
                                if tu1 > tu0;
                                    totufr2 = (1 - fa) * tu0 + fa * tu1;
                                else
                                    totufr2 = tu1;
                                end;
                            end;
                        end
                        
                        % rewrite marginal and total utility matrices next period, currently working in career job, for later calculations;
                        margufw(1, lcat, ecat, acat) ...
                            = calc.transrate(1, age - 25 + 1) * margufw0 ... 
                            + calc.transrate(2, age - 25 + 1) * margufw1 ... 
                            + calc.transrate(3, age - 25 + 1) * margufr2;
                        
                        totufw(1, lcat, ecat, acat) ...
                            = calc.transrate(1, age - 25 + 1) * totufw0 ... 
                            + calc.transrate(2, age - 25 + 1) * totufw1 ... 
                            + calc.transrate(3, age - 25 + 1) * totufr2; 
                    end;
                end;
            end;
        end;
    end;

    % calculate optimal consumption, saving, marginal and total utility, currently working in career job, husband living;
    % initialization;
    cv = zeros(gv.nacatlim, 1); % consumption equivalents of future expected marginal utility;
    tuv = zeros(gv.nacatlim, 1); % total utility next period discounted back to current period;
    
    for vcat = 1 : 2; % survival status;
        if vcat == 1 || age > 62;
            for lcat = 1 : calc.nlcatlim; % DC pension balance;
                if lcat == 1 || (age <= data.penjobend && age > calc.contribstartage); % eligible for DC pension;
                    for ecat = 1 : gv.necatlim; % leisure preference;
                        for acat = 1 : gv.nacatlim; % asset;
                            
                            % pick corresponding marginal and total utility in the current period, currently working in career job, given other current economic status;
                            mu = margufw(vcat, lcat, ecat, acat);
                            tu = totufw(vcat, lcat, ecat, acat);
                            
                            if mu <= 0;
                                fprintf('Error: Bad mu for work section-living consumption: %5d %2d %14.6e %2d %2d %2d %2d',...
                                    obs, age, mu, vcat, lcat, ecat, acat);
                                fprintf('\n fa %d', calc.firstage);
                                return;
                            end;

                            % recover corresponding consumption, given expected marginal utility;
                            % discount future total utility to current period;
                            if vcat == 1; % married couple adjustment;
                                cv(acat, 1) = sqrt2 * (mu / tp) ^ ( 1 / (calc.alpha - 1));
                                tuv(acat, 1) = tu / tp;
                            else
                                cv(acat, 1) = (mu * calc.srate(vcat - 1, age - 25 + 1) / tp)^ (1 / (calc.alpha - 1));
                                tuv(acat, 1) = tu * calc.srate(vcat - 1, age - 25 + 1) / tp;
                            end;
                            
                        end;
                        
                        % calculate SS benefis;
                        sben = calc.ssbenm(age - 25 + 1, vcat);
                        
                        % calculate total income;
                        y = data.ftwage(age - 25 + 1, 1) + sben + calc.othincome(age - 25 + 1, vcat);
                        
                        % insert all resources to GSModelinc to calculate optimal decisions at the current period;
                        [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist);
                        
                        % save optimal decisions for later calculations;   
                        margufw(vcat, lcat, ecat, :) = margu; % marginal utility next period (when calculating backward at age - 1), currently working in career job;
                        totufw(vcat, lcat, ecat, :) = totu; % total utility next period (when calculating backward at age - 1), currently working in career job;
                        
                        if vcat == 1;
                            savw(age - 25 + 1, lcat, ecat, :) = saving; % optimal savings, currently working in career job;
                            totuw(age - 25 + 1, lcat, ecat, :) = totutil; % total utility if both spouses alive, currently working in career job; 
                        end;
                        
                        % at end of career job, transfer DC pension balance to financial assets; 
                        if age == data.penjobend + 1;
                            for lcati = 1 : calc.nlcatlim; % DC pension balance;
                                for acat = 1 : gv.nacatlim; % asset;
                                    
                                    % calculate asset after including DC pension balance;
                                    pwealth = calc.acats(acat, 1) + calc.lcats(lcat, 1);
                                    
                                    % interpolation;
                                    if acat < gv.nacatlim;
                                        lacat = acat + 1;
                                    else
                                        lacat = gv.nacatlim;
                                    end;
                                    while lacat < gv.nacatlim && pwealth > calc.acats(lacat, 1);
                                        lacat = lacat + 1;
                                    end;
                                    lacat = lacat - 1;
                                    
                                    fla = (pwealth - calc.acats(lacat, 1)) / (calc.acats(lacat + 1, 1) - calc.acats(lacat, 1));
                                    
                                    if fla == 0;                                        
                                        margu(acat, 1) = margufw(vcat, 1, ecat, lacat);
                                        totu(acat, 1) = totufw(vcat, 1, ecat, lacat);
                                    else
                                        mu0 = margufw(vcat, 1, ecat, lacat);
                                        mu1 = margufw(vcat, 1, ecat, lacat + 1);
                                        tu0 = totufw(vcat, 1, ecat, lacat);
                                        tu1 = totufw(vcat, 1, ecat, lacat + 1);
                                        
                                        if fla <= 1;
                                            margu(acat, 1) = (1 - fla) * mu0 + fla * mu1;
                                            totu(acat, 1) = (1 - fla) * tu0 + fla * tu1;
                                        else
                                            if mu0 > mu1 && tu1 > tu0;
                                                dmu = mu0 - mu1;
                                                dtu = tu1 - tu0;
                                                muratio = mu0 / mu1;
                                                mufact = dmu / mu0;
                                                
                                                margu(acat, 1) = mu1 / (1 + (fla - 1) * mufact);
                                                totu(acat, 1) = tu1 + (dtu / log(muratio)) * log(1 + (fla - 1) * mufact);
                                            else
                                                margu(acat, 1) = mu1;
                                                
                                                if tu1 > tu0;
                                                    totu(acat, 1) = (1 - fla) * tu0 + fla * tu1;
                                                else
                                                    totu(acat, 1) = tu1;
                                                end;
                                            end;
                                        end;
                                    end;
                                end;
                                
                                % save to marginal and total utility matrices next period (when calculating backward at age - 1), currently working in career job, for later calculations;
                                margufw(vcat, lcati, ecat, :) = margu;
                                totufw(vcat, lcati, ecat, :) = totu;
                            end;
                        end;
                    end;
                end
            end;
        end;
    end;

    % calculate optimal consumption, saving, marginal and total utility, currently working in career job, husband deceased;
    vcat = 3;
    if age > 62;
        for scat = 1 : gv.nscatlim; % SS benefits;
            for acat = 1 : gv.nacatlim; % asset;
                
                if mu <= 0;
                    fprintf('Error: Bad mu for work section-deceased consumption: %2d %14.6e %14.6e %2d %2d', ...
                        age, mu, scat, acat);
                    return;
                end;
                
                % pick corresponding marginal and total utility in the current period, currently retired from career job (died), given other current economic status;
                mu = margufr(3, 1, 1, scat, 1, acat);
                tu = totufr(3, 1, 1, scat, 1, acat);
                
                % recover corresponding consumption, given expected marginal utility;
                % discount future total utility to current period;
                cv(acat, 1) = (mu * calc.srate(vcat - 1, age - 25 + 1) / tp)^( 1 / (calc.alpha - 1));
                tuv(acat, 1) = tu * calc.srate(vcat - 1, age - 25 + 1) / tp;
                
            end;
            
            % calculate total income;
            y = calc.ssbenr(age - 50 + 1, 3, scat) + calc.othincome(age - 25 + 1, vcat);
            
            % insert all resources to GSModelinc to calculate optimal decisions at the current period;
            [margu, totu, saving, totutil] = GSModelinc(vcat, y, cv, tuv, gv, calc, arglist); %#ok<NASGU,ASGLU>
            
            % save to marginal and total utility matrices next period (when calculating backward at age - 1), currently retired from career job, for later calculations;
            margufr(3, 1, 1, scat, 1, :) = margu;
            totufr(3, 1, 1, scat, 1, :) = totu;
            
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% assign the values to structure variable calc for future calculations %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
calc.totuw = totuw;
calc.totud = totud;
calc.savw = savw;
calc.savd = savd;
calc.savr = savr;
calc.mjchoice = mjchoice;
calc.retchoice = retchoice;
