% ----------------------------------------------------------------------- %
% % calcprep function                                                   % %
% % calculate utility, savings, and job choices for the model           % %
% ----------------------------------------------------------------------- %

function [calc] = calcprep(gv, data, calc)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get survival rates and transition rates %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
srate = zeros(2, 75); % survival probabilities;
transrate = zeros(3, 75); % survival status transfer probabilities;

% surv and spsurv are how many of the original 100,000 individuals are still alive given birth year and gender;
surv = squeeze(gv.survtable(data.birthyear - 1900 + 1, 1, :)); % number of survivors of male by age (0 - 100), given birthyear;
spsurv = squeeze(gv.survtable(data.spbirthyear-1900 + 1, 2, :)); % number of survivors of female by age (0 - 100), given birthyear;

% treat the husband's survival rates as certain until he reaches age 62; 
% he could die between age 62 and 63, and later;
% treat the wife's survival rates as certain until she reaches age 62; 
% she could die between age 62 and 63, and later;
for age = 1 : 62;
    surv(age, 1) = surv(63, 1); 
    spsurv(age, 1) = spsurv(63, 1); 
end;

% calculate survival and transition probabilities;
% age + 2 is used in the calculation because surv / spsurv starts from age 0;
% transrate(1, age - 25 + 1): survival status transfer probabilities from both alive at age to both alive at age + 1, by age; 
% transrate(2, age - 25 + 1): survival status transfer probabilities from both alive at age to only husband alive at age + 1, by age; 
% transrate(3, age - 25 + 1): survival status transfer probabilities from both alive at age to only wife alive at age + 1, by age; 
for age = 25 : 99;
    
    spage = age + data.birthyear - data.spbirthyear; % wife's age;
    
    % if out of survival range or irregular survivor numbers, assign a very smaller survival probability;
    if surv(age + 1, 1) > 0 && surv(age + 2, 1) > 0; 
        srate(1, age - 25 + 1) = surv(age + 2, 1) / surv(age + 1, 1); % survival probabilities of the husband, by age; 
    else
        srate(1, age - 25 + 1) = 0.001; % survival probabilities of the husband, by age; 
    end;
    
    if spage < 1; % no wife;
        srate(2, age - 25 + 1) = 1;
    else
        if spage < 119;
            if spsurv(spage + 1, 1) > 0 && spsurv(spage + 2, 1) > 0;
                srate(2, age - 25 + 1) = spsurv(spage + 2, 1) / spsurv(spage + 1, 1); % survival probabilities of the wife, by age;
            else
                srate(2, age - 25 + 1) = 0.001; % survival probabilities of the wife, by age;
            end;
        else
            srate(2, age - 25+1) = 0.001; % survival probabilities of the wife, by age;
        end;
    end;
    
    transrate(1, age - 25 + 1) = srate(1, age - 25 + 1) * srate(2, age - 25 + 1); % transition probability from both alive to both alive;
    transrate(2, age - 25 + 1) = srate(1, age - 25 + 1) * (1 - srate(2, age - 25 + 1)); % transition probability from both alive to only husband alive;
    transrate(3, age - 25 + 1) = (1 - srate(1, age - 25 + 1)) * srate(2, age - 25 + 1); % transition probability from both alive to only wife alive;

end;


%%%%%%%%%%%%%%%%%%%%
% discretize asset %
%%%%%%%%%%%%%%%%%%%%
% what is the logic in this section; ???
% initialization;
acats = zeros(gv.nacatlim, 1); % asset grid;  

% total asset calculation method 1;
% sum up earnings from age 25 to the minimum of age 69 or age in 1992;
% include spouses' earnings and DC pensions if applicable; 
% asset increases by average asset return each year;
totearnings = 0; 
for age = 25 : min(69, 1992 - data.birthyear); % age;
    
    totearnings = totearnings * (1 + gv.realrateg); % asset grows at average asset return each year;
    
    % include largest possible wage (career job wage) and other income;
    totearnings = totearnings + data.ftwage(age - 25 + 1, 1) + data.othincome(age - 25 + 1, 1); 
    
    % if married, include wife's wage and DC pension balance when she retires;
    if data.spbirthyear > 0; % married couples;
        spage = age + data.birthyear - data.spbirthyear; % spouse's age;
        if spage >= 25 && spage < 70;
            totearnings = totearnings + data.spwage(spage - 25 + 1, 1); % include her wage;
        end;
        if spage == data.sppenstart;
            totearnings = totearnings + data.spplumpsum; % include her DC pension balance when she retires;
        end;
    end;
end;

% total asset calculation method 2;
% sum up all earnings from age 70 back to the maximum of age 25 or age in 1992;
% include the largest possible DC pension balance (by assuming he retires at the largest age possible, 70);
% include spouses' earnings and DC pensions if applicable; 
% asset increases by average asset return each year;
earnings = data.plumpsum(70 - 50 + 1, 1); % DC pension balance if works in career job until age 70; 
for age = 70 : -1 : max(25, 1992 - data.birthyear) + 1;  % age;
    
    earnings = earnings / (1 + gv.realrateg); % asset increases by average asset return;
    
    % include largest possible wage (career job wage) and other income;
    % original codes out of boundary; ???
    % people cannot work in their career job at age 70, use wage from part time work instead; programming error or on purpose;
    % revised codes;
    if age == 70; 
        earnings = earnings + data.prwage(1, 1) + data.othincome(age - 25 + 1, 1); 
    else
        earnings = earnings + data.ftwage(age - 25 + 1, 1) + data.othincome(age - 25 + 1, 1); 
    end;
    
    % if married, include wife's wage and DC pension balance when she retires;
    if data.spbirthyear > 0; % married couples;
        spage = age + data.birthyear - data.spbirthyear; % spouse's age;
        if spage >= 25 && spage < 70;
            earnings = earnings + data.spwage(spage - 25 + 1, 1); % include her wage;
        end
        if spage == data.sppenstart;
            earnings = earnings + data.spplumpsum; % include her DC pension balance when she retires;
        end;
    end;
end;

% define asset at largest asset grid point is the larger of the above two calculations; 
acats(gv.nacatlim, 1) = max(totearnings, earnings);

% discretize asset grid, uneven distribution;
for i = gv.nacatlim: -1: 5;
    acats(i - 1, 1) = acats(i, 1) / 1.25;
end;
for i = 4 : -1 : 1;
    acats(i, 1) = (i - 1) * acats(5, 1) / 4;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% discretize leisure preference shock %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization; 
ecats = zeros(gv.necatlim, 1); % leisure preference shock grid;

% at most +/-3 standard deviation away from mean; even distribution;
ecats(1, 1) = -3; 
ecats(gv.necatlim, 1) = 3;
ecats(2 : gv.necatlim - 1, 1) = ecats(1, 1) + (1 : gv.necatlim - 2)' .* (ecats(gv.necatlim, 1) - ecats(1, 1)) / (gv.necatlim - 1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate transition probabilities for epsilon %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization; 
etrans = zeros(gv.necatlim, gv.necatlim); % transition probability matrix of epsilon (the error term in weighting function), AR(1);
 
% if eps(t) = erho * eps(t - 1) + mu;
% E(eps(t)) = erho * eps(t - 1);
% var(eps(t)) =  1 / (1 - erho ^ 2) * var(mu);
for ecat = 1 : gv.necatlim; % initial epsilon, eps(t - 1);
    etrans(ecat, 1) = 0;  
    condmean = calc.erho * ecats(ecat, 1); % conditional mean given eps(t - 1);
    condstddev = sqrt(1 - calc.erho * calc.erho); % conditional standard deviation given eps(t - 1);
    for ecat2 = 2 : gv.necatlim; % epsilon after shock, eps(t);
        % erfp calculates the cumulative standard normal distribution probability; 
        % could be replaced by Matlab built in function normcdf(x, mean, std);
        etrans(ecat, ecat2) = erfp((((ecats(ecat2, 1) + ecats(ecat2 - 1, 1)) / 2) - condmean) / condstddev);
        etrans(ecat, ecat2 - 1) = etrans(ecat, ecat2) - etrans(ecat, ecat2 - 1);
    end;
    etrans(ecat, gv.necatlim) = 1 - etrans(ecat, gv.necatlim);
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% discretize asset return shock %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
rnorm = zeros(gv.nrcatlim, 1); % asset return shock grid;

% at most +/-2 standard deviation away from mean; even distribution;
rnorm(1, 1) = -2;
rnorm(gv.nrcatlim, 1) =  2;
rnorm(2 : gv.nrcatlim - 1, 1) = rnorm(1, 1) + (1 : gv.nrcatlim - 2)' .* (rnorm(gv.nrcatlim, 1) - rnorm(1, 1))/(gv.nrcatlim - 1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate probabilities for asset return shock %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
rprob = zeros(gv.nrcatlim, 1); % probabilities of the corresponding asset return shocks;

% iid asset return shock each period;
rprob(1, 1) = 0;
for rcat = 1 : gv.nrcatlim - 1;
    % erfp calculates the cumulative standard normal distribution probability; 
    % could be replaced by built in function normcdf(x, mean, std);
    rprob(rcat + 1, 1) = erfp((rnorm(rcat + 1, 1) + rnorm(rcat, 1)) / 2);
    rprob(rcat, 1) = rprob(rcat + 1, 1) - rprob(rcat, 1);
end;
rprob(gv.nrcatlim, 1) = 1 - rprob(gv.nrcatlim, 1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate asset transitions after asset return shock %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
rtransa = zeros(gv.nacatlim, gv.nrcatlim); % transition matrix of asset grid points after asset return shock;

% calculate how asset grid point changes after receiving different asset return shocks;
for acat = 1 : gv.nacatlim; % asset;
    for rcat = 1 : gv.nrcatlim; % asset return shock;
        
        if acat == 1; % if initial asset is 0 (the 1st grid), the next period asset is surely 0 (the 1st grid) no matter what asset return shock is;
            
            rtransa(acat, rcat) = 1; 
            
        elseif acat > 1; % if initial asset is not 0;
            
            assets = (1 + gv.realratea + rnorm(rcat, 1) * gv.realratesd) * acats(acat, 1); % asset increases by mean asset return + exogenous shock;
            
            % find the corresponding asset grid point after receiving asset return shock;
            % interpolation;
            acat2 = 1; 
            while acat2 < gv.nacatlim && assets > acats(acat2, 1);
                acat2 = acat2 + 1;
            end;
            if acat2 > 1;
                acat2 = acat2 - 1;
            end;
            rtransa(acat, rcat) = acat2 + (assets - acats(acat2, 1)) / (acats(acat2 + 1, 1) - acats(acat2, 1));
            
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% find ages that start to receiving early / delayed DB pension benefits %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% test whether eligible for DB pension benefits from firstage to lastage;
% variable somepens is 1 if individual could receive DB pension at some age, 0 otherwise;
% variable somezero is 1 if individual cannot recieve DB pension at some age, 0 otherwise;
% it is possible both somepens and somezero are 1;
somepens = 0; 
somezero = 0; 
for age = calc.firstage : calc.lastage + 1; 
    if data.pensben(age - 50 + 1, 1) > 0;
        somepens = 1;
    else
        somezero = 1;
    end;
end;      

% earlypenage: earliest age DB pension benefits can be immediately collected;
% delayedpenage: earliest age delayed DB pension benefits can be collected;
% adjust pension benefits if necessary;
if somepens > 0;
    
    earlypenage = 0; % earliest age DB pension benefits can be immediately collected;
    for age = calc.firstage : calc.lastage + 1;
        if earlypenage == 0 && age == data.penstart(age - 50 + 1, 1);
            earlypenage = age;
        end;
    end;
    
    % setup boundary;
    % control earlypenage as calc.lastage + 2 if cannot collect DB pension benefits immediately until lastage + 1;
    if earlypenage == 0;
        earlypenage = calc.lastage + 2; 
    end;
    
    if earlypenage > calc.firstage;
        delayedpenage = 99; % earliest age delayed DB pension benefits can be collected;
        for age = calc.firstage : earlypenage - 1;
            if delayedpenage > data.penstart(age - 50 + 1, 1);
                delayedpenage = data.penstart(age - 50 + 1, 1);
            end;
        end;
        if delayedpenage < earlypenage || delayedpenage == 99;
            delayedpenage = earlypenage;
        end;
    else
        delayedpenage = earlypenage;
    end;
    
    % adjust deferred DB benefits to the amounts as if the individual starts to recieve DB benefits at delayedpenage; 
    % adjustment factor is provided by function adjben;
    % input original start age and new start age, adjben will do actuarially adjustment for DB pension benefits;
    % pay attention data.pensben is revised;
    for age = calc.firstage : earlypenage - 1; 
        if data.penstart(age - 50 + 1, 1) ~= delayedpenage;
            data.pensben(age - 50 + 1, 1) = data.pensben(age - 50 + 1, 1) * adjben(data.penstart(age - 50 + 1, 1), delayedpenage, gv, calc);
        end;
    end;
    
    % adjust if not currently collectable (if the individual retires after earlypenage, but can not collect the DB benefits immediately); 
    % pay attention data.pensben is revised;
    for age = earlypenage : calc.lastage + 1; 
        if age ~= data.penstart(age - 50 + 1, 1);
            data.pensben(age - 50 + 1, 1) = data.pensben(age - 50 + 1, 1) * adjben(data.penstart(age - 50 + 1, 1), age, gv, calc);
        end;
    end;
    
else
    
    earlypenage = 99;
    delayedpenage = 99;

end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% discretize DB pension benefits %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
initialpcat = zeros(21, 1); % initial DB pension grid point if the individual retires and starts to receive DB pension benefits between 50 and 70;

% The pensions associated with the pcats categories are assumed to decline by the rate gv.inflrate * (1 - gv.penadjrate) every year to reflect that pensions are incompletely inflation indexed;  
% The 1 and 4 categories are the lowest and highest possible DB pensions;
% The amounts of pcats are as of age 50 if the individual would collect them at that age;
% The amounts need to be adjusted to appropriate retirement ages in the calculations;
% note: gv.npcatlim is replaced by calc.npcatlim in later calculations;
% calc.npcatlim is 1 if the individual is not eligible for DB pension benefits;
if somepens == 0; % no DB pension;
    
    calc.npcatlim = 1; % only 1 grid point;
    pcats = 0; % DB pension is always 0;
    for age = firstage : lastage + 1;
        initialpcat(age - 50 + 1, 1) = 1; % always the 1st grid (0 DB pension benefits);
    end;
    
elseif somepens == 1; % eligible for DB pension benefits;
    
    % initialization;
    calc.npcatlim = gv.npcatlim; % regular size of grid points;
    pcats = zeros(calc.npcatlim, 1); % DB pension benefits grid;
    
    maxpen = 0; % largest possible DB pension benefit discounted back to age 50;
    minpen = 0; % smallest possible DB pension benefit discounted back to age 50;
    for age = calc.firstage : calc.lastage + 1;
        if data.pensben(age - 50 + 1, 1) > 0;
            adjpen = data.pensben(age - 50 + 1, 1) * exp(gv.inflrate * (1 - gv.penadjrate) * (data.penstart(age - 50 + 1, 1) - 50)); % continuous time discount factor;
            if minpen == 0 || adjpen < minpen;
                minpen = adjpen;
            end;
            if adjpen > maxpen;
                maxpen = adjpen;
            end;
        end;
    end;
                                    
    if somezero == 1; % need to include 0 in the grid;
        
        pcats(1, 1) = 0; % DB pension benefits at the 1st grid point;
        pcats(2, 1) = 0.99 * minpen; % DB pension benefits at the 2nd grid point;
        pcats(calc.npcatlim, 1) = 1.01 * maxpen; % DB pension benefits at the last grid point;
        
        % discretize DB pension benefits grid, uneven distribution;
        for pcat = 3 : calc.npcatlim - 1;
            pcats(pcat, 1) = pcats(2, 1) * (pcats(calc.npcatlim, 1) / pcats(2, 1)) ^ ((pcat - 2) / (calc.npcatlim - 2));
        end;
                                    
    elseif somezero == 0; % DB pension benefits bigger than 0 at any retirement age;
        
        pcats(1, 1) = 0.99 * minpen; % DB pension benefits at the 1st grid point;
        pcats(calc.npcatlim, 1) = 1.01 * maxpen; % DB pension benefits at the last grid point;
        
        % discretize DB pension benefits grid, uneven distribution;
        for pcat = 2 : calc.npcatlim - 1;
            pcats(pcat, 1) = pcats(1, 1) * ((pcats(calc.npcatlim, 1) / pcats(1, 1))) ^ ((pcat - 1) / (calc.npcatlim - 1));
        end;
        
    end;
    
    % calculate initial pension grid point if the individual starts receiving DB pension benefits at ages from calc.firstage to calc.lastage + 1;
    for age = calc.firstage : calc.lastage + 1;
        if data.pensben(age - 50 + 1, 1) > 0;
            
            adjpen = data.pensben(age - 50 + 1, 1) * exp(gv.inflrate * (1 - gv.penadjrate) * (data.penstart(age - 50 + 1, 1) - 50)); % continuous time discount factor;
            
            % find out corresponding grid point when the individual starts receiving DB pension benefits;
            % interpolation;
            pcat = 1; 
            while pcat < calc.npcatlim && adjpen > pcats(pcat, 1); 
                pcat = pcat + 1;
            end;
            if pcat > 1;
                pcat = pcat - 1;
            end;
            initialpcat(age - 50 + 1, 1) = pcat + (adjpen - pcats(pcat, 1)) / (pcats(pcat + 1, 1) - pcats(pcat, 1));

            % setup boundary;
            if initialpcat(age - 50 + 1, 1) > calc.npcatlim; 
                initialpcat(age - 50 + 1, 1) = calc.npcatlim; 
            end;
            
        else
            
            initialpcat(age - 50 + 1, 1) = 1; % no DB pension benefits;
            
        end;
    end;
    
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate DC pension balance %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% inititialization;
dccontrib = zeros(45, 1); % DC contribution amounts by age (25 - 70);
contribstartage = 99; % age start to accumulate DC pension account;

% DC contribution between age 50 and 69; 
% do two parts calculation because data.plumpsum only available from 50 to 70; 
% take difference to find out how much the husband contributes to DC account;
% find the first age that has a DC pension balance (at the previous age, DC account balance is 0);
% where does 1.022 (DC pension growth rate) come from; ???
for age = 50 : 69;
    
    % take difference to find out how much the husband contributes to DC account;
    dccontrib(age - 25 + 1, 1) = data.plumpsum((age + 1) - 50 + 1, 1) / 1.022 - data.plumpsum(age - 50 + 1, 1); 
    
    % setup boundary;
    if dccontrib(age - 25 + 1, 1) < 0;
        dccontrib(age - 25 + 1, 1) = 0; 
    end;
    
    % find the first age that has a DC pension balance (at the previous age, DC account balance is 0);
    if dccontrib(age - 25 + 1, 1) > 0 && age < contribstartage; 
        contribstartage = age;
    end;
end;

% DC contribution between age 25 and 49;
% DC pension balance is bigger than 0 at age 50 indicates the husband contributes before age 50;
% otherwise all contribution amounts before age 50 are 0;
% assume DC pension contribution rate (DC contribution as a percentage of career job wage) is the same as the last year of contribution;
if data.plumpsum(1, 1) > 0; % the individual contributed before 50;
    
    % calculate DC pension contribution rate as the last year contribution rate;
    contribrate = 0; 
    for age = 50 : 69;
        if contribrate == 0 && dccontrib(age - 25 + 1, 1) > 0 && data.ftwage(age - 25 + 1, 1) > 0;
            contribrate = dccontrib(age - 25 + 1, 1) / data.ftwage(age - 25 + 1, 1);
        end;
    end;
    
    % distribute DC pension account balance at age 50 back to previous years, assuming the same DC contribution rate;
    % if there is leftover balance before starting to work, increase contribution rate using adjfactor;
    % contribution rate is always smaller than 25% of career job wage before age 50;
    if contribrate > 0;
        
        accum = data.plumpsum(1, 1) / 1.022; % accumulated DC balance;
        adjfactor = 0;
        for age = 49 : -1 : 25;
            if accum > 0; % there is leftover balance;
                if data.ftwage(age - 25 + 1, 1) > 0;
                    
                    % distribute DC account balance;
                    if accum > contribrate * data.ftwage(age - 25 + 1, 1);
                        dccontrib(age - 25 + 1, 1) = contribrate * data.ftwage(age - 25 + 1, 1); % DC pension contribution by age;
                        accum = (accum - dccontrib(age - 25 + 1, 1)) / 1.022;
                    else
                        dccontrib(age - 25 + 1, 1) = accum;
                        accum = 0;
                    end;
                    contribstartage = age;
                    
                else % there is leftover balance;
                    
                    % increase contribution rate;
                    adjfactor = 1 + (accum * 1.022 ^ (50 - age)) / data.plumpsum(1, 1); 
                    accum = 0;
                    
                end;
            end;
        end;
        
        % increase DC contribution rates;
        if adjfactor > 0;
            for age = 25 : 49;
                dccontrib(age - 25 + 1, 1) = dccontrib(age - 25 + 1, 1) * adjfactor;
                
                % setup boundary; at most contribute 25% of career job wage;
                if dccontrib(age - 25 + 1, 1) > 0.25 * data.ftwage(age - 25 + 1, 1);
                    dccontrib(age - 25 + 1, 1) = 0.25 * data.ftwage(age - 25 + 1, 1); 
                end;
            end;
        end;
        
    end;
    
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set up DC pension grid and transition matrix %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% note: gv.nlcatlim is replace by calc.nlcatlim;
% calc.nlcatlim is 1 if not eligible for DC pension;
if contribstartage < 99; % DC pension avaialbe;

    calc.nlcatlim = gv.nlcatlim; % regular size of grid points;
    
    % initialization;
    lcats = zeros(calc.nlcatlim, 1); % DC pension grids;
    rtransl = zeros(45, calc.nlcatlim, gv.nrcatlim); % transition matrix of DC pension grid point after asset return shock, by age(25 - 69);

    % calculate largest possible DC pension balance;
    % assume the balance always receives extremely high asset return shock (2.5 standard deviation higher than mean);
    maxaccum = 0; % largest amount of DC balance;
    for age = 25 : calc.lastage - 1;
        if dccontrib(age - 25 + 1, 1) > 0;
            maxaccum = maxaccum + dccontrib(age - 25 + 1, 1) * (1 + gv.realratea + 2.5 * gv.realratesd) ^ sqrt(calc.lastage - age); % each period give the largest possible asset return on DC balance (mean + 2.5 * std);
        end;
    end;
    lcats(calc.nlcatlim, 1) = maxaccum; % DC pension balance at the last grid point;
    
    % discretize DC pension balance, uneven grid;
    for lcat = calc.nlcatlim - 1 : -1 : 5;
        lcats(lcat, 1) = lcats(lcat + 1, 1) / 1.4; 
    end;
    for lcat = 4 : -1 : 1; 
        lcats(lcat, 1) = (lcat - 1) * lcats(5, 1) / 4;
    end;
    
    % set up transition matrixs among DC pension balance grid points, by age (25 - 69);
    for age = contribstartage + 1 : calc.lastage; % age;
        for lcat = 1 : calc.nlcatlim; % DC pension balance;
            for rcat = 1 : gv.nrcatlim; % asset return shock;
                
                % calculated DC pension balance next period after new contribution and asset return shock;
                accum = (lcats(lcat, 1) + dccontrib((age - 1) - 25 + 1, 1)) * (1 + gv.realratea + rnorm(rcat, 1) * gv.realratesd);
                
                % search for the corresponding grid;
                % interpolation;
                lcat2 = 1; 
                while lcat2 < calc.nlcatlim && accum > lcats(lcat2, 1);
                    lcat2 = lcat2 + 1;
                end;
                if lcat2 > 1;
                    lcat2 = lcat2 - 1;
                end;
                rtransl(age - 25 + 1, lcat, rcat) = lcat2 + (accum - lcats(lcat2, 1)) / (lcats(lcat2 + 1, 1) - lcats(lcat2, 1));
                
            end;
        end;
    end;
    
elseif contribstartage == 99; % not eligible for DC pension;
    
    calc.nlcatlim = 1; % only 1 grid point;
    lcats(1, 1) = 0; % no DC balance;
    
    % initialization;
    rtransl = zeros(45, calc.nlcatlim, gv.nrcatlim); % transition matrix of DC pension grid point after asset return shock, by age(25 - 69);
    
    % transition matrix always the 1st grid (0 balance);
    for age = contribstartage + 1 : calc.lastage;
        for lcat = 1 : calc.nlcatlim;
            for rcat = 1 : gv.nrcatlim;
                rtransl(age - 25 + 1, lcat, rcat) = 1;
            end;
        end;
    end;
    
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculates social security categories and benefit amounts %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
calc = sscalc(gv, data, calc);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate spouse earnings, pension, and other income %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% wife's income and pensions are considered as other income for the household;
% initialization;
othincome = zeros(75, 3); % other income by age (25 - 99) and survival status (1 - 3);
sppensben = 0; % wife's DB pension benefits;
if data.spbirthyear > 0; % married couples;
    for age = 25 : 99;
        
        % include spouse's income first;
        % if both spouses alive (vcat == 1), or only wife alive (vcat == 3);
        spage = age + data.birthyear - data.spbirthyear; % wife's age;
        if spage >= 25 && spage < 70;
            othincome(age - 25 + 1, 1) = othincome(age - 25 + 1, 1) + data.spwage(spage - 25 + 1, 1);
            othincome(age - 25 + 1, 3) = othincome(age - 25 + 1, 3) + data.spwage(spage - 25 + 1, 1);
        end;
        
        % include spouse's pensions;
        % no variation in spouse's DB pension benefit; 
        % she always retires at the same age and starts to collect DB pension at the same time;
        if data.sppenstart < 99; % spouse has pension;
            
            % include spouse's DB pension;
            if spage == data.sppenstart;
                sppensben = data.sppensben; 
            end;
            
            if spage >= data.sppenstart && spage < 100;
                othincome(age - 25 + 1, 1) = othincome(age - 25 + 1, 1) + sppensben;
                othincome(age - 25 + 1, 3) = othincome(age - 25 + 1, 3) + sppensben;
            end;
            sppensben = sppensben / (1 + gv.inflrate * (1 - gv.penadjrate)); % real DB pensin benefits decrease by adjustment factor each year;
            
            % include spouse's DC pension; 
            % unless all wives retire before age 62, her DC pension balance disappears if she dies before data.sppenstart; ??? 
            if spage == data.sppenstart;
                othincome(age - 25 + 1, 1) = othincome(age - 25 + 1, 1) + data.spplumpsum;
                othincome(age - 25 + 1, 3) = othincome(age - 25 + 1, 3) + data.spplumpsum;
            end;
            
        end;
    end;
end;

% include other income, such as inheritance, for all survival status;
for age = 25 : 99;
    for vcat = 1 : 3;
        othincome(age - 25 + 1, vcat) = othincome(age - 25 + 1, vcat) + data.othincome(age - 25 + 1, 1);
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate leisure preference %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate utility weights of leisure utility;
% h(t) = exp(beta*X(t) + epsilon(t));
% initialization;
utilr = zeros(gv.necatlim, 20); % utility of leisure by epsilon grid and age;
for age = 50 : 69;
    bX = calc.beta0 + calc.beta1 * (age - 62);
    if age >= data.healthage;
        bX = bX + calc.beta2;
    end;
    for ecat = 1 : gv.necatlim;
        utilr(ecat, age - 50 + 1) = exp(bX + calc.sige * ecats(ecat, 1));
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate utility of partial retirement %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialization;
tcats = zeros(gv.ntcatlim, 20); % utility of partial retirement by partial retirement preference grid and age;
for tcat = 1 : gv.ntcatlim; % partial retirement preference;
    
    if tcat == 1;
        percentile =  5; % least preference of partial retirement;
    elseif tcat == 2;
        percentile = 20;
    elseif tcat == 3;
        percentile = 40;
    elseif tcat == 4;
        percentile = 60;
    elseif tcat == 5;
        percentile = 80;
    elseif tcat == 6;
        percentile = 95; % most preference of partial retirement;
    end;
    
    % utility of partial retirement is:
    % Vp = 0.5 + 0.5 * log(percentile / 100 * (exp(gamma0 + gamma1 * (age - 65)) - 1) + 1) / (gamma0 + gamma1 * (age - 65));
    % if age > healthage, Vp = 0.5;
    for age = 50 : 69;
        exponent = calc.gamma0 + calc.gamma1 * (age - 65);
        delta = log((percentile / 100) * (exp(exponent) - 1) + 1) / exponent;
        if data.healthage <= age;
            delta = 0;
        end;
        tcats(tcat, age - 50 + 1) = 0.5 + 0.5 * delta;
    end;
    
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% assign the values to structure variable calc for future calculations %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
calc.srate = srate;
calc.transrate = transrate;
calc.acats = acats;
calc.ecats = ecats;
calc.etrans = etrans;
calc.rnorm = rnorm;
calc.rprob = rprob;
calc.rtransa = rtransa;
calc.earlypenage = earlypenage;
calc.delayedpenage = delayedpenage;
calc.pcats = pcats;
calc.initialpcat = initialpcat;
calc.contribstartage = contribstartage;
calc.dccontrib = dccontrib;
calc.lcats = lcats;
calc.rtransl = rtransl;
calc.othincome = othincome;
calc.utilr = utilr;
calc.tcats = tcats;












