% ----------------------------------------------------------------------- %
% % getmoments function                                                 % %
% % simulate and calculate moments for the household                    % %
% ----------------------------------------------------------------------- %

function [calc arglist] = getmoments(tcat, gv, data, calc, arglist)

% calculate q statistic for iteration and estimation modes;
% only need to report simulated retirement moments for test and simulation modes;
if gv.mode == 1 || gv.mode == 2; 
    mobs = arglist.nmat(calc.obs, :); % indicator vector for the household, whether the corresponding data moments are available from the HRS
    mvec = zeros(1, gv.nmoments); % moments vector for the household, differences between observed moments and simulated moments;
    wmat = zeros(1, gv.nmoments * (gv.nmoments + 1) / 2); % inverse weighting vector for the household;
end;

if tcat == 1; 
    
    % reset random number generator seed for each household;
    if tcat == 1;
        calc = randreset(calc); % random number generator seed and a couple of indicators;
    end;
    % initialization;
    % intermediate retirement moments vectors for the 10,000 simulations;
    ret0vec = zeros(21, 1); % percent of simulated workers who retired from career job at ages 50 - 70;
    ret1vec = zeros(21, 1); % percent of simulated workers who are not currently working full time at ages 50 - 70;
    ret2vec = zeros(21, 1); % percent of simulated workers who are currently retired at ages 50 - 70;
    returnft = zeros(21, 1); % percent of simulated workers who are previously retired or in part time work, newly return to full time work at ages 50 - 70;
    returnpt = zeros(21, 1); % percent of simulated workers who are previously retired, newly return to part time work at ages 50 - 70;
    newpt = zeros(21, 1); % percent of simulated workers who are newly enter part time work at ages 50 - 70;
    reversal = zeros(5, 1); % percent of simulated workers who are taking reversal jobs at survey 2 - 6;
    everreturnft = 0; % percent of simulated workers ever return to full time work after full or partial retirement
    everreturnpt = 0; % percent of simulated workers ever return to part time work after full retirement
    everreturn   = 0; % percent of simulated workers ever return to full time work after full or partial retirement
else 
    
    % carry on calculating retirement moments for the 10,000 simulations;
    ret0vec = calc.ret0vec;
    ret1vec = calc.ret1vec;
    ret2vec = calc.ret2vec;
    returnft = calc.returnft;
    returnpt = calc.returnpt;
    newpt = calc.newpt;
    reversal = calc.reversal;
    everreturnft = calc.everreturnft;
    everreturnpt = calc.everreturnpt;
    everreturn = calc.everreturn;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simulate individuals from 25 to firstage - 1 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% all simulants must work in career job in this period;
% they all have the same wages;
% individuals do not necessarily have the same saving behaviors given different leisure perference (epsilon);
% calculate wealth and DC pension balance at firstage - 1 as a function of epsilon;
% if actual asset returns are available, use actural asset returns;
% if actual asset returns are not available, use average asset returns;
% instead of doing the same calculations for all simulants (with random leisure preference), do grid calcuations;
% so later use interpolations to retrieve asset and DC pension balance at firstage - 1 for all simulants;  
% 3 dimension interpolation;
% initialization;
epsilonvec = zeros(10 * gv.necatlim + 1, 1); % epsilon, leisure prefence;
wealthvec = zeros(10 * gv.necatlim + 1, 1); % asset, associated with each corresponding epsilon;
dcwealthvec = zeros(10 * gv.necatlim + 1, 1); % DC pension balance, associated with each corresponding epsilon;

% increase grid size to increase accuracy;
epscatlim = 10 * (gv.necatlim -1); % number of grid points in epsilon vector;
for epscat = 1 : epscatlim + 1; % epsilon;
    
    % initialization;
    eps = ((epscatlim - (epscat - 1)) * calc.ecats(1, 1) + (epscat - 1) * calc.ecats(gv.necatlim, 1)) / epscatlim; % epsilon;
    wealth = 0; % asset;
    dcwealth = 0; % DC pension balace;
    
    for age = 25 : calc.firstage - 1; % age;
        
        year = age + data.birthyear; % year;
        
        if year >= 1926 && year <= 2000; % actual asset returns available;
            wealth = (1 + 0.01 * gv.realrate((year - 1) - 1926 + 1, 1)) * wealth;
            dcwealth = (1 + 0.01 * gv.realrate((year - 1) - 1926 + 1, 1)) * dcwealth;
        else % not available;
            wealth = (1 + gv.realrateg) * wealth;
            dcwealth = (1 + gv.realrateg) * dcwealth;
        end;
        
        % interpolation: asset;
        acat = 1;
        while acat < gv.nacatlim && wealth > calc.acats(acat, 1);
            acat = acat + 1;
        end;
        if acat > 1;
            acat = acat - 1;
        end;
        fa = (wealth - calc.acats(acat, 1)) / (calc.acats(acat + 1, 1) - calc.acats(acat, 1));
        
        % if eligible, assumulate DC pension wealth each year between contribstartage and age that ends career job end;
        if calc.nlcatlim > 1 && age <= data.penjobend && age > calc.contribstartage; % elibile for DC pension balance;
            
            % interpolation: DC pension balace;
            lcat = 1;
            while lcat < calc.nlcatlim && dcwealth > calc.lcats(lcat, 1);
                lcat = lcat + 1;
            end;
            if lcat > 1;
                lcat = lcat - 1;
            end;
            fl = (dcwealth - calc.lcats(lcat, 1)) / (calc.lcats(lcat + 1, 1) - calc.lcats(lcat, 1));
            
        else % not eligible;
            lcat = 1;
            fl = 0;
        end;
        
        % choose savings level;
        % pick saving levels from savw (optimal savings, currently working in career job);
        % interpolation: leisure preference;
        ecat = floor((epscat - 1) / 10) + 1; 
        fe = mod((epscat - 1), 10) / 10; 
        for lcati = lcat : lcat + 1;
            if lcati == lcat || fl ~= 0;
                if fe ~= 0;
                    temp  = (1 - fe) * ((1 - fa) * calc.savw(age - 25 + 1, lcati, ecat, acat) ...
                        + fa * calc.savw(age - 25 + 1, lcati, ecat, acat + 1)) ...
                        + fe * ((1 - fa) * calc.savw(age - 25 + 1, lcati, ecat + 1, acat) ...
                        + fa * calc.savw(age - 25 + 1, lcati, ecat + 1, acat + 1));
                else
                    temp = (1 - fa) * calc.savw(age - 25 + 1, lcati, ecat, acat) ...
                        + fa * calc.savw(age - 25 + 1, lcati, ecat, acat + 1);
                end;
                
                if lcati == lcat;
                    templ(1, 1) = temp;
                else
                    templ(2, 1) = temp;
                end;
            end;
        end;
        
        if fl ~= 0;
            saving = (1 - fl) * templ(1, 1) + fl * templ(2, 1);
        else
            saving = templ(1, 1);
        end;
        
        % add DC pension balance and current period DC contribution to asset;
        if age == data.penjobend;
            saving = saving + dcwealth + calc.dccontrib(age - 25 + 1, 1);
        end;
        wealth = wealth + saving; % asset;
        
        % include current period DC contribution;
        if age < data.penjobend && age >= calc.contribstartage;
            dcwealth = dcwealth + calc.dccontrib(age - 25 + 1, 1);
        else
            dcwealth = 0;
        end;
        
    end;
    
    epsilonvec(epscat, 1) = eps; % epsilon;
    wealthvec(epscat, 1) = wealth; % asset;
    dcwealthvec(epscat, 1) = dcwealth; % DC pension balance;
    
end;

% distribute number of simulated households by partial retirement preference;
% so in the final sample of 10,000 simulated households:
% 10% have the lowest partial retirement preference;
% 20% have the second lowest partial retirement preference;
% ...;
% 20% have the second highest partial retirement preference;
% 10% have the highest partial retirement preference;
if tcat == 1;
    llim = (0 * gv.nsims) / 100 + 1;    
    ulim = (10 * gv.nsims) / 100;
elseif tcat == 2;
    llim = (10 * gv.nsims) / 100 + 1;    
    ulim = (30 * gv.nsims) / 100;
elseif tcat == 3;
    llim = (30 * gv.nsims) / 100 + 1;    
    ulim = (50 * gv.nsims) / 100;
elseif tcat == 4;
    llim = (50 * gv.nsims) / 100 + 1;    
    ulim = (70 * gv.nsims) / 100;
elseif tcat == 5;
    llim = (70 * gv.nsims) / 100 + 1;    
    ulim = (90 * gv.nsims) / 100;
elseif tcat == 6;
    llim = (90 * gv.nsims) / 100 + 1;    
    ulim = (100 * gv.nsims) / 100;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simulate individuals from firstage to lastage %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for simcount = llim : ulim;
    
    % generate initial epsilon (leisure preference) for each simulated household;
    [epsilon calc] = gaussdev(calc); % epsilon;
    
    % setup boundary;
    if epsilon < calc.ecats(1, 1)
        epsilon = calc.ecats(1, 1);
    elseif epsilon > calc.ecats(gv.necatlim, 1);
        epsilon = calc.ecats(gv.necatlim, 1);
    end;
    
    % find its asset and DC pension balance at firstage;
    % find correspoind epsilon grid point in epsilonvec;
    if epsilon < calc.ecats(gv.necatlim, 1);
        fepscat = 10 * ((epsilon - calc.ecats(1, 1)) / (calc.ecats(2, 1) - calc.ecats(1, 1))) + 1;
    elseif epsilon == calc.ecats(gv.necatlim, 1);
        fepscat = 10 * (gv.necatlim - 1) + 1;
    end;
    
    % interpolation;
    epscat = floor(fepscat);
    fe = fepscat - epscat;
    if fe ~= 0;
        wealth = (1 - fe) * wealthvec(epscat, 1)   + fe * wealthvec(epscat + 1, 1); % asset;
        dcwealth = (1 - fe) * dcwealthvec(epscat, 1) + fe * dcwealthvec(epscat + 1, 1); % DC pension balance;
    elseif fe == 0;
        wealth = wealthvec(epscat, 1);
        dcwealth = dcwealthvec(epscat, 1);
    end;
    
    % decide retirement status based on current period economic state;
    % initialization;
    retv = zeros(21, 1); % retirement status vector for the household at ages 50 - 69; 5 if working full time; 3 if working part time; 1 if fully retired;
    mainjob = 1; % retirement status: whether currently working in career job;
    
    % prepare for interpolation;
    acat = 1; % asset grid point;
    lcat = 1; % DC pension balance grid point;
    % leisure preference grid point;
    fecat = (epsilon - calc.ecats(1, 1)) / (calc.ecats(2, 1) - calc.ecats(1, 1)) + 1; 
    if fecat < gv.necatlim;
        ecat = floor(fecat);
        fe = fecat - ecat;
    elseif fecat == gv.necatlim;
        ecat = gv.necatlim;
        fe = 0;
    end;
    % initial DB pension benefits grid point and SS benefits grid point if retire at firstage;
    pcat = floor(calc.initialpcat(calc.firstage - 50 + 1, 1));
    fp = calc.initialpcat(calc.firstage - 50 + 1, 1) - pcat;
    scat = floor(calc.initialscat(calc.firstage - 50 + 1, 1)); 
    fs = calc.initialscat(calc.firstage - 50 + 1, 1) - scat;
    
    % whether possible to receive delayed DB pension;
    if calc.earlypenage > calc.firstage && calc.earlypenage < calc. delayedpenage;
        delayedpen = 1;
    else
        delayedpen = 0;
    end;
    
    for age = calc.firstage : calc.lastage; % age;
        
        year = age + data.birthyear; % year;
        
        % asset gets return;
        if year > 1926 && year <= 2001; % actual asset returns available;
            wealth = (1 + 0.01 * gv.realrate((year - 1) - 1926 + 1, 1)) * wealth;
        else % not available;
            wealth = (1 + gv.realrateg) * wealth;
        end;
        
        % if working in career job, keep accumulation DC pension balance;
        % DC pension balance receives the same asset return;
        if mainjob == 1; % currently working in career job;
            if year > 1926 && year <= 2001; % actual asset returns available;
                dcwealth = (1 + 0.01 * gv.realrate((year - 1) - 1926 + 1, 1)) * dcwealth;
            else % not available;
                dcwealth = (1 + gv.realrateg) * dcwealth;
            end;
        end;
        
        % delayed DB pension grid point;
        if delayedpen == 1 && age >= calc.earlypenage && age < calc.delayedpenage; % delayed DB pension;
            dcat = 2;
        else
            dcat = 1;
        end;
        
        % if retired from career job, epsilon could change every period;
        % update epsilon grid point;
        if mainjob == 0; % currently retired from career job;
            fecat = (epsilon - calc.ecats(1, 1)) / (calc.ecats(2, 1) - calc.ecats(1, 1)) + 1;
            ecat = floor(fecat);
            fe = fecat - ecat;
        end;
        
        % prepare for interpolation;
        % find out the corresponding asset grid point;
        if wealth > calc.acats(acat, 1);
            acat = acat + 1;
            while acat < gv.nacatlim && wealth > calc.acats(acat, 1);
                acat = acat + 1;
            end;
            if acat > 1;
                acat = acat - 1;
            end;
        else
            while acat > 1 && wealth < calc.acats(acat, 1);
                acat = acat - 1;
            end;
        end;
        fa = (wealth - calc.acats(acat, 1)) / (calc.acats(acat + 1, 1) - calc.acats(acat, 1));
        
        % find out the corresponding DC pension balance grid point;
        % keep accumulating DC pension balance;
        if mainjob == 1 && calc.nlcatlim > 1 && age < data.penjobend + 1 && age > calc.contribstartage; 
            if dcwealth > calc.lcats(lcat, 1);
                lcat = lcat + 1;
                while lcat < calc.nlcatlim && dcwealth > calc.lcats(lcat, 1);
                    lcat = lcat + 1;
                end;
                if lcat > 1;
                    lcat = lcat - 1;
                end;
            else
                while lcat > 1 && dcwealth < calc.lcats(lcat, 1);
                    lcat = lcat - 1;
                end;
            end;
            fl = (dcwealth - calc.lcats(lcat, 1)) / (calc.lcats(lcat + 1, 1) - calc.lcats(lcat, 1));
        else
            lcat = 1;
            fl = 0;
        end;
        
        % transfer DC pension balance to asset;
        if mainjob == 1 && ((calc.nlcatlim > 1 && age < data.penjobend + 1 && age > calc.contribstartage) || data.eowamount(age - 50 + 1, 1) > 0);
            
            % calcualate asset after including DC pension balance;
            pwealth = wealth + dcwealth + data.eowamount(age - 50 + 1, 1);
            
            lacat = acat + 1;
            while lacat < gv.nacatlim && pwealth > calc.acats(lacat, 1);
                lacat = lacat + 1;
            end;
            if lacat > 1;
                lacat = lacat - 1;
            end;
            fla = (pwealth - calc.acats(lacat, 1)) / (calc.acats(lacat + 1, 1) - calc.acats(lacat, 1));
        else
            lacat = acat;
            fla = fa;
        end;
        
        % pick the optimal retirement decision given current economic state;
        % because retirement decision is a discrete choice, can't do interpolation;
        % pick all possible retirement decisions around the current economic state and run simulations to pick the optimal one;        
        % retirement decisions: 1 is staying in career job; 2 is returned full time work; 3 is part time work; 4 is retired;
        % initialization;
        jobchoice = zeros(4, 1); % indicator of potential retirement decisions;
        choicecount = 0; % number of retirement decisions around the current economic state;
        job = 0; % retirement decision;
        
        if mainjob == 1; % currently working in career job; economic state: age, lcat, ecat, acat;
            for lcati = lcat : lcat + 1; % DC pension balance;
                if lcati == lcat || fl ~= 0;
                    for ecati = ecat : ecat + 1; % leisure preference;
                        if ecati == ecat || fe ~= 0;
                            for acati = acat : acat + 1; % asset;
                                
                                % pick corresponding optimal retirement decision, given current economic state; 
                                choice = calc.mjchoice(age - 50 + 1, lcati, ecati, acati); % retirement decision;
                                if jobchoice(choice, 1) == 0;
                                    jobchoice(choice, 1) = 1; % indicator of potential retirement decision;
                                    choicecount = choicecount + 1; % number of retirement decisions around the current economic state;
                                end;
                                
                            end;
                        end;
                    end;
                end;
            end;
        else % currently retired from career job; economic state: age, dcat, pcat, scat, ecat, acat;
            for pcati = pcat : pcat + 1; % DB pension benefits;
                if pcati == pcat || fp ~= 0;
                    for ecati = ecat : min(ecat + 1, gv.necatlim); % leisure preference;
                        if ecati == ecat || fe ~= 0;
                            for acati = acat : acat + 1; % asset;
                                for scati = scat : scat + 1; % SS benefits;
                                    if scati == scat || fs ~= 0;
                                        
                                        % pick corresponding optimal retirement decision, given current economic state; 
                                        choice = calc.retchoice(age - 50 + 1, dcat, pcati, scati, ecati, acati); % retirement decision;
                                        if jobchoice(choice, 1) == 0;
                                            jobchoice(choice, 1) = 1; % indicator of potential retirement decision;
                                            choicecount = choicecount + 1; % number of retirement decisions around the current economic state;
                                        end;
                                        
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
        
        % pick the optimal retirement decision based on total utility;
        if choicecount > 1; % multiple potential retirement decisions around the current economic state;
            for choice = 1 : 4; % retirement decisions;
                if jobchoice(choice, 1) == 1; % indicator of a potential choice;
                    if choice == 1; % staying in career job; economic state: age, lcat, ecat, acat;
                        for lcati = lcat : lcat + 1; % DC pension balance;
                            if lcati == lcat || fl ~= 0
                                % 3 dimension interpolation;
                                % interpolation: leisure preference and asset;
                                if fe ~= 0;
                                    temp  = (1 - fe) * ((1 - fa) * calc.totuw(age - 25 + 1, lcati, ecat, acat) ...
                                        +  fa * calc.totuw(age - 25 + 1, lcati, ecat, acat + 1))...
                                        +  fe * ((1 - fa) * calc.totuw(age - 25 + 1, lcati, ecat + 1, acat) ...
                                        +  fa * calc.totuw(age - 25 + 1, lcati, ecat + 1, acat + 1));
                                else
                                    temp = (1 - fa) * calc.totuw(age - 25 + 1, lcati, ecat, acat) ...
                                        +  fa * calc.totuw(age - 25 + 1, lcati, ecat, acat + 1);
                                end;
                                
                                if lcati == lcat;
                                    templ(1, 1) = temp;
                                else
                                    templ(2, 1) = temp;
                                end;
                            end;
                        end;
                        
                        % interpolation: DC pension balance;
                        if fl ~= 0;
                            totutil = (1 - fl) * templ(1, 1) + fl * templ(2, 1);
                        else
                            totutil = templ(1, 1);
                        end;
                    else % retire from career job; transfer DC pension balance to asset; economic state: age, job type, dcat, pcat, scat, ecat, acat;
                        for pcati = pcat : pcat + 1; % DB pension benefits;
                            if pcati == pcat || fp ~= 0;
                                for scati = scat : scat + 1; % SS benefits;
                                    if scati == scat || fs ~= 0;  
                                        
                                        % 4 dimension interpolation;
                                        % interpolation: leisure preference and asset (after including DC pension balance);
                                        if fe ~= 0;
                                            temp  = (1 - fe) * ((1 - fla) * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat, lacat) ...
                                                +  fla * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat, lacat + 1)) ...
                                                +  fe * ((1 - fla) * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat + 1, lacat) ...
                                                +  fla * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat + 1, lacat + 1));
                                        else
                                            temp =  (1 - fla) * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat, lacat) ...
                                                +  fla * calc.totud(age - 50 + 1, choice - 1, dcat, pcati, scati, ecat, lacat + 1);
                                        end;
                                        
                                        if scati == scat;
                                            temps(1, 1) = temp;
                                        else
                                            temps(2, 1) = temp;
                                        end;
                                    end;
                                end;
                                
                                % interpolation: SS benefits;
                                if fs ~= 0;
                                    temp = (1 - fs) * temps(1, 1) + fs * temps(2, 1);
                                else
                                    temp = temps(1, 1);
                                end;
                                
                                if pcati == pcat
                                    tempp(1, 1) = temp;
                                else
                                    tempp(2, 1) = temp;
                                end;
                            end;
                        end;
                        
                        % interpolation: DB pension benefits;
                        if fp ~= 0;
                            totutil = (1 - fp) * tempp(1, 1) + fp * tempp(2, 1);
                        else
                            totutil = tempp(1, 1);
                        end;
                    end;
                    
                    % pick the optimal retirement decision which has the highest total utility;
                    if job < 1;
                        job = choice;
                        totalutil = totutil;
                    else
                        if totutil > totalutil;
                            job = choice;
                            totalutil = totutil;
                        end;
                    end;
                end;
            end;
        else % only one potential retirement decision around the current economic state; this is the optimal choice;
            for choice = 1 : 4;
                if jobchoice(choice, 1) == 1;
                    job = choice;
                end;
            end;
        end;
        
        % calculate optimal savings, given optimal retirement decision;
        if job == 1; % staying in career job;
            
            % calculate optimal savings;
            % 3 dimension interpolation;
            for lcati = lcat : lcat + 1; % DC pension balance;
                if lcati == lcat || fl ~= 0;
                    
                    % interpolation: leisure preference and asset;
                    if fe ~= 0;
                        temp = (1 - fe) * ((1 - fa) * calc.savw(age - 25 + 1, lcati, ecat, acat) ...
                            + fa * calc.savw(age - 25 + 1, lcati, ecat, acat + 1)) ...
                            + fe * ((1 - fa) * calc.savw(age - 25 + 1, lcati, ecat + 1, acat) ...
                            + fa * calc.savw(age - 25 + 1, lcati, ecat + 1, acat + 1));
                    else
                        temp = (1 - fa) * calc.savw(age - 25 + 1, lcati, ecat, acat) ...
                            + fa * calc.savw(age - 25 + 1, lcati, ecat, acat + 1);
                    end;
                    
                    if lcati == lcat;
                        templ(1, 1) = temp;
                    else
                        templ(2, 1) = temp;
                    end;
                end;
            end;
            
            % interpolation: DC pension balance;
            if fl ~= 0;
                saving = (1 - fl) * templ(1, 1) + fl * templ(2, 1);
            else
                saving = templ(1, 1);
            end;
            
            % transfer DC pension balance to asset at penjobend (age);
            if age == data.penjobend;
                saving = saving + dcwealth + calc.dccontrib(age - 25 + 1, 1);
            end;
            
            % update;
            % keep leisure preference grid point the same;
            % whether possible to receive delayed DB pension;
            if age + 1 < calc.earlypenage && calc.earlypenage < calc.delayedpenage;
                delayedpen = 1;
            else
                delayedpen = 0;
            end;
            
            % update initial DB pension benefits grid point and SS benefits grid point if retire at age + 1; 
            pcat = floor(calc.initialpcat((age + 1) - 50 + 1, 1));
            fp = calc.initialpcat((age + 1) - 50 + 1, 1) - pcat;
            scat = floor(calc.initialscat((age + 1) - 50 + 1, 1));
            fs = calc.initialscat((age + 1) - 50 + 1, 1) - scat;
            
        else % retired from career job;
            % 4 dimension interpolation;
            for pcati = pcat : pcat + 1; % DC pension balance;
                if pcati == pcat || fp ~= 0;
                    for scati = scat : scat + 1; % SS benefits;
                        if scati == scat || fs ~= 0;
                            
                            % interpolation: leisure preference and asset;
                            if fe ~= 0;
                                temp = (1 - fe) * ((1 - fla) * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat, lacat) ...
                                    + fla * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat, lacat + 1)) ...
                                    + fe * ((1 - fla) * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat + 1, lacat) ...
                                    + fla * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat + 1, lacat + 1));
                            else
                                temp = (1 - fla) * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat, lacat) ...
                                    + fla * calc.savd(age - 50 + 1, job - 1, dcat, pcati, scati, ecat, lacat + 1);
                            end;
                            
                            if scati == scat;
                                temps(1, 1) = temp;
                            else
                                temps(2, 1) = temp;
                            end;
                        end;
                    end;
                    
                    % interpolation: SS benefits;
                    if fs ~= 0;
                        temp = (1 - fs) * temps(1, 1) + fs * temps(2, 1);
                    else
                        temp = temps(1, 1);
                    end;
                    
                    if pcati == pcat;
                        tempp(1, 1) = temp;
                    else
                        tempp(2, 1) = temp;
                    end;
                end;
            end;
            
            % interpolation: DB pension benefits;
            if fp ~= 0;
                saving = (1 - fp) * tempp(1, 1) + fp * tempp(2, 1);
            else
                saving = tempp(1, 1);
            end;
            
            % if currently working in career job and decides to retire, add DC pension balance to asset;
            if mainjob == 1;
                
                if age < data.penjobend + 1 && age > calc.contribstartage; % eligible for DC pension;
                    saving = saving + dcwealth;
                    % update dcwealth in a later section;
                end;
                saving = saving + data.eowamount(age - 50 + 1, 1); 
                
                % update retirement status: retired from career job;
                mainjob = 0;
            end;
        end;
        
        % save retirement decision to retv for later calculations;
        retv(age - 50 + 1, 1) = job;
        
        % update epsilon for next period;
        % keep the same epsilon when working in career job;
        [epschange calc] = gaussdev(calc);
        
        if mainjob == 0;
            
            % update epsilon;
            epsilon = calc.erho * epsilon + epschange * sqrt(1 - calc.erho ^ 2);
            
            % setup boundary;
            if epsilon < calc.ecats(1, 1);
                epsilon = calc.ecats(1, 1);
            elseif epsilon > calc.ecats(gv.necatlim, 1);
                epsilon = calc.ecats(gv.necatlim, 1);
            end;
        end;
        
        % update asset, DC pension balance and SS benefits grid point;
        
        % asset;
        wealth = wealth + saving;
        
        % keep accumulating DC pension balance if staying in career job;
        % if quitting career job, set dcwealth == 0;
        if job == 1; % staying in career job;
            if age < data.penjobend && age >= calc.contribstartage;
                dcwealth = dcwealth + calc.dccontrib(age - 25 + 1, 1); % DC pension balance;
            else
                dcwealth = 0;
            end;
        end;
        
        % update SS benefits grid point for age + 1;
        if age >= calc.sserage && age < 70; % age;
            if job == 2 || job == 3; % returned full time work or part time work;
                
                % interpolation;
                if job == 2; % returned full time work;
                    if fs ~= 0;
                        sbenf = (1 - fs) * (calc.ssbenf(scat, 1) + calc.benchanges(age - 62 + 1, 1, scat)) ...
                            + fs * (calc.ssbenf(scat + 1, 1) + calc.benchanges(age - 62 + 1, 1, scat + 1));
                    else
                        sbenf = calc.ssbenf(scat, 1) + calc.benchanges(age - 62 + 1, 1, scat);
                    end;
                elseif job == 3; % part time work;
                    if fs ~= 0;
                        sbenf = (1 - fs) * (calc.ssbenf(scat, 1) + calc.benchangep(age - 62 + 1, 1, scat))...
                            + fs * (calc.ssbenf(scat + 1, 1) + calc.benchangep(age - 62 + 1, 1, scat + 1));
                    else
                        sbenf = calc.ssbenf(scat, 1) + calc.benchangep(age - 62 + 1, 1, scat);
                    end;
                end;
                
                fscat = -1;
                if scat == 4;
                    scati = 5;
                else
                    for scati = scat + 1 : gv.nscatlim;
                        if fscat == -1 && sbenf < calc.ssbenf(scati, 1);
                            fscat = (scati - 1) + (sbenf - calc.ssbenf(scati - 1, 1)) ...
                                / (calc.ssbenf(scati, 1) - calc.ssbenf(scati - 1, 1));
                        end;
                    end;
                end;
                
                if fscat == -1;
                    fscat = (scati - 2) + 0.99;
                end;
                
                scat = floor(fscat);
                fs = fscat - scat;
            end;
        end;
        
    end;
    
    % summarize results for the simulated household;
    % initialization;
    priorret = 0; % whether retired from career job;
    simeverreturnft = 0; % ever return to full time work after full or partial retirement;
    simeverreturnpt = 0; % ever return to part time work after full retirement;
    simeverreturn = 0; % ever return to full time work after full or partial retirement, or return to part time work after full retirement;
    
    for age = calc.firstage : calc.lastage; % age;
        
        % pick corresponding retirement decision of the simulated household at age;
        job = retv(age - 50 + 1, 1); % retirement decision;
        
        if job == 2 && priorret == 1; % returned full time work;
            
            % update retirement moments;
            ret0vec(age - 50 + 1, 1) = ret0vec(age - 50 + 1, 1) + 1; % retired from career job;
            
            if age > calc.firstage;
                if retv((age - 1) - 50 + 1, 1) >= 3;
                    returnft(age - 50 + 1, 1) = returnft(age - 50 + 1, 1) + 1; % individuals who are partially or fully retired at age - 1, return to full time work at age;
                    simeverreturnft = 1; % ever return to full time work after full or partial retirement;
                    simeverreturn = 1; % ever return to full time work after full or partial retirement, or return to part time work after full retirement;
                end;
            end;
            
        elseif job == 3; % part time work;
            
            ret0vec(age - 50 + 1, 1) = ret0vec(age - 50 + 1, 1) + 1; % retired from career job
            ret1vec(age - 50 + 1, 1) = ret1vec(age - 50 + 1, 1) + 1; % currently not working full time
            
            % update retirement moments;
            if age > calc.firstage;
                if retv((age - 1) - 50 + 1, 1) == 4;
                    returnpt(age - 50 + 1, 1) = returnpt(age - 50 + 1, 1) + 1; % individuals who are fully retired at age - 1, return to full time work at age;
                    simeverreturnpt = 1; % ever return to part time work after full retirement;
                    simeverreturn = 1; % ever return to full time work after full or partial retirement, or return to part time work after full retirement;
                end;
                if retv((age - 1) - 50 + 1, 1) ~= 3;
                    newpt(age - 50 + 1, 1) = newpt(age - 50 + 1, 1) + 1; % workers who are working full time or fully retired at age - 1, newly enter part time work at age;
                end;
            end;
            
            % update retirement status;
            priorret = 1; % whether retired from career job;
            
        elseif job == 4; % retired;
            
            % update retirement moments;
            ret0vec(age - 50 + 1, 1) = ret0vec(age - 50 + 1, 1) + 1; % retired from career job;
            ret1vec(age - 50 + 1, 1) = ret1vec(age - 50 + 1, 1) + 1; % currently not working full time;
            ret2vec(age - 50 + 1, 1) = ret2vec(age - 50 + 1, 1) + 1; % currently retired;
            
            % update retirement status;
            priorret = 1; % whether retired from career job;
        end;
    end;
    
    % total number of individuals in corresponding categories;
    everreturnft = everreturnft + simeverreturnft;
    everreturnpt = everreturnpt + simeverreturnpt;
    everreturn   = everreturn + simeverreturn;
    
    % from survey 1 to 5 because need to know retirement status in next survey (6);
    for survey = 1 : 5; 
        
        % pick corresponding age and retirement status from the HRS data;
        age0 = data.age(survey, 1);
        ret0 = data.ret(survey, 1);
        age1 = data.age(survey + 1, 1);
        ret1 = data.ret(survey + 1, 1);
        
        % decide whether the individual returned to full time job at each survey;
        if age0 >= 54 && age0 <= 66 && ret0 >= 1 && ret0 <= 5 ...
                && age1 >= 54 && age1 <= 66 && ret1 >= 1 && ret1 <= 5 ...
                && age1 <= calc.lastage;
            if retv(age0 - 50 + 1, 1) >= 2 && retv(age1 - 50 + 1, 1) <= 1;
                reversal(survey, 1) = reversal(survey, 1) + 1; % individual takes reversal jobs at survey age;
            end;
        end;
    end;
    
end;

% return if not at last tcat category; 
% keep finishing the 10,000 simulation for the household;
if tcat < gv.ntcatlim; % partial retirement preference; 10,000 simulations have not finished yet;
    
    % save intermediate retirement moments for later calculations;
    calc.ret0vec = ret0vec;
    calc.ret1vec = ret1vec;
    calc.ret2vec = ret2vec;
    calc.returnft = returnft;
    calc.returnpt = returnpt;
    calc.newpt = newpt;
    calc.everreturnft = everreturnft ;
    calc.everreturnpt = everreturnpt ;
    calc.everreturn = everreturn;
    calc.reversal = reversal;
    
elseif tcat == gv.ntcatlim; % partial retirement preference; % last group of the 10,000 simulated households; 10,000 simulations have finished;

    % summarize results for 10,000 simulations;
    for age = 50 : 69;
        ret0vec(age - 50 + 1, 1) = ret0vec(age - 50 + 1, 1) / gv.nsims;
        ret1vec(age - 50 + 1, 1) = ret1vec(age - 50 + 1, 1) / gv.nsims;
        ret2vec(age - 50 + 1, 1) = ret2vec(age - 50 + 1, 1) / gv.nsims;
        returnft(age - 50 + 1, 1) = returnft(age - 50 + 1, 1) / gv.nsims;
        returnpt(age - 50 + 1, 1) = returnpt(age - 50 + 1, 1) / gv.nsims;
        newpt(age - 50 + 1, 1) = newpt(age - 50 + 1, 1) / gv.nsims;
    end;
    everreturnft = everreturnft / gv.nsims;
    everreturnpt = everreturnpt / gv.nsims;
    everreturn = everreturn / gv.nsims;
    
    % for iteration and estimation modes;
    % calculate differences between observed moments and simulated moments;
    % save results to mvec and wmat;
    if gv.mode == 1 || gv.mode == 2;

        for survey = 1 : 5;
            reversal(survey, 1) = reversal(survey, 1) / gv.nsims;
        end;
        
        for survey = 1 : 6;

            age = data.age(survey, 1); % age at survey time;
            ret = data.ret(survey, 1); % retirement status; 1 means fully retired, 3 means partial retired (part time work), 5 means full time work

            if age >= 54 && age <= 66 && ret >= 1&& ret <= 5;

                if ret == 1 || ret == 3; % observed current not working full time;
                    m1 = 1 - ret1vec(age - 50 + 1, 1);
                else  % observed current working full time;
                    m1 = - ret1vec(age - 50 + 1, 1);
                end;

                if ret == 1; % observed current retired;
                    m2 = 1 - ret2vec(age - 50 + 1, 1);
                else % observed currently not retired;
                    m2 = - ret2vec(age - 50 + 1, 1);
                end;

                moment.dif.age(age - 54 + 1, 1) = m1; % 13 moments of whether fully retired at ages 54 - 66;

                if age == 55;
                    k =  1;
                elseif age == 58;
                    k =  2;
                elseif age == 60;
                    k =  3;
                elseif age == 62;
                    k =  4;
                elseif age == 65;
                    k =  5;
                else
                    k = 0;
                end;

                if k >= 1;
                    moment.dif.pr(k, 1) = m2; % 5 moments of whether partial retired at ages 55, 58, 60, 62 and 65;
                    if data.inccat == 1;      
                        moment.dif.lowinc(k, 1) = m1; % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in the lower third of lifetime income;
                    end;
                    if data.inccat == 3;
                        moment.dif.highinc(k, 1) = m1; % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in the upper third of lifetime income;
                    end;
                    if  age >= data.healthage;
                        moment.dif.health(k, 1) = m1; % 5 moments of whether fully retired at ages 55, 58, 60, 62 and 65 if in poor health;
                    end;
                    if  age >= data.healthage;
                        moment.dif.healthpr(k, 1) = m2; % 5 moments of whether partial retired at ages 55, 58, 60, 62 and 65 if in poor health;
                    end;
                end;
            end;
        end;

        for survey = 1 : 5;
            age0 = data.age(survey, 1);
            ret0 = data.ret(survey, 1);
            age1 = data.age(survey + 1, 1);
            ret1 = data.ret(survey + 1, 1);

            if 54 <= age0 && age0 <= 66 && 1 <= ret0 && ret0 <= 5 ...
                    && 54 <= age1 && age1 <= 66 && 1 <= ret1 && ret1 <= 5 ...
                    && age1 <= calc.lastage;

                if (ret0 == 1 || ret0 == 3) && ret1 == 5;
                    moment.dif.reversal(survey, 1) = 1 - reversal(survey, 1); % 5 moments of whether in reversal job at ages 55, 58, 60, 62 and 65
                else
                    moment.dif.reversal(survey, 1) = - reversal(survey, 1);
                end;
            end;
        end;
        
        % put everything together;
        momentvec = zeros(gv.nmoments, 1);
        momentvec(1 : 13, 1) = moments.dif.age;
        momentvec(14 : 18, 1) = moments.dif.pr;
        momentvec(19 : 23, 1) = moments.dif.lowinc;
        momentvec(24 : 28, 1) = moments.dif.highinc;
        momentvec(29 : 33, 1) = moments.dif.health;
        momentvec(34 : 38, 1) = moments.dif.healthpr;
        momentvec(39 : 43, 1) = moments.dif.reversal;

        % drop moments not observed in the HRS data;
        for i = 1 : gv.nmoments;
            if mobs(i, 1) > 0; % corresponding moment in the HRS data observed;
                mvec(i, 1) = mvec(i, 1) + momentvec(i, 1);
            end;
        end;

        % update weighting matrix;
        for i = 1 : gv.nmoments;
            if mobs(i, 1) > 0; % ith moment observed; 
                for j = 1 : i;
                    if mobs(j, 1) > 0; % jth moment observed;
                        k = i * (i + 1) / 2 + j;
                        wmat(k, 1) = momentvec(i, 1) * momentvec(j, 1);
                    end;
                end;
            end;
        end;
        
        arglist.mvec(calc.obs, :) = mvec; % differences between observed and simulated moments;
        arglist.wmat(calc.obs, :) = wmat; % inverse weighting matrix;        

    end;
    
    % for modes other than estimation, save retirement moments to arglist stracture variable for later calculations;
    if gv.mode ~= 2;
        arglist.ret0mat(calc.obs, :) = ret0vec;
        arglist.ret1mat(calc.obs, :) = ret1vec;
        arglist.ret2mat(calc.obs, :) = ret2vec;
        arglist.returnft(calc.obs, :) = returnft;
        arglist.returnpt(calc.obs, :) = returnpt;
        arglist.newpt(calc.obs, :) = newpt;
        arglist.everreturnft(calc.obs, 1) = everreturnft;
        arglist.everreturnpt(calc.obs, 1) = everreturnpt;
        arglist.everreturn(calc.obs, 1) = everreturn;
    end
end;












