% ----------------------------------------------------------------------- %
% % indivmodel function                                                 % %
% % calculate the moments of each household                             % %
% ----------------------------------------------------------------------- %

function [arglist] = indivmodel(gv, oridata, arglist, cparm)

% initialization;
arglist.mvec = zeros(gv.nobs, gv.nmoments); % moments matrix, differences between observed moments and simulated moments;
arglist.wmat = zeros(gv.nobs, gv.nmoments * (gv.nmoments + 1) / 2); % weighting matrix;
arglist.ret0mat = zeros(gv.nobs, 21); % percent of simulated workers who retired from career job; or are 1) previously retired, currently in full time job; 2) currently in part time work; and 3) currently retired;
arglist.ret1mat = zeros(gv.nobs, 21); % percent of simulated workers who are not currently working full time; or are 1) currently in part time work; and 2) currently retired;
arglist.ret2mat = zeros(gv.nobs, 21); % percent of simulated workers who are currently retired;
arglist.returnft = zeros(gv.nobs, 21); % percent of simulated workers who are previously retired or in part time work, newly return to full time work;
arglist.returnpt = zeros(gv.nobs, 21); % percent of simulated workers who are previously retired, newly return to part time work;
arglist.newpt = zeros(gv.nobs, 21); % percent of simulated workers who are newly enter part time work;
arglist.everreturnft = zeros(gv.nobs, 1); % percent of simulated workers ever return to full time work after full or partial retirement;
arglist.everreturnpt = zeros(gv.nobs, 1); % percent of simulated workers ever return to part time work after full retirement;
arglist.everreturn = zeros(gv.nobs, 1); % percent of simulated workers ever return to full time work after full or partial retirement, or return to part time work after full retirement;

% find optimal retirement decisions for each household in the HRS dataset, given current set of preference parameters;
for obs = 1 : gv.nobs; % household;
    
    % control for the order of households;
    % could further check by testing whether calc.obs == data.inptobs;
    calc.obs = obs; % observation number;
    
    data = oridata(calc.obs, 1); % retrive the corresponding household's information from the original HRS dataset;
    
    % preference parameter manipulation; ??? 
    calc.sige = cparm(1, 1) * cparm(4, 1); % sige * beta1;
    calc.erho = cparm(2, 1); % erho; 
    calc.beta0 = cparm(3, 1) + 10 * (cparm(8, 1) + 1); % beta0 + 10 * (alpha + 1);   % To enhance stability; ???
    calc.beta1 = cparm(4, 1); % beta1;
    calc.beta2 = cparm(5, 1) * cparm(4, 1); % beta2 * beta1;
    calc.gamma0 = cparm(6, 1); % gamma0; 
    calc.gamma1 = cparm(7, 1); % gamma1;
    calc.alpha = cparm(8, 1); % alpha;

    % first, find firstage and lastage for the household;
    % firstage is the earliest possible age to retire from career job;
    % lastage is the latest possible age to work in any type of jobs;
    if gv.mode == 0; % test mode;
        
        firstage = 50;
        lastage  = 69;
        
    elseif gv.mode == 1 || calc.mode == 2; % iteration and estimation modes;
        
        firstage = 99;
        lastage = 0;
        
        for survey = 1 : 6; % 6 waves of surveys, 92, 94, 96, 98, 00, 02;
            
            age = data.age(survey, 1); % age at survey time;
            ret = data.ret(survey, 1); % retirement status at survey time;
            
            % assign firstage and lastage based on survey data;
            if age >= 54 && age <= 66 && ret >=1 && ret <= 5;
                if age < firstage;
                    firstage = age; % age at the earliest wave if between 54 and 66 with known retirement status;
                end;
                if age > lastage;
                    lastage = age; % age at the lastest wave if between 54 and 66 with known retirement status;
                end;
            end; 
            
        end;
        
        if firstage > lastage;
            
            fprintf('Error: firstage > lastage');
            break;
            
        elseif firstage <= lastage;
            
            firstage = firstage - 1; % smallest possible firstage is 53; why not 50; ???
            
            if lastage < 60;
                lastage = lastage + 2;
            else
                lastage = lastage + 3; % largest possible lastage is 69;
            end;
            
        end;
        
    else
        
        firstage = 50;
        lastage = 69;
        
    end;
    
    % assign the values to structure variable calc for future calculations;
    calc.firstage = firstage;
    calc.lastage = lastage;
    
    % start calcualting the HRS houshold;
    % prepare household specific variables to enter backward induction, such as survival probabilities, Social Security benefits, etc;
    calc = calcprep(gv, data, calc); 

    % calculate household specific time discount factor, rho;
    % keep adjusting rho to make simulated 1992 asset consistent with observed 1992 asset;
    % save it to arglist.rvec(obs, 1);
    % everytime alpha, the consumption parameter, changes, re-calculate rho, the time discount factor;
    if arglist.newalpha == 1;
        [calc, arglist] = rhocalc(gv, data, calc); 
    end;

    % run over all types of partial retirement preference types; 
    for tcat = 1 : gv.ntcatlim;
        
        % calculate policy functions, i.e. when households face certain economic state and shocks, what decisions they should make to maximize their expected future lifetime utility;
        calc = calcmodel(tcat, gv, data, calc, arglist);
%         save (['temp',num2str(tcat)]); % test purpose only;
        
        % simulate 10,000 households with different tastes of partial retirement for each HRS household and collect simulated moments;
        [calc, arglist] = getmoments(tcat, gv, data, calc, arglist);
        
    end;
%     save temp9 arglist; % test purpose only;
end;
















