% ----------------------------------------------------------------------- %
% % qmin function                                                       % %
% % minimize the q criterion for GMM estimates                          % % 
% ----------------------------------------------------------------------- %

function cparm = qmin(gv, oridata)

% initial parameter values;
cparm(1, 1) = gv.parm.sige;
cparm(2, 1) = gv.parm.erho;
cparm(3, 1) = gv.parm.beta0;
cparm(4, 1) = gv.parm.beta1;
cparm(5, 1) = gv.parm.beta2;
cparm(6, 1) = gv.parm.gamma0;
cparm(7, 1) = gv.parm.gamma1;
cparm(8, 1) = gv.parm.alpha;

% initialization;
arglist.oldalphavalue = 0; % value of alpha, the consumption parameter, in last round of calculations;
arglist.newalpha = 1; % indicator of new value of consumption parameter alpha; if it is 1, re-calculate rho;
mvec = zeros(gv.nmoments, 1); % moment (differences between observed moments and simulated moments) vector;
mvec0 = zeros(gv.nmoments, 1); % intemediate moment vector;
mvec1 = zeros(gv.nmoments, 1); % intemediate moment vector;
mvec2 = zeros(gv.nmoments, 1); % intemediate moment vector;
winv = zeros(gv.nmoments * (gv.nmoments + 1) / 2, 1); % inverse weighting matrix;
dmat = zeros(gv.nmoments * gv.nparms, 1);
intervals = [0.25, 0.05, 0.250, 0.025, 0.25, 0.25, 0.10, 0.05; ...
             0.25, 0.05, 0.025, 0.025, 0.25, 0.25, 0.10, 0.05; ...
             0.05, 0.02, 0.005, 0.005, 0.05, 0.05, 0.03, 0.02; ...
             0.01, 0.01, 0.001, 0.001, 0.01, 0.01, 0.01, 0.01]; % 8 parameters X 4 levels of changes;

for intindex = 1 : 4; % level of parameter change in minimization process;
    
    sameparms = 0; % indicator: whether the parameters are the same;
    
    for iter = 1 : 10; % loop 10 times;
        if sameparms == 0; % at least one parameter is new;
            
            % save current parameter values to lparm;
            lparm = cparm; % old parameter values;
            
            % run f function to generate q statistic associated with current set of parameter values;
            sw = 1; % re-calculate weighting matrix; 
            q1 = f(sw, cparm, gv, oridata, arglist);
            fprintf('\n %9.3f', q1);
            
            % experimenting parameter values (one by one) to minimize q statistic;
            for k = 1 : gv.nparms; % order of parameter to be experimented;
                
                if k == 1;
                    m = 3;
                elseif k == 2;
                    m = 4;
                elseif k == 3;
                    m = 8;
                elseif k == 4;
                    m = 1;
                elseif k == 5;
                    m = 5;
                elseif k == 6;
                    m = 2;
                elseif k == 7;
                    m = 6;
                elseif k == 8;
                    m = 7;
                end;
                
                v1 = cparm(m, 1); % save old parameter to v1;
                incr = intervals(intindex, m); % increment of parameter value change;
                v0 = v1 - incr; % new parameter value; move to one direction;
                cparm(m, 1) = v0; % replace with new parameter value;
                
                % input new parameter into f function to generate new q statistic;
                sw = 0;
                q0 = f(sw, cparm, gv, oridata, arglist);
                fprintf('\n %9.3f', q0);
                
                newincr = incr; % increment of parameter value change in next round of calculation;
                
                if q0 < q1; % new q statistic is smaller; % correct experimenting direction;
                    while q0 < q1; % keep experimenting in the same direction to get the smallest q statistic;
                        
                        % save boundary;
                        v2 = v1;
                        q2 = q1;
                        mvec2 = mvec1;
                        v1 = v0;
                        q1 = q0;
                        mvec1 = mvec0;
                        
                        v0 = v1 - newincr; % new parameter value; move in the same direction;
                        cparm(m, 1) = v0; % replace with new parameter value;
                        newincr = 2 * newincr; % increment of parameter value change in next round of calculation;
                        
                        % input new parameter into f function to generate new q statistic;
                        sw = 0;
                        q0 = f(sw, cparm, gv, oridata, arglist);
                        fprintf('\n %9.3f', q0);
                    end;
                else % new q statistic is larger;
                    
                    % keep experimenting, but move to the other direction;
                    v2 = v1 + newincr; % new parameter value; move to the other direction;
                    cparm(m, 1) = v2; % replace with new paramter value;
                    
                    % input new parameters into f function to generate new q statistic;
                    sw = 0;
                    q2 = f(mvec2, winv, nobs, 0);
                    fprintf('\n %9.3f', q2);
                    
                    while q2 <= q1; % new q statistic is smaller; % correct experimenting direction;
                        
                        % save boundary;
                        v0 = v1;
                        q0 = q1;
                        mvec0 = mvec1;
                        v1 = v2;
                        q1 = q2;
                        mvec1 = mvec2;
                        
                        v2 = v1 + newincr; % new parameter value; move in the same direction;
                        cparm(m, 1) = v2; % replace with new parameter value;
                        newincr = 2 * newincr; % increment of parameter value change in next round of calculation;
                        
                        % input new parameter into f function to generate new q statistic;
                        sw = 0;
                        q2 = f(sw, cparm, gv, oridata, arglist);
                        fprintf('\n %9.3f', q2);
                    end;
                end;
                
                % aftering finding a range, keep doing to find more accurate parameter value that minimizes q statistic;
                while v1 - v0 > 1.01 * incr || v2 - v1 > 1.01 * incr; % tolerance level;
                    
                    dv0 = v1 - v0;
                    dq0 = q0 - q1;
                    dv02 = v1 * v1 - v0 * v0;
                    dv1 = v2 - v1;
                    dq1 = q2 - q1;
                    dv12 = v2 * v2 - v1 * v1;
                    v = (dv12 * dq0 + dv02 * dq1) / (2 * (dv1 * dq0 + dv0 * dq1));
                    
                    if v2 - v1 <= 1.01 * incr || (v < v1 && v1 - v0 > 1.01 * incr);
                        
                        v = (v0 + v1) / 2;
                        cparm(m, 1) = v;
                        
                        sw = 0;
                        q = f(sw, cparm, gv, oridata, arglist);
                        fprintf('\n %9.3f', q);
                        
                        if q < q1;
                            v2 = v1;
                            q2 = q1;
                            mvec2 = mvec1; 
                            v1 = v;
                            q1 = q;
                            mvec1 = mvec;
                        else
                            v0 = v;
                            q0 = q;
                            mvec0 = mvec;
                        end;
                        
                    else
                        
                        v = (v1 + v2) / 2;
                        cparm(m, 1) = v;
                        
                        sw = 0;
                        q = f(sw, cparm, gv, oridata, arglist);
                        fprintf('\n %9.3f', q);
                        
                        if q < q1;
                            v0 = v1;
                            q0 = q1;
                            mvec0 = mvec1;
                            v1 = v;
                            q1 = q;
                            mvec1 = mvec;
                        else
                            v2 = v;
                            q2 = q;
                            mvec2 = mvec;
                        end;
                        
                    end;
                    
                end;
                 
                % save the last parameter value;
                cparm(m, 1) = v1;
                
                % calculate dmat;
                for l = 1 : gv.nmoments
                    dmat(gv.nmoments * m + l, 1) = (mvec2(l, 1) - mvec0(l, 1)) / (2 * incr);
                end;

            end;
            
            % check whether all parameter values are the same, i.e. the current set of parameter values gets the minimum q statistic;
            % if not, keep running the loop;
            sameparms = 1;
            for k = 1 : gv.nparms;
                if cparm(k, 1) ~= lparm(k, 1);
                    sameparms = 0;
                end;
            end;
            
        end;
    end;
end;

% calculate variance matrix of estimated parameters;
vari = covari(winv, dmat);


%%%%%%%%%%%%%%%%%%
% output results %
%%%%%%%%%%%%%%%%%%
% print estimated parameters, it's standard error and t statistic;
fprintf('\n\n          coeff          s.e.           t');
k = 0;
for i = 1 : gv.nparms;
    k = k + i;
    se = sqrt(vari(k, 1));
    t = cparm(i, 1) / se;
    fprintf('\n%3i%14.6f%14.6f%12.4f', i, cparm(i, 1), se, t);
end;
fprintf('\n\nnobs: %8d', gv.nobs); % number of observations;
fprintf('\nq: %11.3f', q1); % q statistic;

