% ----------------------------------------------------------------------- %
% % sscalc function                                                     % %
% % calculate Social Security grids and benefit amounts                 % %
% ----------------------------------------------------------------------- %

function [calc] = sscalc(gv, data, calc)

% policy 0: apply actual Social Security rules;
% policy 1: apply Social Security rules of a 6 years younger cohort;
% policy 2: apply Social Security rules of a 12 years younger cohort;
if gv.policy == 0;
    advance = 0;
elseif gv.policy == 1;
    advance = 6;
elseif gv.policy == 2;
    advance = 12;
end;

% initilizatoin;
ssbenf = zeros(gv.nscatlim, 1); % husband's SS own benefits as if the husband retires at scat ages (50, EEA, NRA, 70);
ssbenm = zeros(45, 2); % SS benefits if working in career job by age (25 - 69) and survival status (1 - 3);
ssbenr = zeros(50, 3, gv.nscatlim); % SS benefits if fully retired by age (50 - 99), survival status (1 - 3) and retirement (scat) ages (1 - gv.nscatlim);
ssbens = zeros(20, 2, gv.nscatlim); % SS benefits if working in returned full time work by age (50 - 69), survival status (1 - 2) and retirement (scat) ages (1 - gv.nscatlim);
ssbenp = zeros(20, 2, gv.nscatlim); % SS benefits if working part time by age (50 - 69), survival status (1 - 2) and retirement (scat) ages (1 - gv.nscatlim);
benchanges = zeros(8, 2, gv.nscatlim); % SS benefits adjustments if working in returned full time work by age (62 - 69), surv status (1 - 2) and retirement (scat) ages (1 - gv.nscatlim);
benchangep = zeros(8, 2, gv.nscatlim); % SS benefits adjustments if working part time by age (62 - 69), surv status (1 - 2) and retirement (scat) ages (1 - gv.nscatlim);
initialscat = zeros(21, 1); % initial scat grid point if retires at ages 50 - 70;
fbenefit = zeros(21, 1); % full SS own benefits as if the husband retires at ages 50 - 70;
adjownv = zeros(4, 1); % husband's own benefit factor (percent of PIA) as if he retires at 50, sserage, NRA, 70;
adjspv = zeros(4, 1); % husband's spousal benefit factor (percent of half of wife's PIA) as if he retires at 50, sserage, NRA, 70;
spadjownv = zeros(4, 1); % wife's own benefit factor (percent of PIA) as if he retires at 50, sserage, NRA, 70;
spadjspv = zeros(4, 1); % wife's spousal benefit factor (percent of half of husband's PIA) as if he retires at 50, sserage, NRA, 70;
adjownm = zeros(21, 4); % husband's own benefit factor (percent of PIA) by age (50 - 70) and retirement age (50, sserage, NRA, 70);
adjspm = zeros(21, 4); % husband's spousal benefit factor (percent of half of wife's PIA) by age (50 - 70) and retirement age (50, sserage, NRA, 70);
spadjownm = zeros(21, 4); % wife's own benefit factor (percent of PIA) by age (50 - 70) and retirement age (50, sserage, NRA, 70);
spadjspm = zeros(21, 4); % wife's spousal benefit factor (percent of half of husband's PIA) by age (50 - 70) and retirement age (50, sserage, NRA, 70);

% Social Security early entitlement age (EEA) is fixed at 62. 
sserage = 62; % Social Security early entitlement age;

% find Social Security normal retirement age (NRA) based on individua's birth year;
% This program uses a simplified NRA; The full version is:
% The NRA of individuals born in and before 1937 is 65;
% The NRA of individuals born in 1938 is 65 and 2 months;
% The NRA of individuals born in 1939 is 65 and 4 months;
% The NRA of individuals born in 1940 is 65 and 6 months;
% The NRA of individuals born in 1941 is 65 and 8 months;
% The NRA of individuals born in 1942 is 65 and 10 months;
% The NRA of individuals born in 1943 - 54 is 66;
% The NRA of individuals born in 1955 is 66 and 2 months;
% The NRA of individuals born in 1956 is 66 and 4 months;
% The NRA of individuals born in 1957 is 66 and 6 months;
% The NRA of individuals born in 1958 is 66 and 8 months;
% The NRA of individuals born in 1959 is 66 and 10 months;
% The NRA of individuals born in 1960 and later is 67;

if data.birthyear <= 1939 - advance;
    nrage = 65; 
elseif data.birthyear > 1939 - advance && data.birthyear <= 1956 - advance;
    nrage = 66;
elseif data.birthyear > 1956 - advance;
    nrage = 67;
end;

% find delayed retirement credit (DRC) based on individua's birth year;
if data.birthyear <= 1924 - advance;
    delayedret = 0.03;
elseif data.birthyear > 1924 - advance && data.birthyear <= 1942 - advance;
    delayedret = 0.03 + 0.005 * ((data.birthyear - advance - 1923) / 2);
elseif data.birthyear > 1942 - advance;
    delayedret = 0.08;
end;

% find wife's NRA and DRC based on her birth year;
if data.spbirthyear > 0; % married couple;
    if data.spbirthyear <= 1939 - advance; 
        spnrage = 65;
    elseif data.spbirthyear > 1939 - advance && data.spbirthyear <= 1956 - advance;
        spnrage = 66;
    elseif data.spbirthyear > 1956 - advance;
        spnrage = 67;
    end;
    if data.spbirthyear <= 1924 - advance;
        spdelayedret = 0.03;
    elseif data.spbirthyear > 1924 - advance && data.spbirthyear <= 1942 - advance;
        spdelayedret = 0.03 + 0.005 * ((data.spbirthyear - advance - 1923) / 2);
    elseif data.spbirthyear > 1942 - advance;
        spdelayedret = 0.08;        
    end;
elseif data.spbirthyear == 0; % single;
    spnrage = 65;
    spdelayedret = 0.08;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate SS benefits if working in career job %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initial value of adjustment factor (as a percent of PIA) when the husband starts to receive own benefits at EEA; 
% if less than or equal to 3 years between claim age and NRA, benefits are reduced 20/3 percent each year; 
% if more than 3 years, each year starting from the 4th year reduces benefits by 5 percent;
% given the EEA is 62: if NRA is 65, adjown is 0.8; if NRA is 66, adjown is 0.75; if NRA is 67, adjown is 0.7;
% adjown will be updated after each age;
adjown = 1 - (0.2 / 3) * min(nrage - sserage, 3) - 0.05 * max(nrage - sserage - 3, 0); 
% initial value of adjustment factor (as a percent of half of spouse's PIA) when the husband starts to receive spousal benefits at EEA, given the wife has started receiving benefits; 
% if less than or equal to 3 years between claim age and NRA, benefits are reduced 25/3 percent each year; 
% if more than 3 years, each year starting from the 4th year reduces benefits by 5 percent;
% given the ERA is 62: if NRA is 65, adjsp is 0.75; if NRA is 66, adjsp is 0.7; if NRA is 67, adjsp is 0.65;
adjsp  = 1 - (0.25 / 3) * min(nrage - sserage, 3) - 0.05 * max(nrage - sserage - 3, 0); 
% initial value of adjustment factor (as a percent of PIA) when the wife starts to receive own benefits at EEA; 
spadjown = 1 - (0.2 / 3) * min(spnrage - sserage, 3) - 0.05 * max(spnrage - sserage - 3, 0);
% initial value of adjustment factor (as a percent of half of spouse's PIA) when the wife starts to receive spousal benefits at EEA, given the husband has started receiving benefits; 
spadjsp  = 1 - (0.25 / 3) * min(spnrage - sserage, 3) - 0.05 * max(spnrage - sserage - 3, 0);

% starting from 25 because the wife could be much older and eligible to receive SS;
% claim as soon as possible;
for age = 25 : 69; 
    
    year = age + data.birthyear; % year
    spage = year - data.spbirthyear; % wife's age;
    
    % setup boundary;
    if age <= 50;
        pia = data.pia(1,1); % pia;
        ncpens = data.ncpens(1,1); % non-covered pension;
    elseif age > 50 && age <= 69;
        pia = data.pia(age - 50 + 1, 1);
        ncpens = data.ncpens(age - 50 + 1, 1);
    end;

    if age >= 50;
        fbenefit(age - 50 + 1, 1) = adjown * pia; % full SS own benefits without earnings' test at the year of retirement;
    end;
    
    if age >= sserage || spage >= sserage; % husband is eligible for SS or wife is eligible for SS;
        
        % setup boundary;
        if spage <= 50;
            sppia = data.sppia(1, 1); % wife's pia;
            spncpens = data.spncpens(1, 1); % wife's non-covered pension;
        elseif spage > 50 && spage <= 69;
            sppia = data.sppia(spage - 50 + 1, 1);
            spncpens = data.spncpens(spage - 50 + 1, 1);
        elseif spage > 69;
            sppia = data.sppia(21, 1);
            spncpens = data.spncpens(21, 1);
        end;
        
        % calculate own benefit amounts for husband and wife;
        if age >= sserage; % husband is eligible for own benefits;
            ownben = adjown * pia; 
        elseif age < sserage; % not eligible;
            ownben = 0; 
        end;
        
        if spage >= sserage; % wife is eligible for own benefits;
            spownben = spadjown * sppia;
        elseif spage < sserage; % not eligible;
            spownben = 0; 
        end;
        
        % if both spouses are eligible for SS, do spousal benefit calculations; 
        % this is because spousal benefits are only available after claiming of the spouse;
        % it is on top of own benefits;
        % pay attention to non-covered pension;
        if age >= sserage && spage >= sserage; % both spouses are eligible for SS;
            benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0); % husband's spousal benefits;
            spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0); % wife's spousal benefits;
        else % not eligible;
            benefitsp = 0;
            spbenefitsp = 0 ;
        end;
        
        % social Security earnings' test;
        % before 2000 (http://www.ssa.gov/oact/cola/rteahistory.html):
        % withhold $1 in benefits for every $2 of earnings in excess of the lower exempt amount (applies to people younger than NRA);
        % withhold $1 in benefits for every $3 of earnings in excess of the higher exempt amount (applies to people between NRA (including) and 69 (including));
        % after 2000 (including):
        % withhold $1 in benefits for every $2 of earnings in excess of the lower exempt amount (applies to people younger than NRA). 
        % withhold $1 in benefits for every $3 of earnings in excess of the higher exempt amount (applies only to earnings in months prior to the month of reaching NRA). 
        % earnings in or after the month you reach NRA do not count toward the retirement test.
        % $7440 is 1992 earnings' test level for people under age 65;
        % $10200 is 1992 earnings' test level for people between age 65 - 69;
        % assume earnings' test threshholds increase by a real 1% each year;
        adjfactor = 1.01 ^ (year + advance - 1992); % adjustment factor for SS earnings' test thresholds;
        
        % husband's earnings' test;
        if age < nrage || (age < 70 && year < 2001); % why 2001? the revised earnings' test rules start in 2000; ???
            earntest = 1; % husband is subject to earnings' test;
            if age < nrage; % use lower threshold;
                earningstest = 7440 * adjfactor; % earnings' test lower threshold;
                rednrate = 0.5; % withhold $1 for every $2 excess earnings;
            else % use higher threshold;
                earningstest = 10200 * adjfactor; % earnings' test higher threshold;
                rednrate = 0.333333; % withhold $1 for every $3 excess earnings;
            end;
        else
            earntest = 0; % not subject to earnings' test;
        end;
        
        % wife's earnings' test;
        if spage < spnrage || (spage < 70 && year < 2001); % why 2001? the revised earnings' test rules start in 2000; ???
            spearntest = 1; % wife is subject to earnings' test;
            if spage < spnrage; % use lower threshold;
                spearningstest = 7440 * adjfactor; % earnings' test lower threshold;
                sprednrate = 0.5; % withhold $1 for every $2 excess earnings;
            else % use higher threshold;
                spearningstest = 10200 * adjfactor; % earnings' test higher threshold;
                sprednrate = 0.333333; % withhold $1 for every $3 excess earnings;
            end;
        else
            spearntest = 0; % not subject to earnings' test;
        end;
        
        % find out husband's earnings and wife's earnings at the corresponding year;
        earnings = data.ftwage(age - 25 + 1, 1); % husband's earning;
        if spage >= 25 && spage < 70; % wife's earning;
            spearnings = data.spwage(spage - 25 + 1, 1);
        else
            spearnings = 0;
        end;
        
        % both spouses alive (vcat == 1);
        % find out how much benefits are withhold for husband's own benefit;
        if earntest == 1; % husband is subject to earnings' test;
            if ownben + spbenefitsp > 0;
                pctlost = rednrate * (earnings - earningstest) / (ownben + spbenefitsp);
                
                % setup boundary;
                if pctlost < 0;
                    pctlost = 0;
                elseif pctlost > 1;
                    pctlost = 1;
                end;
            else
                pctlost = 1;
            end;
        else
            pctlost = 0;
        end;
        
        % find out how much benefits are withhold for wife's own benefit;
        if spearntest == 1; % wife is subject to earnings' test;
            if spownben + benefitsp > 0 
                sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);
                
                % setup boundary;
                if sppctlost < 0;
                    sppctlost = 0;
                elseif sppctlost > 1;
                    sppctlost = 1;
                end;
            else
                sppctlost = 1;
            end;
        else
            sppctlost = 0;
        end;
        
        % find out how much benefits are withhold for husband's spousal benefit;
        % wife is not eligible for spousal benefits;  
        if benefitsp > 0; % husband is eligible for spousal benefits;
            if pctlost > sppctlost; 
                sppctlost = sppctlost - (pctlost - sppctlost) * benefitsp / spownben;  % update withhold percent of wife's own benefits;
                pctlostsp = pctlost;
                if sppctlost < 0;
                    sppctlost = 0;
                end;
            else
                pctlostsp = sppctlost;
            end;
        else
            pctlostsp = 1;
        end;
        
        
        % find out how much benefits are withhold for wife's spousal benefit;
        % husband is not eligible for spousal benefits;        
        if spbenefitsp > 0; % wife is eligible for spousal benefits;
            if pctlost < sppctlost;
                pctlost = pctlost - (sppctlost - pctlost) * spbenefitsp / ownben;  % update withhold percent of husband's own benefits;
                sppctlostsp = sppctlost;
                if pctlost < 0;
                    pctlost = 0;
                end;
            else
                sppctlostsp = pctlost;
            end;
        else
            sppctlostsp = 1;
        end;
        
        % sum up all four kinds of benefits received; 
        % spousal benefits could be 0;
        % earnings' test withhold applies where appliable;
        % save benefit amounts to corresponding SS benefit matrix; 
        ssbenm(age -25 + 1, 1) = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp + (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;
        
        
        % only husband survives (vcat = 2); 
        % wife only case is recorded in ssbenr (death of husband is considered as he retires at death);
        if age >= sserage; % husband is eligible for SS;
            
            % he receives the larger of own benefits or survivor's benefits;
            % survivor's benefits are no less than 82.5% of spouse's PIA, if claimed after EEA;
            % but he can claim survivor's benefits between 60 and 62 if she is older and dies in the range or earlier; 
            % in this case, the floor is 71.5%. ???
            survben = max(ownben, sppia * max(0.825, spadjown)); 
            if survben > 0; 
                
                % earnings' test;
                if earntest == 1; % husband is subject to earnings' test;
                    
                    % find out how much benefits are withhold after she dies;
                    survpctlost = rednrate * (earnings - earningstest) / survben;
                    
                    % setup boundary;
                    if survpctlost < 0;
                        survpctlost = 0;
                    elseif survpctlost > 1
                        survpctlost = 1;
                    end;
                else
                    survpctlost = 0;
                end;
                
                % benefit received after she dies;
                ssben = (1 - survpctlost) * survben; 
            else
                ssben = 0;
            end;
        else
            ssben = 0;
        end;
        
        % save benefit amounts to corresponding SS benefit matrix; 
        ssbenm(age - 25 + 1, 2) = ssben;
        
        % update adjustment factors (percents of PIAs can be received) for husband;
        % percent withhold is distributed into later payments; actual rule: increase only happens after FRA; ???
        % if withhold all benefits, benefits increase as he delays claiming at a later ages; 
        if age >= sserage && age < 70;
            if age < nrage - 3; % EEA (including) to NRA - 3;  
                pctchange = 0.05 * pctlost; 
                pctchangesp = 0.05 * pctlostsp;
            elseif age >= nrage - 3 && age < nrage; % 3 years before NRA to NRA (not including);
                pctchange = (0.2 / 3) * pctlost;  
                pctchangesp = (0.25 / 3) * pctlostsp; 
            elseif age >= nrage; % older than NRA;
                pctchange = delayedret * pctlost;
                pctchangesp = 0; % no delayed retirement credit for spousal benefits;
            end;
        else
            pctchange = 0;
            pctchangesp = 0;
        end;
        
        % update adjustment factors (percents of PIAs can be received) for wife;
        % percent withhold is distributed into later payments; 
        if spage >= sserage && spage < 70;
            if spage < spnrage - 3;
                sppctchange = 0.05 * sppctlost;
                sppctchangesp = 0.05 * sppctlostsp;
            elseif spage < spnrage;
                sppctchange = (0.2 / 3) * sppctlost;
                sppctchangesp = (0.25 / 3) * sppctlostsp;
            elseif spage >= spnrage;
                sppctchange = spdelayedret * sppctlost;
                sppctchangesp = 0;
            end;
        else
            sppctchange = 0;
            sppctchangesp = 0;
        end;
        
        % add additional percents of PIAs;
        adjown = adjown + pctchange; 
        adjsp = adjsp + pctchangesp;
        spadjown = spadjown + sppctchange;
        spadjsp = spadjsp + sppctchangesp;
        
        if age == 69; % adjustment factor has been updated; so it is actually for age 70;
            fbenefit(70 - 50 + 1, 1) = adjown * data.pia(70 - 50 + 1, 1);
        end;
        
    else
        
        ssbenm(age - 25 + 1, 1) = 0;
        ssbenm(age - 25 + 1, 2) = 0;
        
    end;
    
    % collect percents of PIAs can be received at ages corresponding to scat categories;
    if age + 1 == 50;
        scati = 1;
    elseif age + 1 == sserage;
        scati = 2;
    elseif age + 1 == nrage;
        scati = 3;
    elseif age + 1 == 70;
        scati = 4;
    else
        scati = 0;
    end;
    if scati > 0;
        adjownv(scati, 1) = adjown;
        adjspv(scati, 1) = adjsp;
        spadjownv(scati, 1) = spadjown;
        spadjspv(scati, 1) = spadjsp;
    end;
    
end;

% husband's SS own benefits as if the husband retires at 50, EEA, NRA and 70;
ssbenf(1,1) = fbenefit(1, 1); % retires at 50;
ssbenf(2,1) = fbenefit(sserage - 50 + 1, 1); % retires at EEA;
ssbenf(3,1) = fbenefit(nrage - 50 + 1, 1); % retires at NRA;
ssbenf(4,1) = fbenefit(21, 1); % retires at 70;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate SS benefits if fully retired %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SS benefits as if he retires at 50, EEA, NRA, and 70;
for scat = 1 : gv.nscatlim; % retirement age category;
    if scat == 1;
        retage = 50;
    elseif scat == 2;
        retage = sserage;
    elseif scat == 3;
        retage = nrage;
    elseif scat == 4;
        retage = 70;
    end;
    
    pia = data.pia(retage - 50 + 1, 1); % pia;
    ncpens = data.ncpens(retage - 50 + 1, 1); % non-covered pension;

    for age = 50 : 99;
        year = age + data.birthyear; % year;
        spage = year - data.spbirthyear; % spouse age;
        
        % find out corresponding adjustment factors (percents of PIAs);
        if age < retage;
            adjown = adjownv(scat, 1) - (adjownm(retage - 50 + 1, scat - 1) - adjownm(age - 50 + 1, scat - 1)); % husband's own benefits;
            adjsp = adjspv(scat, 1) - (adjspm(retage - 50 + 1, scat - 1) - adjspm(age - 50 + 1, scat - 1)); % husband's spousal benefits;
            spadjown = spadjownv(scat, 1) - (spadjownm(retage - 50 + 1, scat - 1) - spadjownm(age - 50 + 1, scat - 1)); % wife's own benefits; 
            spadjsp = spadjspv(scat, 1) - (spadjspm(retage - 50 + 1, scat - 1) - spadjspm(age - 50 + 1, scat - 1)); % wife's spousal benefits;
        elseif age == retage;
            adjown = adjownv(scat, 1); % husband's own benefits;
            adjsp = adjspv(scat, 1); % husband's spousal benefits;
            spadjown = spadjownv(scat, 1); % wife's own benefits;
            spadjsp = spadjspv(scat, 1); % wife's spousal benefits;
        end;
        
        % update adjustment factors;
        if age <= 70;
            adjownm(age - 50 + 1, scat) = adjown; % husband's own benefits;
            adjspm(age - 50 + 1, scat) = adjsp; % husband's spousal benefits;
            spadjownm(age - 50 + 1, scat) = spadjown; % wife's own benefits;
            spadjspm(age - 50 + 1, scat) = spadjsp; % wife's spousal benefits;
        end;
        
        if age >= sserage || spage >= sserage; % husband is eligible for SS or wife is eligible for SS;
            
            % setup boundary;
            if spage < 50;
                sppia = data.sppia(1, 1); % spouse's pia;
                spncpens = data.spncpens(1, 1); % spouse's non-covered pensions;
            elseif spage >= 50 && spage < 70;
                sppia = data.sppia(spage - 50 + 1, 1);
                spncpens = data.spncpens(spage - 50 + 1, 1);
            elseif spage >= 70;
                sppia = data.sppia(21, 1);
                spncpens = data.spncpens(21, 1);
            end;
            
            % calculate own benefit amounts for husband and wife;
            if age >= sserage; % husband is eligible for own benefits;
                ownben = adjown * pia;
            elseif age < sserage; % not eligible;
                ownben = 0;
            end;
            if spage >= sserage; % wife is eligible for own benefits;
                spownben = spadjown * sppia;
            elseif spage < sserage; % not eligible;
                spownben = 0;
            end;
            
            % if both spouses are eligible for SS benefits, do spousal benefits calculations;
            if age >= sserage && spage >= sserage; % both spouses are eligible for SS;
                benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0); % husband's spousal benefits;
                spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0); % wife's spousal benefits;
            else % not eligible;
                benefitsp = 0;
                spbenefitsp = 0;
            end;
            
            % SS earnings' test;
            % husband is not subject to earnings' test because he has retired;
            % wife's earnings' test;
            adjfactor = 1.01 ^ (year + advance - 1992); % earings' test threshhold adjustment factor;
            if spage < spnrage || (spage < 70 && year < 2001);
                spearntest = 1; % wife is subject to earnings' test;
                if spage < spnrage; % use lower threshold;
                    spearningstest = 7440 * adjfactor; % earnings' test lower threshold;
                    sprednrate = 0.5; % withhold $1 for every $2 excess earnings;
                else % use higher threshold;
                    spearningstest = 10200 * adjfactor; % earnings' test higher threshold;
                    sprednrate = 0.333333; % withhold $1 for every $3 excess earnings;
                end;
            else
                spearntest = 0; % not subject to earnings' test;
            end;
            
            % find out wife's earnings at the corresponding year;
            if spage >= 25 && spage < 70;
                spearnings = data.spwage(spage-25 + 1, 1);
            else
                spearnings = 0;
            end;
            
            % both spouses alive (vcat = 1);
            % husband is not subject to earnings' test because he has retired;
            pctlost = 0; 
            
            % find out how much benefits are withhold for wife's own benefit;
            if spearntest == 1; % wife is subject to earnings' test;
                if spownben + benefitsp > 0;
                    sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);
                    if sppctlost < 0;
                        sppctlost = 0;
                    elseif sppctlost > 1
                        sppctlost = 1;
                    end;
                else
                    sppctlost = 1;
                end;
            else
                sppctlost = 0;
            end;
            
            % find out how much benefits are withhold for husband's spousal benefit;
            if benefitsp > 0; % husband is eligible for spousal benefits;
                pctlostsp = sppctlost;
            else
                pctlostsp = 1;
            end;
            
            % find out how much benefits are withhold for wife's spousal benefit;
            if spbenefitsp > 0; % wife is eligible for spousal benefits; 
                sppctlostsp = sppctlost;
            else
                sppctlostsp = 1;
            end;
        
            % sum up all four kinds of benefits received; 
            % save benefit amounts to corresponding SS benefit matrix; 
            ssbenr(age - 50 + 1, 1, scat) = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp...
                + (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;
            
            % only husband alive (vcat = 2);
            if age >= sserage; % husband is eligible for SS;
                % he receives the larger of own benefits or survivor's benefits;
                % no earnings' test because he has retired;
                survben = max(ownben, sppia * max(0.825, spadjown));
            elseif age < sserage;
                survben = 0;
            end;
            
            % save benefit amounts to corresponding SS benefits matrix; 
            ssbenr(age - 50 + 1, 2, scat) = survben;
            
            % only wife alive (vcat = 3);
            if spage >= sserage; % wife is eligible for SS;
                % she receives the larger of own benefits or survivor's benefits;
                spsurvben = max(spownben, pia * max(0.825, adjown));
                if spsurvben > 0;
                    if spearntest == 1; % the wife is subject to earnings' test;
                        spsurvpctlost = sprednrate * (spearnings - spearningstest) / spsurvben;
                        
                        % setup boundary;
                        if spsurvpctlost < 0;
                            spsurvpctlost = 0;
                        elseif spsurvpctlost > 1;
                            spsurvpctlost = 1;
                        end;
                    else
                        spsurvpctlost = 0;
                    end;
                    
                    % benefit received after he dies;
                    ssben = (1 - spsurvpctlost) * spsurvben;
                    
                else
                    ssben = 0;
                end;
                
            else
                ssben = 0;
            end;
            
            % save benefit amounts to corresponding SS benefit matrix; 
            ssbenr(age - 50 + 1, 3, scat) = ssben;
            
            % update adjustment factors (percents of PIAs) for husband;
            % percent withhold is distributed into later payments; 
            if age >= sserage && age < 70
                if age < nrage - 3; 
                    pctchangesp = 0.05 * pctlostsp;
                elseif age >= nrage - 3 && age < nrage;
                    pctchangesp = (0.25 / 3) * pctlostsp;
                elseif age >= nrage;
                    pctchangesp = 0;
                end;
            else
                pctchangesp = 0;
            end;
            
            % update adjustment factors (percents of PIAs) for wife;
            % percent withhold is distributed into later payments; 
            if spage >= sserage && spage < 70
                if spage < spnrage - 3;
                    sppctchange = 0.05 * sppctlost;
                    sppctchangesp = 0.05 * sppctlostsp;
                elseif spage < spnrage
                    sppctchange = (0.2 / 3) * sppctlost;
                    sppctchangesp = (0.25 / 3) * sppctlostsp;
                elseif spage >= spnrage;
                    sppctchange = spdelayedret * sppctlost;
                    sppctchangesp = 0;
                end;
            else
                sppctchange = 0;
                sppctchangesp = 0;
            end;
            
            % add additional percents of PIAs;
            adjsp = adjsp + pctchangesp;
            spadjown = spadjown + sppctchange;
            spadjsp = spadjsp  + sppctchangesp;
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate SS benefits and adjustments if working in part time or returned full time job %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for jcat = 1 : 2; % job category; 1 means returned full time work; 2 means part time work;
    for scat = 1: gv.nscatlim; % retirement age category;
        if scat == 1;
            retage = 50;
        elseif scat == 2;
            retage = sserage;
        elseif scat == 3;
            retage = nrage;
        elseif scat == 4;
            retage = 70;
        end;
        
        pia = data.pia(retage - 50 + 1, 1); % pia;
        ncpens = data.ncpens(retage - 50 + 1, 1); % non-covered pensions;
        
        for age = 50 : 69;
            year = age + data.birthyear; % year;
            spage = year - data.spbirthyear; % wife's age;
            
            % adjustment factors (percents of PIAs) given age and retirement age;
            adjown = adjownm(age - 50 + 1, scat); % husband's own benefits;
            adjsp = adjspm(age - 50 + 1, scat); % husband's spousal benefits;
            spadjown = spadjownm(age - 50 + 1, scat); % wife's own benefits;
            spadjsp = spadjspm(age - 50 + 1, scat); % wife's spousal benefits;
            
            if age >= sserage || spage >= sserage;
                
                % setup boundary;
                if spage < 50;
                    sppia = data.sppia(1, 1); % wife's pia;
                    spncpens = data.spncpens(1, 1); % wife's non-covered pension;
                elseif spage >=50 && spage < 70;
                    sppia = data.sppia(spage - 50 + 1, 1);
                    spncpens = data.spncpens(spage - 50 + 1, 1);
                elseif spage >= 70;
                    sppia = data.sppia(21, 1);
                    spncpens = data.spncpens(21, 1);
                end;
        
                % calculate own benefit amounts for husband and wife;
                if age >= sserage; % husband is eligible for own benefits;
                    ownben = adjown * pia;
                elseif age < sserage; % not eligible;
                    ownben = 0;
                end;
                if spage >= sserage; % wife is eligible for own benefits;
                    spownben = spadjown * sppia;
                elseif spage < sserage; % not eligible;
                    spownben = 0;
                end;
                
                % if both spouses are eligible for SS, do spousal benefit calculations; 
                if age >= sserage && spage >= sserage; % both spouses are eligible for SS;
                    benefitsp = max(adjsp * (0.5 * sppia - pia) - ncpens, 0); % husband's spousal benefits;
                    spbenefitsp = max(spadjsp * (0.5 * pia - sppia) - spncpens, 0); % wife's spousal benefits;
                else % not eligible;
                    benefitsp = 0;
                    spbenefitsp = 0;
                end;
                
                % SS earnings' test;
                adjfactor = 1.01 ^ (year + advance - 1992); % earings' test threshhold adjustment factor;  
                
                % husband's earnings' test;
                if age < nrage || (age < 70 && year < 2001);
                    earntest = 1; % husband is subject to earnings' test;
                    if age < nrage; % use lower threshold;
                        earningstest = 7440 * adjfactor; % earnings' test lower threshold;
                        rednrate = 0.5; % withhold $1 for every $2 excess earnings;
                    else % use higher threshold;
                        earningstest = 10200 * adjfactor; % earnings' test higher threshold;
                        rednrate = 0.333333; % withhold $1 for every $3 excess earnings;
                    end;
                else
                    earntest = 0;
                end;
                
                % wife's earnings' test;
                if spage < spnrage || (spage < 70 && year < 2001);
                    spearntest = 1; % wife is subject to earnings' test;
                    if spage < spnrage; % use lower threshold;
                        spearningstest = 7440 * adjfactor; % earnings' test lower threshold;
                        sprednrate = 0.5; % withhold $1 for every $2 excess earnings;
                    else % use higher threshold;
                        spearningstest = 10200 * adjfactor; % earnings' test higher threshold;
                        sprednrate = 0.333333; % withhold $1 for every $3 excess earnings;
                    end;
                else % not subject to earnings' test;
                    spearntest = 0;
                end;
                
                % find out husband's earnings at the corresponding year given job type;    
                if jcat == 1; % returned full time work;
                    earnings = data.secwage(age - 50 + 1, 1);
                elseif jcat == 2; % part time work;
                    earnings = data.prwage(age - 50 + 1, 1);
                end;
                
                % find out wife's earnings at the corresponding year;    
                if spage >= 25 && spage < 70;
                    spearnings = data.spwage(spage - 25 + 1, 1);
                else
                    spearnings = 0;
                end;
                
                % both spouses survive (vcat = 1);
                % find out how much benefits are withhold for husband's own benefit;      
                if earntest == 1
                    if ownben + spbenefitsp > 0;
                        pctlost = rednrate * (earnings - earningstest) / (ownben + spbenefitsp);
                        if pctlost < 0;
                            pctlost = 0;
                        elseif pctlost > 1;
                            pctlost = 1;
                        end;
                    else
                        pctlost = 1;
                    end;
                else
                    pctlost = 0;
                end;
                
                % find out how much benefits are withhold for wife's own benefit; 
                if spearntest == 1;
                    if spownben + benefitsp > 0;
                        sppctlost = sprednrate * (spearnings - spearningstest) / (spownben + benefitsp);
                        if sppctlost < 0;
                            sppctlost = 0;
                        elseif sppctlost > 1;
                            sppctlost = 1;
                        end;
                    else
                        sppctlost = 1;
                    end;
                else
                    sppctlost = 0;
                end;
                
                % find out how much benefits are withhold for husband's spousal benefit; 
                if benefitsp > 0;  % husband is eligible for spousal benefits;
                    if sppctlost < pctlost; 
                        sppctlost = sppctlost - (pctlost - sppctlost) * benefitsp / spownben; % update withhold percent of wife's own benefits;
                        pctlostsp = pctlost;
                        if sppctlost < 0;
                            sppctlost = 0;
                        end;
                    else
                        pctlostsp = sppctlost;
                    end;
                else
                    pctlostsp = 1;
                end;
                
                % find out how much benefits are withhold for wife's spousal benefit; 
                if spbenefitsp > 0; % wife is eligible for spousal benefits;
                    if pctlost < sppctlost % additional loss of spouse benefits;
                        pctlost = pctlost - (sppctlost - pctlost) * spbenefitsp / ownben; % update withhold percent of husband's own benefits;
                        sppctlostsp = sppctlost;
                        if pctlost < 0;
                            pctlost = 0;
                        end;
                    else
                        sppctlostsp = pctlost;
                    end;
                else
                    sppctlostsp = 1;
                end;          
                
                % sum up all four kinds of benefits received; 
                ssben = (1 - pctlost) * ownben + (1 - pctlostsp) * benefitsp + ...
                    (1 - sppctlost) * spownben + (1 - sppctlostsp) * spbenefitsp;

                % save benefit amounts to corresponding SS benefit matrix; 
                if jcat == 1; % returned full time work;
                    ssbens(age - 50 + 1, 1, scat) = ssben;
                elseif jcat == 2; % part time work;
                    ssbenp(age - 50 + 1, 1, scat) = ssben;
                end;
                    
                % only husband alive (vcat = 2);                
                % wife only case is recorded in ssbenr (death of husband is considered as he retires at death);
                if age >= sserage; % husband is eligible for SS;
                    
                    % he receives the larger of own benefits or survivor's benefits;
                    survben = max(ownben, sppia * max(0.825, spadjown));
                    if survben > 0;
                        
                        % earnings' test;
                        if earntest == 1; % husband is subject to earnings' test;
                            
                            % find out how much benefits are withhold after she dies;
                            survpctlost = rednrate * (earnings - earningstest) / survben;
                            
                            % setup boundary;
                            if survpctlost < 0;
                                survpctlost = 0;
                            elseif survpctlost > 1;
                                survpctlost = 1;
                            end;
                        else
                            survpctlost = 0;
                        end;
                    else
                        survpctlost = 1;
                    end;
                else
                    survpctlost = 0;
                end;
                
                % benefit received after she dies;
                ssben = (1 - survpctlost) * survben;
                
                % save benefit amounts to corresponding SS benefit matrix; 
                if jcat == 1; % returned full time work;
                    ssbens(age - 50 + 1, 2, scat) = ssben;
                elseif jcat == 2; % part time work;
                    ssbenp(age - 50 + 1, 2, scat) = ssben;
                end
                
                % calculate benefit amount changes to husband's own benefits;
                % percent withhold is distributed into later payments; 
                % both spouses alive (vcat == 1);
                if age >= sserage;
                    if age >= sserage && age < 70;
                        if age < nrage - 3;
                            pctchange = 0.05 * pctlost;
                        elseif age < nrage;
                                pctchange = (0.2 / 3) * pctlost;
                        elseif age >= nrage;
                            pctchange = delayedret * pctlost;
                        end;
                    else
                        pctchange = 0;
                    end;
                    
                    % calculate benefit amount changes;
                    benchange = pctchange * ownben;
                    
                    % save benefit amount changes to corresponding SS benefit matrix; 
                    if jcat == 1; % returned full time work;
                        benchanges(age - 62 + 1, 1, scat) = benchange;
                    elseif jcat == 2; % part time work;
                        benchangep(age - 62 + 1, 1, scat) = benchange;
                    end;
                    
                    % only husband alive (vcat == 2);
                    if age >= sserage && age < 70;
                        if age < nrage - 3;
                            pctchange = 0.05 * survpctlost;
                        elseif age < nrage;
                            pctchange = (0.2 / 3) * survpctlost;
                        elseif age >= nrage;
                            pctchange = delayedret * survpctlost;
                        end;
                    else
                        pctchange = 0;
                    end;
                    
                    % calculate benefit amount changes;
                    benchange = pctchange * ownben;
                    
                    % save benefit amount changes to corresponding SS benefit matrix; 
                    if jcat == 1; % returned full time work;
                        benchanges(age - 62 + 1, 2, scat) = benchange;
                    elseif jcat == 2; % part time work;
                        benchangep(age - 62 + 1, 2, scat) = benchange;
                    end;
                end;
            end;
        end;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate initial scat grid point for each age of leaving main job %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for age = 50 : 70; % retirement age;
    if age < sserage;
        scat = 1; % retirement age 50;
        firstage = 50;
        lastage = sserage;
    elseif age < nrage;
        scat = 2; % retirement age EEA;
        firstage = sserage;
        lastage = nrage;
    elseif age >= nrage;
        scat = 3; % retirement age NRA;
        firstage = nrage;
        lastage = 70;
    end;
    
    % find out corresponding scat grind point; 
    % scat is 1 if retires at 50; scat is 2 if retires at EEA; scat is 3 if retires at NRA; scat is 4 if retires at 70; 
    % scat is in between if retires at other age;
    % interpolation;
    if fbenefit(lastage - 50 + 1, 1) - fbenefit(firstage - 50 + 1, 1) > 0;
        initialscat(age - 50 + 1, 1) = scat + (fbenefit(age - 50 + 1, 1) - fbenefit(firstage - 50 + 1, 1))/ (fbenefit(lastage - 50 + 1, 1) - fbenefit(firstage - 50 + 1, 1));
        if initialscat(age - 50 + 1, 1) < scat;
            initialscat(age - 50 + 1, 1) = scat;
        elseif initialscat(age - 50 + 1, 1) > scat + 1
            if scat < 2;
                initialscat(age - 50 + 1, 1) = scat + 1;
            elseif scat >= 2;
                initialscat(age - 50 + 1, 1) = scat + 0.999;
            end;
        end;
    else
        initialscat(age - 50 + 1, 1) = scat;
    end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% assign the values to structure variable calc for future calculations %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
calc.sserage = sserage;
calc.nrage = nrage;
calc.spnrage = spnrage;
calc.delayedret = delayedret;
calc.spdelayedret = spdelayedret;
calc.initialscat = initialscat;
calc.ssbenm = ssbenm;
calc.ssbenr = ssbenr;
calc.ssbens = ssbens;
calc.ssbenp = ssbenp;
calc.benchanges = benchanges;
calc.benchangep = benchangep;
calc.ssbenf = ssbenf;




