% ----------------------------------------------------------------------- %
% % test function                                                       % %
% % run backward induction for a hypothetical household to test codes   % % 
% ----------------------------------------------------------------------- %

% run the test mode with the initially assigned parameter values;
function test(gv, oridata)

% print current set of parameter values;
cparm(1, 1) = gv.parm.sige;
cparm(2, 1) = gv.parm.erho;
cparm(3, 1) = gv.parm.beta0;
cparm(4, 1) = gv.parm.beta1;
cparm(5, 1) = gv.parm.beta2;
cparm(6, 1) = gv.parm.gamma0;
cparm(7, 1) = gv.parm.gamma1;
cparm(8, 1) = gv.parm.alpha;
fprintf('\n current parametes:');
fprintf('\n sige: %6.3f', cparm(1, 1));
fprintf('\n erho: %6.3f', cparm(2, 1));
fprintf('\n beta0: %6.3f', cparm(3, 1));
fprintf('\n beta1: %6.3f', cparm(4, 1));
fprintf('\n beta2: %6.3f', cparm(5, 1));
fprintf('\n gamma0: %6.3f', cparm(6, 1));
fprintf('\n gamma1: %6.3f', cparm(7, 1));
fprintf('\n alpha: %6.3f', cparm(8, 1));
fprintf('\n');

% arglist.newalpha is an indicator whether alpha, the consumption parameter, has been changed; 
% if it is 1, re-calculate rho, the time discount factor;
arglist.newalpha = 1; % indicator of new value of consumption parameter alpha;


%%%%%%%%%%%%%%%%%%%
% calculate model %
%%%%%%%%%%%%%%%%%%%
% calculate each household's optimal work (retirement) decisions;
arglist = indivmodel(gv, oridata, arglist, cparm);


%%%%%%%%%%%%%%%%%%
% output results %
%%%%%%%%%%%%%%%%%%
% initialization;
% ret0mat: percent of simulated workers who retired from career job; or are 1) previously retired, currently in full time job; 2) currently in part time work; and 3) currently retired, by age (50 - 70);
% ret1mat: percent of simulated workers who are not currently working full time; or are 1) currently in part time work; and 2) currently retired, by age (50 - 70), gv.nobs X 21;
% ret2mat: percent of simulated workers who are currently retired, by age (50 - 70), gv.nobs X 21;
% returnft: percent of simulated workers who are previously retired or in part time work, newly return to full time work, by age (50 - 70), gv.nobs X 21;
% returnpt: percent of simulated workers who are previously retired, newly return to part time work, by age (50 - 70), gv.nobs X 21;
% newpt: percent of simulated workers who are newly enter part time work, by age (50 - 70), gv.nobs X 21;
% everreturnft: percent of simulated workers ever return to full time work after full or partial retirement, gv.nobs X 1;
% everreturnpt: percent of simulated workers ever return to part time work after full retirement, gv.nobs X 1;
% everreturn: percent of simulated workers ever return to full time work after full or partial retirement, or return to part time work after full retirement, gv.nobs X 1;
ret0mat = zeros(21, 1);
ret1mat = zeros(21, 1);
ret2mat = zeros(21, 1);
returnft = zeros(21, 1);
returnpt = zeros(21, 1);
newpt = zeros(21, 1);

% retrieve retirement moments from structure variable arglist;
% there is only one household in test mode, average over all households in iteration and simulation modes;
for age = 50 : 69;
    ret0mat(age - 50 + 1, 1) = 100 * arglist.ret0mat(1, age - 50 + 1);
    ret1mat(age - 50 + 1, 1) = 100 * arglist.ret1mat(1, age - 50 + 1);
    ret2mat(age - 50 + 1, 1) = 100 * arglist.ret2mat(1, age - 50 + 1);
    returnft(age - 50 + 1, 1) = 100 * arglist.returnft(1, age - 50 + 1);
    returnpt(age - 50 + 1, 1) = 100 * arglist.returnpt(1, age - 50 + 1);
    newpt(age - 50 + 1, 1) = 100 * arglist.newpt(1, age - 50 + 1);
end;
everreturnft = 100 * arglist.everreturnft;
everreturnpt = 100 * arglist.everreturnpt;
everreturn   = 100 * arglist.everreturn;

% print tabulations;
% ret1(age) - ret1(age - 1): increased percent of simulated workers who are not in full time job; reversal jobs may be invovled;
% ret2(age) - ret2(age - 1): increased percent of simulated workers who are retired; reversal jobs may be invovled;
% 100 - ret0(age): percent of simulated workers who are in career job;
% ret0(age) - ret1(age): percent of simulated workers who previously quit career job but are currently working full time;
% ret1(age) - ret2(age): percent of simulated workers who are in part time job;
fprintf('\n                Percent                   Percent');
fprintf('\n            Pseudo-Retiring               Retired');
fprintf('\n            From                          From');
fprintf('\n Age        FT Work  Completely            FT Work  Completely');
fprintf('\n');
fprintf('\n %2d                 %5.1f                          %5.1f', 50, ret1mat(1, 1), ret2mat(1, 1));
for age = 51 : 69;
    fprintf('\n %2d        %5.1f    %5.1f                %5.1f     %5.1f', ...
        age, ...
        ret1mat(age - 50 + 1, 1) - ret1mat((age - 1) - 50 + 1, 1), ret2mat(age - 50 + 1, 1) - ret2mat((age - 1) - 50 + 1, 1), ...
        ret1mat(age - 50 + 1, 1), ret2mat(age - 50 + 1, 1));
end;

fprintf('\n\n                       Percent    Percent                          Percent Newly');
fprintf('\n                         in        Newly                   Percent   Returned');
fprintf('\n        Percent        FT Work    Returned       Percent    Newly   to PT Work');
fprintf('\n Age    in Main         after       to             in        in     Previously');
fprintf('\n          Job          Retiring   FT Work        PT Work   PT Work   Retired');
fprintf('\n');
fprintf('\n %2d    %5.1f                                    %5.1f',...
    50, 100 - ret0mat(1, 1), ret1mat(1, 1) - ret2mat(1, 1));
for age = 51 : 69;
    fprintf('\n %2d    %5.1f          %5.1f      %5.1f          %5.1f     %5.1f     %5.1f', ...
        age,...
        100 - ret0mat(age - 50 + 1, 1), ret0mat(age - 50 + 1, 1) - ret1mat(age - 50 + 1, 1), returnft(age - 50 + 1, 1), ...
        ret1mat(age - 50 + 1, 1) - ret2mat(age - 50 + 1, 1), newpt(age - 50 + 1, 1), returnpt(age - 50 + 1, 1));
end;
fprintf('\n\n Percent returning to full time work after full or partial retirement: %5.1f', everreturnft);
fprintf('\n Percent returning to part time work after full retirement: %5.1f', everreturnpt);
fprintf('\n Percent returning to full time work after full or partial retirement');
fprintf('\n   or returning to part time work after full retirement: %5.1f', everreturn);
fprintf('\n');



















